@@ -2,6 +2,9 @@
 /* whether or not --enable-mini-stl is used */
 #undef HAVE_MINI_STL
 
+/* whether CORBASec Level 2 is used */
+#undef USE_CSL2
+
 /* whether we have byteorder big endian */
 #undef HAVE_BYTEORDER_BE
 
@@ -62,6 +65,9 @@
 /* whether or not he have SSLeay */
 #undef HAVE_SSL
 
+/* whether or not he have Postgresql */
+#undef HAVE_PGSQL
+
 /* whether or not bool is supported by C++ compiler */
 #undef HAVE_BOOL_TYPE
 
@@ -96,6 +102,18 @@
 /* whether a derived class can override a typedef in the base class */
 #undef HAVE_TYPEDEF_OVERLOAD
 
+/* Define if you have pthread support */
+#undef HAVE_PTHREADS
+
+/* Define if you have DCE thread support */
+#undef HAVE_DCE_THREADS
+
+/* Define if you have Solaris thread support */
+#undef HAVE_SOLARIS_THREADS
+
+/* Define if you want thread support at all */
+#undef HAVE_THREADS
+
 /* whether to build a Minimum CORBA ORB */
 #undef HAVE_MINIMUM_CORBA
 
@@ -108,3 +126,6 @@
 /* path information */
 #undef ABSEXECDIR
 #undef ABSSHRDDIR
+
+/* whether ANSI C++ header files are present */
+#undef HAVE_ANSI_CPLUSPLUS_HEADERS
