@@ -23,10 +23,18 @@
 #define MICO_CONF_IMR
 #define MICO_CONF_IR
 
-#include <CORBA-SMALL.h>
+#include <CORBA.h>
 #include "CosEvent_impl.h"
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <iostream>
+#include <fstream>
+#else
 #include <iostream.h>
 #include <fstream.h>
+#endif
+
+
+using namespace std;
 
 //------------------------------------------------------------------------
 
@@ -54,8 +62,15 @@ void ProxyPushConsumer_impl::push (const CORBA::Any& data)
 
 void ProxyPushConsumer_impl::disconnect_push_consumer ()
 {
-    if (!CORBA::is_nil (supplier) && !supplier->_non_existent()) {
+    if (!CORBA::is_nil (supplier)) {
+#ifdef HAVE_EXCEPTIONS
+      try {
+#endif
 	supplier->disconnect_push_supplier();
+#ifdef HAVE_EXCEPTIONS
+      } catch (...) {
+      }
+#endif
     }
     supplier = CosEventComm::PushSupplier::_nil();
     connected = FALSE;
@@ -71,7 +86,9 @@ ProxyPushSupplier_impl::ProxyPushSupplier_impl (EventChannel_impl *impl)
 void ProxyPushSupplier_impl::connect_push_consumer (
     CosEventComm::PushConsumer_ptr push_consumer)
 {
-    if (!CORBA::is_nil (consumer) && !CORBA::is_nil (push_consumer))
+    if (CORBA::is_nil(push_consumer))
+        return;
+    if (!CORBA::is_nil (consumer))
         mico_throw (CosEventChannelAdmin::AlreadyConnected ());
     consumer = CosEventComm::PushConsumer::_duplicate (push_consumer);
     channel->listen (TRUE);
@@ -79,8 +96,15 @@ void ProxyPushSupplier_impl::connect_push_consumer (
 
 void ProxyPushSupplier_impl::disconnect_push_supplier ()
 {
-    if (!CORBA::is_nil (consumer) && !consumer->_non_existent()) {
+    if (!CORBA::is_nil (consumer)) {
+#ifdef HAVE_EXCEPTIONS
+      try {
+#endif
         consumer->disconnect_push_consumer();
+#ifdef HAVE_EXCEPTIONS
+      } catch (...) {
+      }
+#endif
     }
     int was_connected = !CORBA::is_nil (consumer);
     consumer = CosEventComm::PushConsumer::_nil();
@@ -90,6 +114,7 @@ void ProxyPushSupplier_impl::disconnect_push_supplier ()
 
 void ProxyPushSupplier_impl::notify (const CORBA::Any &any)
 {
+    MICOMT::AutoLock lock(requests);
     if (CORBA::is_nil (consumer) ||
         requests.size() >= channel->max_queue_size())
         return;
@@ -105,7 +130,7 @@ ProxyPushSupplier_impl::callback (CORBA::Request_ptr req2,
                                   CORBA::RequestCallback::Event ev)
 {
     assert (ev == CORBA::RequestCallback::RequestDone);
-
+    MICOMT::AutoLock lock(requests);
     list<CORBA::Request_var>::iterator j = requests.begin();
     for (; j != requests.end(); ++j) {
         CORBA::Request_ptr req = (*j);
@@ -124,13 +149,11 @@ ProxyPushSupplier_impl::callback (CORBA::Request_ptr req2,
 
 //------------------------------------------------------------------------
 
-PullSupplier_skel2::PullSupplier_skel2 (const CORBA::BOA::ReferenceData &_id)
-    : MICO_SCOPE(CosEventComm,PullSupplier_skel) (_id)
+PullSupplier_skel2::PullSupplier_skel2 ()
 {
 }
 
 PullSupplier_skel2::PullSupplier_skel2 (CORBA::Object_ptr _obj)
-    : MICO_SCOPE(CosEventComm,PullSupplier_skel) (_obj)
 {
 }
 
@@ -138,15 +161,24 @@ PullSupplier_skel2::~PullSupplier_skel2()
 {
 }
 
-bool
-PullSupplier_skel2::dispatch (CORBA::StaticServerRequest_ptr _req,
-                              CORBA::Environment &_env)
+void
+PullSupplier_skel2::invoke(CORBA::StaticServerRequest_ptr _req)
 {
   if( strcmp( _req->op_name(), "pull" ) == 0 ) {
-    pull (_req);
-    return true;
+#ifdef HAVE_EXCEPTIONS
+    try {
+#endif
+      pull (_req);
+#ifdef HAVE_EXCEPTIONS
+  } catch (CosEventComm::Disconnected &_ex) {
+    _req->set_exception(_ex->_clone());
+    _req->write_results ();
+    return;
   }
-  return MICO_SCOPE(CosEventComm,PullSupplier_skel)::dispatch (_req, _env);
+#endif
+    return;
+  }
+  return POA_CosEventComm::PullSupplier::invoke(_req);
 }
 
 //------------------------------------------------------------------------
@@ -161,10 +193,11 @@ void
 ProxyPullSupplier_impl::connect_pull_consumer (
     CosEventComm::PullConsumer_ptr pull_consumer)
 {
-    if (!CORBA::is_nil (consumer) && !CORBA::is_nil (pull_consumer))
+    if (!CORBA::is_nil (consumer))
         mico_throw (CosEventChannelAdmin::AlreadyConnected());
 
     consumer = CosEventComm::PullConsumer::_duplicate (pull_consumer);
+    connected = TRUE;
 }
 
 void
@@ -174,21 +207,52 @@ ProxyPullSupplier_impl::disconnect_pull_supplier ()
 	consumer->disconnect_pull_consumer();
     }
     consumer = CosEventComm::PullConsumer::_nil();
-    connected = FALSE;
+    if (connected == TRUE) {
+	MICOMT::AutoLock r_lock(requests);
+	MICOMT::AutoLock e_lock(events);
+
+        connected = FALSE;
+
+	if (requests.size() > 0)
+	  channel->listen(FALSE);
+
+	list<CORBA::StaticServerRequest_var>::iterator i = requests.begin();
+        for (; i!=requests.end(); ++i) {
+	  (*i)->set_exception(new CosEventComm::Disconnected());
+	  (*i)->write_results ();
+	}
+
+	requests.clear();
+	events.clear();
+    }
 }
 
+
+void
+ProxyPullSupplier_impl::invoke(CORBA::StaticServerRequest_ptr req)
+{
+    return PullSupplier_skel2::invoke(req);
+}
+
+
 void
 ProxyPullSupplier_impl::pull (CORBA::StaticServerRequest_ptr req)
 {
     if (!connected)
         mico_throw (CosEventComm::Disconnected());
 
+    MICOMT::AutoLock requests_lock(requests);
+    MICOMT::AutoLock events_lock(events);
+
+    assert(requests.size()==0 || events.size()==0);
+
     // _duplicate()ing the request will defer completion
     requests.push_back (CORBA::StaticServerRequest::_duplicate (req));
-    if (requests.size() > events.size())
-	channel->listen (TRUE);
+
+    if (requests.size() == 1)
+         channel->listen (TRUE);
+
     if (events.size() > 0 && requests.size() > 0) {
-        assert (requests.size() > 0);
         CORBA::Any any = events.front();
         events.pop_front();
         notify (any);
@@ -208,6 +272,9 @@ ProxyPullSupplier_impl::try_pull (CORBA::Boolean &has_event)
     if (!connected)
         mico_throw (CosEventComm::Disconnected());
 
+    MICOMT::AutoLock requests_lock(requests);
+    MICOMT::AutoLock events_lock(events);
+
     if (events.size() > 0) {
         CORBA::Any *a = new CORBA::Any (events.front());
         events.pop_front();
@@ -228,29 +295,30 @@ ProxyPullSupplier_impl::try_pull (CORBA::Boolean &has_event)
 void
 ProxyPullSupplier_impl::notify (const CORBA::Any &any)
 {
+    MICOMT::AutoLock requests_lock(requests);
+    MICOMT::AutoLock events_lock(events);
+
     if (!connected || events.size() >= channel->max_queue_size())
         return;
-
-    events.push_back (any);
-
-    if (requests.size() >= events.size())
-	channel->listen (FALSE);
-
-    if (requests.size() == 0)
-        return;
-
+  
+    if (requests.size() == 0) {
+        events.push_back (any);
+	return;
+    }
+  
     CORBA::StaticServerRequest_var svreq =
         CORBA::StaticServerRequest::_duplicate (requests.front());
     requests.pop_front();
 
-    CORBA::StaticAny _res (CORBA::_stc_any, &events.front());
+    if (requests.size() == 0)
+        channel->listen(FALSE);
+  
+    CORBA::StaticAny _res (CORBA::_stc_any, &any);
     svreq->set_result (&_res);
     if (!svreq->read_args())
 	assert (0);
     svreq->write_results();
-
-    events.pop_front();
-
+  
     // when leaving scope svreq will be released which will anwer the request
 }
 
@@ -265,7 +333,9 @@ void
 ProxyPullConsumer_impl::connect_pull_supplier (
     CosEventComm::PullSupplier_ptr pull_supplier)
 {
-    if (!CORBA::is_nil (supplier) && !CORBA::is_nil (pull_supplier))
+    if (CORBA::is_nil(pull_supplier))
+        return;
+    if (!CORBA::is_nil (supplier))
         mico_throw (CosEventChannelAdmin::AlreadyConnected());
 
     supplier = CosEventComm::PullSupplier::_duplicate (pull_supplier);
@@ -337,8 +407,8 @@ CosEventChannelAdmin::ProxyPushSupplier_ptr
 ConsumerAdmin_impl::obtain_push_supplier ()
 {
     // create a new proxy
-    CosEventChannelAdmin::ProxyPushSupplier_ptr ptr =
-        new ProxyPushSupplier_impl (event_channel);
+    ProxyPushSupplier_impl* impl = new ProxyPushSupplier_impl (event_channel);
+    CosEventChannelAdmin::ProxyPushSupplier_ptr ptr = impl->_this();
     // and register it 
     event_channel->_reg_push_supplier (ptr);
     return ptr;
@@ -348,8 +418,8 @@ CosEventChannelAdmin::ProxyPullSupplier_ptr
 ConsumerAdmin_impl::obtain_pull_supplier ()
 {
     // create a new proxy
-    CosEventChannelAdmin::ProxyPullSupplier_ptr ptr =
-        new ProxyPullSupplier_impl (event_channel);
+    ProxyPullSupplier_impl* impl = new ProxyPullSupplier_impl (event_channel);
+    CosEventChannelAdmin::ProxyPullSupplier_ptr ptr = impl->_this();
     // and register it 
     event_channel->_reg_pull_supplier (ptr);
     return ptr;
@@ -366,8 +436,8 @@ CosEventChannelAdmin::ProxyPushConsumer_ptr
 SupplierAdmin_impl::obtain_push_consumer ()
 {
     // create a new proxy
-    CosEventChannelAdmin::ProxyPushConsumer_ptr ptr =
-        new ProxyPushConsumer_impl (event_channel);
+    ProxyPushConsumer_impl* impl = new ProxyPushConsumer_impl (event_channel);
+    CosEventChannelAdmin::ProxyPushConsumer_ptr ptr = impl->_this();
     // and register
     event_channel->_reg_push_consumer (ptr);
     return ptr;
@@ -377,8 +447,8 @@ CosEventChannelAdmin::ProxyPullConsumer_ptr
 SupplierAdmin_impl::obtain_pull_consumer ()
 {
     // create a new proxy
-    CosEventChannelAdmin::ProxyPullConsumer_ptr ptr =
-        new ProxyPullConsumer_impl (event_channel);
+    ProxyPullConsumer_impl* impl = new ProxyPullConsumer_impl (event_channel);
+    CosEventChannelAdmin::ProxyPullConsumer_ptr ptr = impl->_this();
     // and register
     event_channel->_reg_pull_consumer (ptr);
     return ptr;
@@ -386,15 +456,17 @@ SupplierAdmin_impl::obtain_pull_consumer ()
 
 //------------------------------------------------------------------------
 
-EventChannel_impl::EventChannel_impl (const CORBA::BOA::ReferenceData& tag) :
-  CosEventChannelAdmin::EventChannel_skel (tag) 
+EventChannel_impl::EventChannel_impl ()
+    : _push_supp(FALSE, MICOMT::Mutex::Recursive), _push_cons(FALSE, MICOMT::Mutex::Recursive),
+      _pull_supp(FALSE, MICOMT::Mutex::Recursive), _pull_cons(FALSE, MICOMT::Mutex::Recursive)
 {
     _listeners = 0;
     _max_queue_size = 0x7fffffffl;
 }
 
-EventChannel_impl::EventChannel_impl (CORBA::Object_ptr obj) :
-  CosEventChannelAdmin::EventChannel_skel (obj)
+EventChannel_impl::EventChannel_impl (CORBA::Object_ptr obj)
+    : _push_supp(FALSE, MICOMT::Mutex::Recursive), _push_cons(FALSE, MICOMT::Mutex::Recursive),
+      _pull_supp(FALSE, MICOMT::Mutex::Recursive), _pull_cons(FALSE, MICOMT::Mutex::Recursive)
 {
   _max_queue_size = 0x7fffffffl;
 #if 0
@@ -429,11 +501,13 @@ EventChannel_impl::EventChannel_impl (CORBA::Object_ptr obj) :
 }
 
 EventChannel_impl::EventChannel_impl (CORBA::Long mqs)
+    : _push_supp(FALSE, MICOMT::Mutex::Recursive), _push_cons(FALSE, MICOMT::Mutex::Recursive),
+      _pull_supp(FALSE, MICOMT::Mutex::Recursive), _pull_cons(FALSE, MICOMT::Mutex::Recursive)
 {
     _listeners = 0;
     _max_queue_size = mqs;
-    supplier_admin = new SupplierAdmin_impl (this);
-    consumer_admin = new ConsumerAdmin_impl (this);
+    supplier_admin = (new SupplierAdmin_impl (this))->_this();
+    consumer_admin = (new ConsumerAdmin_impl (this))->_this();
 }
 
 CORBA::Boolean
@@ -471,13 +545,14 @@ EventChannel_impl::for_suppliers ()
 void EventChannel_impl::destroy ()
 {
   _disconnect ();
-  CORBA::release (this);
+  _remove_ref(); //CORBA::release (this);
 }
 
 void
 EventChannel_impl::_reg_push_consumer (
     CosEventChannelAdmin::ProxyPushConsumer_ptr push_cons)
 {
+    MICOMT::AutoLock lock(_push_cons);
     _push_cons.push_back (
         CosEventChannelAdmin::ProxyPushConsumer::_duplicate (push_cons));
 }
@@ -486,6 +561,7 @@ void
 EventChannel_impl::_reg_push_supplier (
     CosEventChannelAdmin::ProxyPushSupplier_ptr push_supp)
 {
+    MICOMT::AutoLock lock(_push_supp);
     _push_supp.push_back (
         CosEventChannelAdmin::ProxyPushSupplier::_duplicate (push_supp));
 }
@@ -494,6 +570,7 @@ void
 EventChannel_impl::_reg_pull_consumer (
     CosEventChannelAdmin::ProxyPullConsumer_ptr pull_cons)
 {
+    MICOMT::AutoLock lock(_pull_cons);
     _pull_cons.push_back (
         CosEventChannelAdmin::ProxyPullConsumer::_duplicate (pull_cons));
 }
@@ -502,6 +579,7 @@ void
 EventChannel_impl::_reg_pull_supplier (
     CosEventChannelAdmin::ProxyPullSupplier_ptr pull_supp)
 {
+    MICOMT::AutoLock lock(_pull_supp);
     _pull_supp.push_back (
         CosEventChannelAdmin::ProxyPullSupplier::_duplicate (pull_supp));
 }
@@ -509,6 +587,8 @@ EventChannel_impl::_reg_pull_supplier (
 void
 EventChannel_impl::notify (const CORBA::Any &any)
 {
+    MICOMT::AutoLock push_lock(_push_supp);
+    MICOMT::AutoLock pull_lock(_pull_supp);
     list<CosEventChannelAdmin::ProxyPushSupplier_var>::iterator i;
     for (i = _push_supp.begin(); i != _push_supp.end (); i++)
         (*i)->notify (any);
@@ -520,7 +600,9 @@ EventChannel_impl::notify (const CORBA::Any &any)
 void
 EventChannel_impl::listen (CORBA::Boolean on)
 {
-    if (on && ++_listeners == 1) {
+    MICOMT::AutoLock lock(_pull_cons);
+    MICOMT::AutoLock list_lock(_listeners_mutex);
+    if (on && ++_listeners >= 1) {
 	list<CosEventChannelAdmin::ProxyPullConsumer_var>::iterator i;
 	for (i = _pull_cons.begin(); i != _pull_cons.end (); i++) {
 	    (*i)->listen ();
@@ -534,12 +616,17 @@ EventChannel_impl::listen (CORBA::Boolean on)
 CORBA::Boolean
 EventChannel_impl::listen ()
 {
+    MICOMT::AutoLock list_lock(_listeners_mutex);
     return _listeners > 0;
 }
 
 void
 EventChannel_impl::_disconnect ()
 {
+    MICOMT::AutoLock push_supp_lock(_push_supp);
+    MICOMT::AutoLock pull_supp_lock(_pull_supp);
+    MICOMT::AutoLock push_cons_lock(_push_cons);
+    MICOMT::AutoLock pull_cons_lock(_pull_cons);
     list<CosEventChannelAdmin::ProxyPushSupplier_var>::iterator i;
     for (i = _push_supp.begin(); i != _push_supp.end (); i++)
         (*i)->disconnect_push_supplier ();
@@ -559,15 +646,12 @@ EventChannel_impl::_disconnect ()
 
 //------------------------------------------------------------------------
 
-EventChannelFactory_impl::EventChannelFactory_impl 
-(const CORBA::BOA::ReferenceData& tag) :
-  SimpleEventChannelAdmin::EventChannelFactory_skel (tag) 
+EventChannelFactory_impl::EventChannelFactory_impl ()
 {
   _max_queue_size = 0x7fffffffl;
 }
 
-EventChannelFactory_impl::EventChannelFactory_impl (CORBA::Object_ptr obj) :
-  SimpleEventChannelAdmin::EventChannelFactory_skel (obj)
+EventChannelFactory_impl::EventChannelFactory_impl (CORBA::Object_ptr obj)
 {
   _max_queue_size = 0x7fffffffl;
 }
@@ -580,8 +664,9 @@ EventChannelFactory_impl::EventChannelFactory_impl (CORBA::Long mqs)
 CosEventChannelAdmin::EventChannel_ptr 
 EventChannelFactory_impl::create_eventchannel ()
 {
-  CosEventChannelAdmin::EventChannel_ptr channel =
-    new EventChannel_impl (_max_queue_size);
+  EventChannel_impl* impl = new EventChannel_impl (_max_queue_size);
+  CosEventChannelAdmin::EventChannel_ptr channel = impl->_this();
+    
 
   return CosEventChannelAdmin::EventChannel::_duplicate (channel);
 }
@@ -589,25 +674,6 @@ EventChannelFactory_impl::create_eventchannel ()
 void EventChannelFactory_impl::_shutdown ()
 {
   CORBA::ORB_var orb = CORBA::ORB_instance ("mico-local-orb");
-  CORBA::BOA_var boa = orb->BOA_instance ("mico-local-boa");
-  cerr << "shutdown event service" << endl;
-  boa->deactivate_impl (CORBA::ImplementationDef::_nil ());
   orb->shutdown (TRUE);
 }
 
-CORBA::Boolean EventChannelLoader::restore (CORBA::Object_ptr obj)
-{
-  if (::strcmp (obj->_repoid (), 
-		"IDL:CosEventChannelAdmin/EventChannel:1.0") == 0) {
-    (void)new EventChannel_impl (obj);
-    return TRUE;
-  }
-  else if (::strcmp (obj->_repoid (), 
-		     "IDL:SimpleEventChannelAdmin/EventChannelFactory:1.0") 
-	   == 0) {
-    (void)new EventChannelFactory_impl (obj);
-    return TRUE;
-  }
-  return FALSE;
-}
-    
