@@ -21,7 +21,7 @@
  */
 
 #include <CORBA-SMALL.h>
-#include <mico/CosNaming.h>
+#include <coss/CosNaming.h>
 #include <mico/util.h>
 #include "NamingClient.h"
 #include <signal.h>
@@ -34,6 +34,9 @@ extern "C" {
 }
 #endif
 
+
+using namespace std;
+
 static const char info[] = "CosNaming administration tool for MICO(tm)";
 
 int parse_cmd (CORBA::ORB_var& orb, NamingClient& nsclient, 
