@@ -22,18 +22,26 @@
 
 #define MICO_CONF_IMR
 #define MICO_CONF_POA
-
+#define MICO_CONF_IR
+#define MICO_CONF_INTERCEPT
 #include <CORBA-SMALL.h>
 #include <stdlib.h>
 #include <signal.h>
 #include <unistd.h>
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <iostream>
+#include <fstream>
+#else
 #include <iostream.h>
 #include <fstream.h>
+#endif
 #include <mico/template_impl.h>
 #include <mico/util.h>
 #include "Naming_impl.h"
 
 
+using namespace std;
+
 bool finished;
 
 void sighandler (int sig)
@@ -140,16 +148,20 @@ int main (int argc, char *_argv[])
    */
 
   CORBA::PolicyList pl;
-  pl.length (1);
+  pl.length (2);
   pl[0] = RootPOA->create_lifespan_policy (PortableServer::PERSISTENT);
+  // It would be nice to have fully multi-threaded naming service, but
+  // the code isn't prepared for running in MT environment such that
+  // we select to use SINGLE_THREAD_MODEL policy for now.
+  pl[1] = RootPOA->create_thread_policy (PortableServer::SINGLE_THREAD_MODEL);
   PortableServer::POA_var ncpoa =
     RootPOA->create_POA ("NamingContexts",
 			 PortableServer::POAManager::_nil (),
 			 pl);
   PortableServer::POAManager_var rcmgr = ncpoa->the_POAManager ();
 
-  pl.length (2);
-  pl[1] = RootPOA->create_id_assignment_policy (PortableServer::USER_ID);
+  pl.length (3);
+  pl[2] = RootPOA->create_id_assignment_policy (PortableServer::USER_ID);
   PortableServer::POA_var rcpoa =
     RootPOA->create_POA ("NameService", rcmgr.in(), pl);
 
