@@ -301,7 +301,7 @@ ImplRec::add_request (MICO::ReqQueueRec *inv)
 }
 
 CORBA::Boolean
-ImplRec::must_queue (MsgId id)
+ImplRec::must_queue (CORBA::ORBMsgId id)
 {
     if (_invqueue.size() == 0)
         return FALSE;
@@ -1144,8 +1144,17 @@ MediatorImpl::is_local () const
     return TRUE;
 }
 
+#ifdef USE_CSL2
+CORBA::Principal_ptr
+MediatorImpl::get_principal(CORBA::Object_ptr obj){
+  assert(0);
+  return (CORBA::Principal_ptr)0;
+}
+#endif /* USE_CSL2  */
+
 CORBA::Boolean
-MediatorImpl::invoke (MsgId id, CORBA::Object_ptr obj,
+MediatorImpl::invoke (CORBA::ORBMsgId id,
+		      CORBA::Object_ptr obj,
 		      CORBA::ORBRequest *req,
 		      CORBA::Principal_ptr pr,
 		      CORBA::Boolean response_exp)
@@ -1201,11 +1210,13 @@ MediatorImpl::invoke (MsgId id, CORBA::Object_ptr obj,
     }
 
     // do the invocation (on the remote object)
-    MsgId orbid = _orbnc()->new_msgid();
-    if (response_exp)
-	_requests[orbid] = id;
+    CORBA::ORBMsgId neworbid = 0;
+    if (response_exp) {
+	neworbid = _orbnc()->new_orbid();
+	_requests[CORBA::ORB::get_msgid(neworbid)] = CORBA::ORB::get_msgid(id);
+    }
     _orbnc()->invoke_async (orec->remote_obj(), req, pr, response_exp,
-			    this, orbid);
+			    this, neworbid);
 
     pstat = irec->after_invoke (orec, req, pr);
     switch (pstat) {
@@ -1219,7 +1230,7 @@ MediatorImpl::invoke (MsgId id, CORBA::Object_ptr obj,
 }
 
 CORBA::Boolean
-MediatorImpl::bind (MsgId id, const char *repoid,
+MediatorImpl::bind (CORBA::ORBMsgId id, const char *repoid,
 		    const CORBA::ORB::ObjectTag &tag,
 		    CORBA::Address *addr)
 {
@@ -1305,7 +1316,7 @@ MediatorImpl::bind (MsgId id, const char *repoid,
             if ((*j).second->state() == ServerActive) {
                 // only active servers ...
 		msgids.push_back (_orbnc()->new_msgid());
-		_requests[msgids.back()] = id;
+		_requests[msgids.back()] = CORBA::ORB::get_msgid(id);
 	    }
 	    count++;
 	}
@@ -1347,8 +1358,11 @@ MediatorImpl::bind (MsgId id, const char *repoid,
 	    assert (addr);
 
 	    assert (msgids.size() > 0);
-	    MsgId orbid = _orbnc()->bind_async (repoid, tag, addr, this,
-						msgids.back());
+	    MsgId orbid = _orbnc()->bind_async (repoid,
+						tag,
+						addr,
+						this,
+						_orbnc()->new_orbid(msgids.back()) );
 	    msgids.pop_back();
 
 	    assert (orbid != 0);
@@ -1358,7 +1372,7 @@ MediatorImpl::bind (MsgId id, const char *repoid,
 }
 
 CORBA::Boolean
-MediatorImpl::locate (MsgId id, CORBA::Object_ptr obj)
+MediatorImpl::locate (CORBA::ORBMsgId id, CORBA::Object_ptr obj)
 {
     assert (find_obj (obj));
 
@@ -1376,8 +1390,9 @@ MediatorImpl::skeleton (CORBA::Object_ptr)
 }
 
 void
-MediatorImpl::cancel (MsgId id)
+MediatorImpl::cancel (CORBA::ORBMsgId orbid)
 {
+    MsgId id = CORBA::ORB::get_msgid(orbid);
     int again = 1;
     while (again) {
 	again = 0;
@@ -1401,7 +1416,7 @@ MediatorImpl::shutdown (CORBA::Boolean wait_for_completion)
 }
 
 void
-MediatorImpl::answer_invoke (CORBA::ULong, CORBA::Object_ptr,
+MediatorImpl::answer_invoke (CORBA::ORBMsgId, CORBA::Object_ptr,
 			     CORBA::ORBRequest *, CORBA::InvokeStatus)
 {
     assert (0);
@@ -1638,9 +1653,20 @@ MediatorImpl::force_activation (CORBA::ImplementationDef_ptr impl)
     }
 }
 
+CORBA::Boolean
+MediatorImpl::waitfor (CORBA::ORB_ptr,
+		       CORBA::ORBMsgId msgid,
+		       CORBA::ORBCallback::Event ev,
+		       CORBA::Long tmout)
+{
+    assert(0);
+    return TRUE;
+}
+
 void
-MediatorImpl::callback (CORBA::ORB_ptr, MsgId id,
-			CORBA::ORBCallback::Event ev)
+MediatorImpl::notify (CORBA::ORB_ptr,
+		      CORBA::ORBMsgId id,
+		      CORBA::ORBCallback::Event ev)
 {
     switch (ev) {
     case CORBA::ORBCallback::Invoke: {
