@@ -1,9 +1,14 @@
 #define MICO_CONF_IMR
 #include <CORBA-SMALL.h>
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <fstream>
+#else
 #include <fstream.h>
+#endif
 #include "calc.h"
 
 
+using namespace std;
 
 class calc_impl : virtual public Calc2::calc_skel   {
    complex      value;
