@@ -1,7 +1,12 @@
-#include <fstream.h>
 #include <CORBA-SMALL.h>
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <fstream>
+#else
+#include <fstream.h>
+#endif
 #include "tty.h"           /* wurde aus 'tty.idl' generiert */
 
+using namespace std;
 
 /*
  * Client
