@@ -21,17 +21,29 @@
 */
 
 
+#ifdef FAST_PCH
+#include "idl_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #include <CORBA.h>
-#include <fstream.h>
 #include <ctype.h>
 #include <stdio.h>
 #include "codegen-c++-impl.h"
 #include <mico/template_impl.h>
 #include <mico/util.h>
 
+#endif // FAST_PCH
+
 #define _IMPL "_impl"
 
 
+using namespace std;
+
 CodeGenCPPImpl::CodeGenCPPImpl( DB &db, IDLParam &params,
 				CORBA::Container_ptr con )
   : CodeGen( con ),
@@ -61,7 +73,7 @@ void CodeGenCPPImpl::emit_impl()
     
     CORBA::InterfaceDef_var in;
     in = CORBA::InterfaceDef::_narrow( _idl_objs[ i ]->obj );
-    if( CORBA::is_nil( in ) )
+    if( CORBA::is_nil( in ) || in->def_kind() != CORBA::dk_Interface )
       continue;
     
     if( _idl_objs[ i ]->iface_as_forward )
