@@ -21,14 +21,26 @@
  */
 
 
+#ifdef FAST_PCH
+#include "idl_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #include <CORBA.h>
-#include <fstream.h>
 #include <ctype.h>
 #include <stdio.h>
 #include "codegen-c++-skel.h"
 #include <mico/template_impl.h>
 #include <mico/util.h>
 
+#endif // FAST_PCH
+
+
+using namespace std;
 
 CodeGenCPPSkel::CodeGenCPPSkel( DB &db, IDLParam &params,
 				CORBA::Container_ptr con )
@@ -248,10 +260,17 @@ void CodeGenCPPSkel::emit_skel()
     
     if( check_for_included_defn( in ) )
       continue;
-    
+
     if (_idl_objs[i0]->obj->def_kind() == CORBA::dk_LocalInterface)
       continue;
     
+    if (!CORBA::is_nil (con)) {
+      CORBA::String_var an = con->absolute_name ();
+      if (_db->is_implicit (an.in())) {
+	continue;
+      }
+    }
+    
     if( !CORBA::is_nil( val ) ) {
       CORBA::InterfaceDefSeq_var ids = val->supported_interfaces ();
       for (CORBA::ULong j=0; j<ids->length(); j++) {
@@ -298,7 +317,7 @@ void CodeGenCPPSkel::emit_skel()
       o << "_impl," << endl;
       o << "\"" << in_id << "\" );" << exdent << endl;
       o << "register_dispatcher( new ";
-      o << "StaticInterfaceDispatcherWrapper<";
+      o << "StaticInterfaceDispatcherWrapper< ";
       o << className << _SKEL;
       o << ">( this ) );" << endl;
       o << BL_CLOSE << endl;
@@ -315,7 +334,7 @@ void CodeGenCPPSkel::emit_skel()
       o << "0," << endl;
       o << "_impl );" << exdent << endl;
       o << "register_dispatcher( new ";
-      o << "StaticInterfaceDispatcherWrapper<";
+      o << "StaticInterfaceDispatcherWrapper< ";
       o << className << _SKEL;
       o << ">( this ) );" << endl;
       o << BL_CLOSE << endl;
@@ -406,7 +425,7 @@ void CodeGenCPPSkel::emit_skel()
       o << BL_OPEN;
 
       // XXX win32 hack
-      o << "CORBA::Object_var obj = MICO_SCOPE(PortableServer,ServantBase::_this) ();" << endl;
+      o << "CORBA::Object_var obj = PortableServer::ServantBase::_this();" << endl;
       o << "return ";
       emit_type_name (in);
       o << "::_narrow (obj);" << endl;
@@ -429,11 +448,6 @@ void CodeGenCPPSkel::emit_skel()
 	  string base_name;
 	  base_name = ID(tmp);
 	  base_name = base_name.substr( 2 );
-#ifdef _VC_NAMESPACE_BUG
-          // emits a typedef, the new name is returned and used in
-          // in the next line
-	  base_name = vc_emit_workaround ("POA_", base_name);    
-#endif
 	  o << "if (POA_" << base_name << "::_is_a (repoid)) " << BL_OPEN;
 	  o << "return TRUE;" << endl;
 	  o << BL_CLOSE;
@@ -448,11 +462,6 @@ void CodeGenCPPSkel::emit_skel()
 	string base_name;
 	base_name = ID(tmp);
 	base_name = base_name.substr( 2 );
-#ifdef _VC_NAMESPACE_BUG
-	// emits a typedef, the new name is returned and used in
-	// in the next line
-	base_name = vc_emit_workaround ("POA_", base_name);    
-#endif
 	o << "if (POA_" << base_name << "::_is_a (repoid)) " << BL_OPEN;
 	o << "return TRUE;" << endl;
 	o << BL_CLOSE;
@@ -467,9 +476,6 @@ void CodeGenCPPSkel::emit_skel()
 	      tmp = base_value->absolute_name();
 	      base_name = ID(tmp);
 	      base_name = base_name.substr (2);
-#ifdef _VC_NAMESPACE_BUG
-	      base_name = vc_emit_workaround ("POA_", base_name);
-#endif
 	      o << "if (POA_" << base_name << "::_is_a (repoid)) " << BL_OPEN;
 	      o << "return TRUE;" << endl;
 	      o << BL_CLOSE;
@@ -489,7 +495,7 @@ void CodeGenCPPSkel::emit_skel()
       o << BL_OPEN;
       // XXX Win32 hack
       o << "CORBA::InterfaceDef_ptr ifd = "
-        << "MICO_SCOPE(PortableServer,ServantBase)::_get_interface "
+        << "PortableServer::ServantBase::_get_interface "
         << "(\"" << in_id << "\");"
 	<< endl << endl;
       o << "if (CORBA::is_nil (ifd)) " << BL_OPEN;
@@ -559,7 +565,9 @@ void CodeGenCPPSkel::emit_skel()
 	  o << "__req->write_results();" << endl;
 	  o << "return true;" << exdent << endl;
 	  o << "} catch( ... ) " << BL_OPEN;
-	  o << "assert( 0 );" << endl;
+	  o << "CORBA::UNKNOWN _ex (CORBA::OMGVMCID | 1, CORBA::COMPLETED_MAYBE);" << endl;
+	  o << "__req->set_exception (_ex->_clone());" << endl;
+	  o << "__req->write_results ();" << endl;
 	  o << "return true;" << endl;
 	  o << BL_CLOSE;
 	  o << "#endif" << endl;
@@ -576,9 +584,6 @@ void CodeGenCPPSkel::emit_skel()
 	  string base_name;
 	  base_name = ID(tmp);
 	  base_name = base_name.substr( 2 );
-#ifdef _VC_NAMESPACE_BUG
-          base_name = vc_emit_workaround ("POA_", base_name);
-#endif
 	  o << "if (POA_" << base_name << "::dispatch (__req)) " << BL_OPEN;
 	  o << "return true;" << endl;
 	  o << BL_CLOSE << endl;
@@ -589,9 +594,6 @@ void CodeGenCPPSkel::emit_skel()
 	tmp = in->absolute_name();
 	base_name = ID(tmp);
 	base_name = base_name.substr( 2 );
-#ifdef _VC_NAMESPACE_BUG
-	base_name = vc_emit_workaround ("POA_", base_name);
-#endif
 	o << "if (POA_" << base_name << "::dispatch (__req)) ";
 
 	o << BL_OPEN << "return true;" << endl;
@@ -607,9 +609,6 @@ void CodeGenCPPSkel::emit_skel()
 	      tmp = base_supported[i]->absolute_name();
 	      base_name = ID(tmp);
 	      base_name = base_name.substr (2);
-#ifdef _VC_NAMESPACE_BUG
-	      base_name = vc_emit_workaround ("POA_", base_name);    
-#endif
 	      o << "if (POA_" << base_name << "::dispatch (__req)) " << BL_OPEN;
 	      o << "return TRUE;" << endl;
 	      o << BL_CLOSE;
@@ -648,7 +647,17 @@ CodeGenCPPSkel::emit_attr_dispatcher (CORBA::AttributeDescription &attr,
   CORBA::IDLType_var idl_type;
   string dispatch_name;
 
-  idl_type = lookup_attribute_by_id( attr.id );
+  CORBA::Contained_var con = _repo->lookup_id (attr.id);
+  CORBA::AttributeDef_var a = CORBA::AttributeDef::_narrow (con);
+  CORBA::ExtAttributeDef_var ead = CORBA::ExtAttributeDef::_narrow (a);
+  CORBA::ExtAttributeDescription_var eads;
+  assert (!CORBA::is_nil (a));
+
+  if (!CORBA::is_nil (ead)) {
+    eads = ead->describe_attribute();
+  }
+
+  idl_type = a->type_def();
 
   if( which == CORBA::PARAM_OUT || which == CORBA::PARAM_INOUT ) {
     op.name = (const char *) attr.name;
@@ -659,7 +668,10 @@ CodeGenCPPSkel::emit_attr_dispatcher (CORBA::AttributeDescription &attr,
     op.mode = CORBA::OP_NORMAL;
     // op.contexts not needed for attributes
     op.parameters.length( 0 );  // No parameters for _get
-    // op.exceptions not needed for attributes
+
+    if (!CORBA::is_nil (ead)) {
+      op.exceptions = eads->get_exceptions;
+    }
 
     dispatch_name = string("_get_") + (const char *)attr.name;
 
@@ -684,7 +696,10 @@ CodeGenCPPSkel::emit_attr_dispatcher (CORBA::AttributeDescription &attr,
     // op.contexts not needed for attributes
     op.parameters.length( 1 );
     op.parameters[ 0 ] = attr_param;
-    // op.exceptions not needed for attributes
+
+    if (!CORBA::is_nil (ead)) {
+      op.exceptions = eads->set_exceptions;
+    }
     
     dispatch_name = string("_set_") + (const char *)attr.name;
     
@@ -710,7 +725,7 @@ CodeGenCPPSkel::emit_sii_method_dispatcher( string &dispatch_name,
     if( p[ k ].mode != CORBA::PARAM_OUT ) {
       emit_type_for_variable( p[ k ].type_def, automatic );
       if( !p[k].type->is_array() ) {
-	o << " " << ID(p[ k ].name);
+	o << " _par_" << ID(p[ k ].name);
 	emit_array_suffix( p[ k ].type_def );
 #if 0
 	if (p[k].mode == CORBA::PARAM_INOUT && is_var_inout (p[k].type_def))
@@ -718,32 +733,32 @@ CodeGenCPPSkel::emit_sii_method_dispatcher( string &dispatch_name,
 #endif
 	o << ";" << endl;
       } else {
-	o << "_slice* " << ID(p[ k ].name) << " = ";
+	o << "_slice* _par_" << ID(p[ k ].name) << " = ";
 	emit_type_for_variable( p[ k ].type_def, automatic );
 	o << "_alloc();" << endl;
       }
     } else {
       if( p[k].type->is_array() ) {
 	emit_type_for_variable( p[ k ].type_def, automatic );
-	o << "_slice* " << ID(p[ k ].name) << " = ";
+	o << "_slice* _par_" << ID(p[ k ].name) << " = ";
 	emit_type_for_variable( p[ k ].type_def, automatic );
 	o << "_alloc();" << endl;
       } else {
 	emit_type_for_variable( p[ k ].type_def, manual );
-	o << " " << ID(p[ k ].name) << ";" << endl;
+	o << " _par_" << ID(p[ k ].name) << ";" << endl;
       }
     }
 
     // Emit StaticAny for argument
-    o << "CORBA::StaticAny _" << ID(p[k].name) << "( ";
+    o << "CORBA::StaticAny _sa_" << ID(p[k].name) << "( ";
     emit_marshaller_ref( p[k].type_def );
     if( p[k].mode != CORBA::PARAM_OUT || !is_variable_for_sii (p[k].type) ) {
       o << ", ";
       if( !p[k].type->is_array() )
 	o << "&";
-      o << ID(p[k].name);
+      o << "_par_" << ID(p[k].name);
       if (p[k].mode != CORBA::PARAM_OUT)
-	emit_marshaller_suffix( p[k].type_def, FALSE );
+	emit_marshaller_suffix( p[k].type_def, TRUE );
     }
     o << " );" << endl;
   }
@@ -783,7 +798,7 @@ CodeGenCPPSkel::emit_sii_method_dispatcher( string &dispatch_name,
     default:
       assert( 0 );
     }
-    o << "_arg( &_" << ID(p[k].name) << " );" << endl;
+    o << "_arg( &_sa_" << ID(p[k].name) << " );" << endl;
   }
 
   if( tc_result->kind() != CORBA::tk_void ) {
@@ -811,7 +826,7 @@ CodeGenCPPSkel::emit_sii_method_dispatcher( string &dispatch_name,
   for( k = 0; k < p.length(); k++ ) {
     if( k != 0 )
       o << ",";
-    o << " " << ID(p[ k ].name);
+    o << " _par_" << ID(p[ k ].name);
     if (p[k].mode != CORBA::PARAM_OUT)
 	emit_marshaller_suffix( p[k].type_def, FALSE );
   }
@@ -828,9 +843,9 @@ CodeGenCPPSkel::emit_sii_method_dispatcher( string &dispatch_name,
   // Write back variable length out & result parameters to StaticAny
   for( k = 0; k != p.length(); k++ ) {
     if( p[k].mode == CORBA::PARAM_OUT && is_variable_for_sii (p[k].type) ) {
-      o << "_" << ID(p[k].name) << ".value( ";
+      o << "_sa_" << ID(p[k].name) << ".value( ";
       emit_marshaller_ref( p[k].type_def );
-      o << ", " << ID(p[k].name) << " );" << endl;
+      o << ", _par_" << ID(p[k].name) << " );" << endl;
     }
   }
   // arrays are always variable as return types ...
@@ -866,7 +881,8 @@ CodeGenCPPSkel::emit_sii_method_dispatcher( string &dispatch_name,
   // Release memory of variable length out and inout parameters
   for( k = 0; k < p.length(); k++ ) {
     if( ( p[ k ].mode == CORBA::PARAM_OUT ) || p[ k ].type->is_array() ) {
-      string name = ID(p[k].name);
+      string name = "_par_";
+      name += ID(p[k].name);
       emit_release( p[k].type_def, name.c_str() );
     }
   }
