@@ -1,6 +1,6 @@
 /*
  *  MICO --- an Open Source CORBA implementation
- *  Copyright (c) 1997-2001 by The Mico Team
+ *  Copyright (c) 1997-2003 by The Mico Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,10 +20,28 @@
  *  http://www.mico.org/
  */
 
+#ifdef FAST_PCH
+#include "idl_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #include <CORBA.h>
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <sstream>
+#else
+#include <strstream.h>
+#endif
 #include <mico/util.h>
 #include "codegen.h"
 
+#endif // FAST_PCH
+
+
+using namespace std;
 
 //-- OutputStream -------------------------------------------------------
 
@@ -32,7 +50,11 @@ OutputStream::OutputStream( int indent )
   _indent = indent;
   _current_indent = 0;
   _bofl = true;
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+  _ostr = new ostringstream;
+#else
   _ostr = new ostrstream;
+#endif
 }
 
 
@@ -115,7 +137,7 @@ OutputStream &OutputStream::operator<<( int i )
 OutputStream &OutputStream::operator<<( CORBA::LongDouble ld )
 {
   checkBofL();
-#if defined(__hpux) && !defined(__GNUG__)
+#if (defined(__hpux) && !defined(__GNUG__)) || (defined(__linux__) && defined(__COMO__) && defined(_STLPORT_VERSION))
   // XXX
   *_ostr << double(ld);
 #else
@@ -145,11 +167,7 @@ OutputStream &OutputStream::operator<<( CORBA::Double d )
 OutputStream &OutputStream::operator<<( CORBA::LongLong ll )
 {
   checkBofL();
-#if defined(_WINDOWS) && !defined(__MINGW32__)
-  *_ostr << long( ll );  //XXX
-#else
   *_ostr << ll;
-#endif
   return *this;
 }
 
@@ -157,11 +175,7 @@ OutputStream &OutputStream::operator<<( CORBA::LongLong ll )
 OutputStream &OutputStream::operator<<( CORBA::ULongLong ull )
 {
   checkBofL();
-#if defined(_WINDOWS) && !defined(__MINGW32__)
-  *_ostr << unsigned long( ull ); //XXX
-#else
   *_ostr << ull;
-#endif
   return *this;
 }
 
@@ -206,7 +220,7 @@ OutputStream &OutputStream::operator<<( CORBA::Char c )
 }
 
 
-#ifdef _WINDOWS
+#if defined(_WIN32) && defined(_MSC_VER)
 OutputStream &OutputStream::operator<<( MICO_WCharWrapper c )
 #else
 OutputStream &OutputStream::operator<<( CORBA::WChar c )
@@ -365,7 +379,7 @@ OutputStream &OutputStream::operator<<( const CORBA::Any *a )
 	   * else it will be scanned part of the hex sequence
 	   */
 	  char buf[ 10 ];
-	  sprintf( buf, "\\x%04x", (unsigned long) *s );
+	  sprintf( buf, "\\x%04x", (unsigned int) *s );
 	  *this << buf;
 	  hex=true;
 	}
@@ -384,7 +398,7 @@ OutputStream &OutputStream::operator<<( const CORBA::Any *a )
 	}
 	else {
 	  char buf[ 10 ];
-	  sprintf( buf, "\\x%04x", (unsigned long) *s );
+	  sprintf( buf, "\\x%04x", (unsigned int) *s );
 	  *this << buf;
 	  hex=true;
 	}
@@ -416,7 +430,7 @@ OutputStream &OutputStream::operator<<( const CORBA::Any *a )
 	*this << "'\\" << (CORBA::Char)c << "'";
       else if( c < L' ' || c > L'z' ) {
 	char buf[ 10 ];
-	sprintf( buf, "'\\u%04x'", (unsigned long) c);
+	sprintf( buf, "'\\u%04x'", (unsigned int) c);
 	*this << buf;
       }	else
 	*this << "'" << (CORBA::Char)c << "'";
@@ -449,11 +463,13 @@ OutputStream &OutputStream::operator<<( const CORBA::Any *a )
 
 OutputStream &OutputStream::operator<<( endlnfp e )
 {
-#ifdef __SUNPRO_CC
+#if defined(__SUNPRO_CC) || defined(_STLPORT_VERSION) || _CPPLIB_VER >= 400
   // XXX
+  // kcg: this is just hack which needs to be investigated. I've added
+  // stlport and cpplib defines. sunpro is original.
   if (1) {
 #else
-  if( e == endl ) {
+  if( e == (endlnfp) endl ) {
 #endif
     *_ostr << endl;
     _bofl = true;
@@ -502,9 +518,13 @@ void Output::free_streams()
     if( _streams[ i ] == NULL )
       continue;
     if( _ostr != NULL ) {
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+      *_ostr << _streams[ i ]->_ostr->str();
+#else
       *_streams[ i ]->_ostr << ends;
       *_ostr << _streams[ i ]->_ostr->str();
       _streams[ i ]->_ostr->rdbuf()->freeze( 0 );
+#endif
     }
     delete _streams[ i ];
     _streams[ i ] = NULL;
@@ -662,7 +682,7 @@ Output &Output::operator<<( CORBA::Char c )
 }
 
 
-#ifdef _WINDOWS
+#if defined(_WIN32) && defined(_MSC_VER)
 Output &Output::operator<<( MICO_WCharWrapper c )
 #else
 Output &Output::operator<<( CORBA::WChar c )
