@@ -20,8 +20,21 @@
  *  http://www.mico.org/
  */
 
+#ifdef FAST_PCH
+#include "idl_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #include "dep.h"
 
+#endif // FAST_PCH
+
+
+using namespace std;
 
 /*
  * Class IDLDep computes the dependencies between IDL definitions.
@@ -75,9 +88,30 @@ IDLDep::build_dependencies()
   set_current_iface( CORBA::Container::_nil() );
   traverse_container( _container );
   compute_dependencies();
+
   // print (_top);
+
+#if 0
+  {
+    cout << "Ordered objs:" << endl;
+    for (CORBA::ULong i=0; i<_ordered_objs.length(); i++) {
+      CORBA::Contained_var co =
+	CORBA::Contained::_narrow (_ordered_objs[i]->obj);
+      if (!CORBA::is_nil (co)) {
+	CORBA::String_var an = co->absolute_name ();
+	if (_ordered_objs[i]->iface_as_forward) {
+	  cout << "  " << an << " (forward decl)" << endl;
+	}
+	else {
+	  cout << "  " << an << endl;
+	}
+      }
+    }
+  }
+#endif
+
   return _ordered_objs;
-};
+}
 
 void
 IDLDep::print (Dep * where)
@@ -160,7 +194,7 @@ IDLDep::is_circular_dep( Dep* from, Dep* to )
   if( o1->_is_equivalent( o2 ) )
     return true;
   
-  for( int i = 0; i < to->deps.size(); i++ ) {
+  for( unsigned int i = 0; i < to->deps.size(); i++ ) {
     if( is_circular_dep( from, to->deps[ i ] ) )
       return true;
   }
@@ -330,6 +364,8 @@ IDLDep::locate_dep( CORBA::IRObject_ptr obj,
       create_dep( d, elem_dep );
       break;
     }
+  default:
+    break;
   }
   
   return d;
@@ -349,7 +385,6 @@ IDLDep::dep_Struct( CORBA::StructDef_ptr s )
   CORBA::StructMemberSeq_var members = s->members();
 
   for( CORBA::ULong i = 0; i < members->length(); i++ ) {
-    CORBA::TypeCode_ptr tc = members[ i ].type;
     CORBA::IDLType_ptr m = members[ i ].type_def;
     Dep* to = locate_dep( m, true );
     create_dep( from, to );
@@ -370,7 +405,6 @@ IDLDep::dep_Union( CORBA::UnionDef_ptr u )
   CORBA::UnionMemberSeq_var members = u->members();
 
   for( CORBA::ULong i = 0; i < members->length(); i++ ) {
-    CORBA::TypeCode_ptr tc = members[ i ].type;
     CORBA::IDLType_ptr m = members[ i ].type_def;
     Dep* to = locate_dep( m, true );
     create_dep( from, to );
@@ -487,6 +521,25 @@ IDLDep::dep_Attribute( CORBA::AttributeDef_ptr a )
   CORBA::IDLType_var t = a->type_def();
   Dep* d = locate_dep( t, true );
   create_dep( from, d );
+
+  CORBA::ExtAttributeDef_var ead = CORBA::ExtAttributeDef::_narrow (a);
+
+  if (!CORBA::is_nil (ead)) {
+    CORBA::ExceptionDefSeq_var exs = ead->get_exceptions ();
+
+    for (CORBA::ULong i0=0; i0<exs->length(); i0++) {
+      d = locate_dep (exs[i0], true);
+      create_dep (from, d);
+    }
+
+    exs = ead->set_exceptions ();
+
+    for (CORBA::ULong i1=0; i1<exs->length(); i1++) {
+      d = locate_dep (exs[i1], true);
+      create_dep (from, d);
+    }
+  }
+
 }
 
 void
@@ -610,7 +663,7 @@ IDLDep::dep_Uses (CORBA::ComponentIR::UsesDef_ptr me)
 }
 
 void
-IDLDep::dep_Event (CORBA::ComponentIR::EventDef_ptr me)
+IDLDep::dep_EventPort (CORBA::ComponentIR::EventPortDef_ptr me)
 {
   Dep* from = locate_dep( me );
   create_iface_dep( from );
@@ -636,6 +689,12 @@ IDLDep::dep_Home (CORBA::ComponentIR::HomeDef_ptr me)
     create_dep (from, to);
   }
 
+  CORBA::InterfaceDefSeq_var supported = me->supported_interfaces();
+  for (CORBA::ULong si=0; si<supported->length(); si++) {
+    Dep * to = locate_dep (supported[si], true, true);
+    create_dep (from, to);
+  }
+
   CORBA::ComponentIR::ComponentDef_var managed = me->managed_component ();
   if (!CORBA::is_nil (managed)) {
     Dep * to = locate_dep (managed, true, true);
@@ -730,6 +789,9 @@ IDLDep::traverse_container( CORBA::Container_ptr con )
 	break;
       }
     case CORBA::dk_Value:
+#ifdef USE_CCM
+    case CORBA::dk_Event:
+#endif
       {
 	CORBA::ValueDef_var val = CORBA::ValueDef::_narrow( c[ i ] );
 	dep_Value( val );
@@ -775,9 +837,9 @@ IDLDep::traverse_container( CORBA::Container_ptr con )
     case CORBA::dk_Publishes:
     case CORBA::dk_Consumes:
       {
-	CORBA::ComponentIR::EventDef_var comp =
-	  CORBA::ComponentIR::EventDef::_narrow (c[i]);
-	dep_Event (comp);
+	CORBA::ComponentIR::EventPortDef_var comp =
+	  CORBA::ComponentIR::EventPortDef::_narrow (c[i]);
+	dep_EventPort (comp);
 	break;
       }
     case CORBA::dk_Home:
@@ -790,7 +852,7 @@ IDLDep::traverse_container( CORBA::Container_ptr con )
     case CORBA::dk_Factory:
     case CORBA::dk_Finder:
       {
-	CORBA::OperationDef_var op = CORBA::OperationDef::_narrow (c[1]);
+	CORBA::OperationDef_var op = CORBA::OperationDef::_narrow (c[i]);
 	dep_Operation (op);
 	break;
       }
@@ -923,6 +985,6 @@ IDLDep::add_anonymous_types( Dep* dep )
   }
   
   // recurse over whole dependency graph
-  for( int i = 0; i < dep->deps.size(); i++ )
+  for( unsigned int i = 0; i < dep->deps.size(); i++ )
     add_anonymous_types( dep->deps[ i ] );
 }
