@@ -42,7 +42,7 @@ struct Dep {
   }
   CORBA::IRObject_var obj;
   bool                iface_as_forward;
-  vector<Dep*>        deps;
+  std::vector<Dep*>        deps;
   bool                visited;
 };
 
@@ -57,7 +57,7 @@ struct IRObj {
   }
   
   CORBA::IRObject_var obj;
-  string              scope;
+  std::string              scope;
   bool                iface_as_forward;
 };
 
@@ -69,13 +69,13 @@ typedef SequenceTmpl<IRObj_var,MICO_TID_DEF> IRObjSeq;
 
 class IDLDep {
 private:
-  struct objcomp : binary_function<CORBA::Object *, CORBA::Object *, bool> {
+  struct objcomp : std::binary_function<CORBA::Object *, CORBA::Object *, bool> {
     bool operator() (CORBA::Object *o1, CORBA::Object *o2) const
     {
       return *o1->_ior() < *o2->_ior();
     }
   };
-  typedef map<CORBA::Object *, Dep *, objcomp> MapObjDep;
+  typedef std::map<CORBA::Object *, Dep *, objcomp> MapObjDep;
 
   Dep_var                 _top;
   Dep_var                 _bottom;
@@ -113,7 +113,7 @@ private:
   void dep_Component (CORBA::ComponentIR::ComponentDef_ptr);
   void dep_Provides (CORBA::ComponentIR::ProvidesDef_ptr);
   void dep_Uses (CORBA::ComponentIR::UsesDef_ptr);
-  void dep_Event (CORBA::ComponentIR::EventDef_ptr);
+  void dep_EventPort (CORBA::ComponentIR::EventPortDef_ptr);
   void dep_Home (CORBA::ComponentIR::HomeDef_ptr);
 #endif
   
