@@ -20,6 +20,15 @@
  *  http://www.mico.org/
  */
 
+#ifdef FAST_PCH
+#include "idl_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #include <CORBA.h>
 #include <mico/throw.h>
 #ifdef USE_CCM
@@ -29,19 +38,30 @@
 #endif
 #include <mico/ir_creator.h>
 #include <mico/template_impl.h>
+#include <mico/os-misc.h>
 #include "idlparser.h"
 #include "error.h"
 
+#endif // FAST_PCH
+
+
+using namespace std;
+
+CORBA::Container_ptr
+IRCopier (DB & db, IDLParam & params,
+	  CORBA::Repository_ptr repo,
+	  CORBA::Container_ptr cont);
 
 
 //-- IDLParser ----------------------------------------------------------
 
-IDLParser::IDLParser( DB& db, bool feed_included_defs )
+IDLParser::IDLParser( DB& db, IDLParam & params )
 {
   _current_parse_node = NULL;
-  _feed_included_defs = feed_included_defs;
+  _feed_included_defs = params.feed_included_defs;
   _repo               = CORBA::Repository::_nil ();
   _db                 = &db;
+  _params             = &params;
   _version            = "1.0";
 }
 
@@ -57,7 +77,12 @@ IDLParser::collect (CORBA::Container_ptr cont, ParseNode * root)
     _repo = temp->containing_repository ();
   }
 
-  //  _root->print ();
+  // root->print ();
+
+  /*
+   * Traverse the parse tree
+   */
+
 #ifdef HAVE_EXCEPTIONS
   try {
 #endif
@@ -128,6 +153,7 @@ IDLParser::collect (CORBA::Container_ptr cont, ParseNode * root)
     default:
       cerr << IDLError( 31, node ) << endl;
       ex._print (cerr);
+      cerr << endl;
       break;
     }
     exit( 1 );
@@ -238,6 +264,9 @@ void IDLParser::scan( CORBA::Container_ptr con, ParseNode *node )
   case t_include:
     collectInclude( con, node );
     break;
+  case t_import:
+    collectImport (con, node);
+    break;
   case exports:
   case definitions:
     scan( con, node->getBranch( 1 ) );
@@ -267,7 +296,9 @@ void IDLParser::scan( CORBA::Container_ptr con, ParseNode *node )
   case local_forward_dcl:
   case struct_forward_decl:
   case union_forward_decl:
+#ifdef USE_CCM
   case component_forward_dcl:
+#endif
     collectForwardDcl (con, node);
     break;
   case interface_dcl:
@@ -294,19 +325,23 @@ void IDLParser::scan( CORBA::Container_ptr con, ParseNode *node )
       break;
     }
   case value_forward_dcl:
-    collectValueForwardDcl( con, node, FALSE );
-    break;
   case abstract_value_forward_dcl:
-    collectValueForwardDcl( con, node, TRUE );
+#ifdef USE_CCM
+  case event_forward_dcl:
+  case abstract_event_forward_dcl:
+#endif
+    collectValueForwardDcl( con, node );
     break;
   case value_box_dcl:
     collectValueBoxDcl( con, node );
     break;
   case value_dcl:
-    collectValueDcl( con, node, FALSE );
-    break;
   case abstract_value_dcl:
-    collectValueDcl( con, node, TRUE );
+#ifdef USE_CCM
+  case event_dcl:
+  case abstract_event_dcl:
+#endif
+    collectValueDcl( con, node );
     break;
   case public_state_member:
     collectStateMember (con, node, CORBA::PUBLIC_MEMBER);
@@ -328,7 +363,7 @@ void IDLParser::scan( CORBA::Container_ptr con, ParseNode *node )
   case emits_dcl:
   case publishes_dcl:
   case consumes_dcl:
-    collectEvent (con, node);
+    collectEventPort (con, node);
     break;
   case home_dcl:
     collectHome (con, node);
@@ -344,6 +379,211 @@ void IDLParser::scan( CORBA::Container_ptr con, ParseNode *node )
   }
 }
 
+/*
+ * Process import statement:
+ *    import "Repository Id";
+ *    import <scoped-name>
+ * Imports are always at the beginning of a file, outside any scope.
+ */
+
+void
+IDLParser::collectImport (CORBA::Container_ptr con, ParseNode *node)
+{
+  assert (node);
+  assert (node->getType() == t_import);
+
+  node = node->getBranch (1);
+
+  assert (node);
+  assert (node->getType() == string_literals ||
+	  node->getType() == scoped_name);
+
+  CORBA::String_var to_be_imported = CORBA::string_dup (node->getIdent ());
+
+  /*
+   * Import statements must be at toplevel. Well, actually, import
+   * statements can appear only at the beginning of a file, but
+   * adhering to that would screw up the grammar with respect to
+   * includes.
+   */
+
+  if (con->def_kind() != CORBA::dk_Repository) {
+    cerr << IDLError (40, node) << endl;
+    exit (1);
+  }
+
+  /*
+   * We may need the Interface Repository
+   */
+
+  CORBA::ORB_var myorb = CORBA::ORB_instance ("mico-local-orb");
+  CORBA::Object_var myobj =
+    myorb->resolve_initial_references ("InterfaceRepository");
+  CORBA::Repository_var myrepo = CORBA::Repository::_narrow (myobj);
+
+  /*
+   * Try to look up Repository Id or scoped name in the Repository
+   */
+
+  CORBA::Contained_var icont = CORBA::Contained::_nil ();
+
+  if (node->getType() == string_literals) {
+    if (CORBA::is_nil (myrepo)) {
+      cerr << IDLError (36, node, to_be_imported.in()) << endl;
+      exit (1);
+    }
+
+    icont = myrepo->lookup_id (to_be_imported.in());
+
+    if (CORBA::is_nil (icont)) {
+      cerr << IDLError (37, node, to_be_imported.in()) << endl;
+      exit (1);
+    }
+  }
+  else {
+    /*
+     * We try to look up the named scope in the Repository, too.
+     * If this fails, we go on searching for a matching file, see
+     * below.
+     */
+    if (!CORBA::is_nil (myrepo)) {
+      icont = myrepo->lookup (to_be_imported.in());
+    }
+  }
+
+  /*
+   * If the import was found in the Interface Repository, employ the
+   * IR copier to pull it in.
+   */
+
+  if (!CORBA::is_nil (icont)) {
+    CORBA::Container_var import_container = CORBA::Container::_narrow (icont);
+
+    if (CORBA::is_nil (import_container)) {
+      cerr << IDLError (39, node, to_be_imported.in()) << endl;
+      exit (1);
+    }
+
+    CORBA::Container_var imported =
+      IRCopier (*_db, *_params, _repo, import_container);
+    assert (!CORBA::is_nil (imported));
+
+    return;
+  }
+
+  /*
+   * Okay, to_be_imported is a scoped name. Translate this to a file name
+   * by replacing '::' with '/'.
+   */
+
+  string file_name = to_be_imported.in();
+
+  if (file_name[0] == ':' && file_name[1] == ':') {
+    file_name = file_name.substr (2);
+  }
+
+  int pos = file_name.find ("::");
+
+  while (pos != -1) {
+    file_name = file_name.replace (pos, 2, "/");
+    pos = file_name.find ("::");
+  }
+
+  file_name += ".idl";
+
+  /*
+   * Now traverse our include paths and see if we can find this file.
+   */
+
+  string absolute_file_name;
+  vector<string> tested_paths;
+  vector<string>::size_type i;
+
+  for (i=0; i<_params->inc_paths.size(); i++) {
+    absolute_file_name = _params->inc_paths[i];
+    if (absolute_file_name.length() > 0 &&
+	absolute_file_name[absolute_file_name.length()-1] != '/') {
+      absolute_file_name += '/';
+    }
+
+    absolute_file_name += file_name;
+    tested_paths.push_back (absolute_file_name);
+
+    if (!OSMisc::access (absolute_file_name.c_str(), OSMisc::ACCESS_READ)) {
+      break;
+    }
+  }
+
+  if (i == _params->inc_paths.size()) {
+    cerr << IDLError (38, node, to_be_imported.in()) << endl;
+    cerr << "The following locations were tried:" << endl;
+    for (i=0; i<tested_paths.size(); i++) {
+      cerr << "  " << tested_paths[i] << endl;
+    }
+    exit (1);
+  }
+
+  /*
+   * File was found. Load it.
+   */
+
+  // cout << "loading " << absolute_file_name << endl;
+
+  /*
+   * Step 1: Invoke the Preprocessor
+   */
+#ifndef _WIN32
+  string cmd = "mico-cpp -D __MICO_IDL__ ";
+#else // _WIN32
+  string cmd = "mico-cpp.exe -D __MICO_IDL__ ";
+  char sz_path[256] = "";
+  GetModuleFileName(0, sz_path, 255);
+  string p(sz_path);
+  p.erase(p.find_last_of("\\") + 1, p.length());
+  cmd = "\"" + p + cmd + "\"";
+#endif // _WIN32
+
+  cmd += _params->cpp_options;
+  cmd += " -I";
+  cmd += ABSEXECDIR;
+  cmd += "/include ";
+  cmd += absolute_file_name;
+
+  FILE * inp_file = OSMisc::popen (cmd.c_str(), "r");
+
+  if (inp_file == NULL) {
+    cerr << "error: cannot open " << absolute_file_name << endl;
+    exit (1);
+  }
+
+  int dummy_char = fgetc (inp_file);
+
+  if (dummy_char == EOF) {
+    cerr << "error: cannot execute mico-cpp for import: " << cmd << endl;
+    exit( 1 );
+  }
+
+  ungetc (dummy_char, inp_file);
+  
+  /*
+   * Step 2: Invoke the Parser (bison)
+   */
+    
+  Parser parser (inp_file, file_name.c_str());
+  parser.parse();
+  
+  OSMisc::pclose (inp_file);
+
+  /*
+   * Step 3: Traverse the Parse Tree and load Interface Repository
+   */
+  
+  IDLParser idlParser (*_db, *_params);
+  idlParser.collect (_repo, parser.getRootNode());
+  _db->set_repoids (_repo);
+  return;
+}
+
 /*
  * responsible for #pragma, typeId and typePrefix
  */
@@ -437,6 +677,10 @@ IDLParser::collectPragma( CORBA::Container_ptr con, ParseNode *node )
     }
   }
 
+  // Strip leading '_'
+  if (name.length() > 0 && name[0] == '_')
+      name = name.substr (1);
+  
   if (name.length() > 0 && name[0] != ':') {
     // Name is relative, make it absolute
     CORBA::Contained_var in = con->lookup (name.c_str());
@@ -508,7 +752,7 @@ void IDLParser::collectInclude( CORBA::Container_ptr con, ParseNode *node )
   j = s.find( '"', ++i );
   assert( j >= i );
 
-  if (s.length() > j+2) {
+  if (s.length() > (unsigned int)j+2) {
     if (s[j+2] == '1')
       _db->enter_include_file();
 
@@ -658,7 +902,7 @@ void IDLParser::collectConstDcl( CORBA::Container_ptr con, ParseNode *node )
       string scoped_name = collectScopedName (node->getBranch(2));
       CORBA::ULong i = scoped_name.find_last_of (":");
 
-      if (i != -1) {
+      if (i != string::npos) {
 	scoped_name = scoped_name.substr (i+1);
       }
 
@@ -685,11 +929,37 @@ void IDLParser::collectConstDcl( CORBA::Container_ptr con, ParseNode *node )
   }
 
   CORBA::String_var repo_id = gen_repo_id( node );
-  CORBA::ConstantDef_var c = con->create_constant( repo_id,
-						   node->getIdent(),
-						   _version,
-						   type,
-						   val );
+  CORBA::ConstantDef_var c;
+
+  if( node->is_toplevel() || _feed_included_defs ) {
+    CORBA::ContainedSeq_var cs = con->lookup_name (node->getIdent(), 1,
+						   CORBA::dk_all, 1);
+    assert (cs->length() <= 1);
+
+    if (cs->length() > 0) {
+      c = CORBA::ConstantDef::_narrow (cs[(MICO_ULong)0]);
+
+      if ( CORBA::is_nil( c ) ) {
+	cerr << IDLError( 23, node, node->getIdent() ) << endl;
+	exit( 1 );
+      }
+      
+      if ( strcmp( c->id(), repo_id ) != 0 ) {
+	cerr << IDLError( 33, node, repo_id.in() ) << endl;
+	exit( 1 );
+      }
+
+      c->type_def (type);
+      c->value (val);
+    }
+    else {
+      c = con->create_constant (repo_id,
+				node->getIdent(),
+				_version,
+				type,
+				val);
+    }
+  }
 }
 
 /*
@@ -741,10 +1011,34 @@ void IDLParser::collectException( CORBA::Container_ptr con, ParseNode *node )
   _db->enter_scope( name );
   
   CORBA::StructMemberSeq members;
-  CORBA::ExceptionDef_var e = con->create_exception( repo_id,
-						     node->getIdent(),
-						     _version,
-						     members );
+  CORBA::ExceptionDef_var e;
+
+  if( node->is_toplevel() || _feed_included_defs ) {
+    CORBA::ContainedSeq_var cs = con->lookup_name (node->getIdent(), 1,
+						   CORBA::dk_all, 1);
+    assert (cs->length() <= 1);
+
+    if (cs->length() > 0) {
+      e = CORBA::ExceptionDef::_narrow (cs[(MICO_ULong)0]);
+
+      if ( CORBA::is_nil( e ) ) {
+	cerr << IDLError( 23, node, node->getIdent() ) << endl;
+	exit( 1 );
+      }
+      
+      if ( strcmp( e->id(), repo_id ) != 0 ) {
+	cerr << IDLError( 33, node, repo_id.in() ) << endl;
+	exit( 1 );
+      }
+    }
+    else {
+      e = con->create_exception( repo_id,
+				 node->getIdent(),
+				 _version,
+				 members );
+    }
+  }
+
   collectStructMembers( e, node->getBranch( 1 ), members );
   e->members( members );
   current_parse_node( node );
@@ -1464,6 +1758,18 @@ IDLParser::collectInterface( CORBA::Container_ptr con, ParseNode *node )
   header = node->getBranch( 1 );
   repo_id = gen_repo_id (header);
   in->id (repo_id);
+
+#ifdef USE_CCM
+  /*
+   * make implicit CCM_<name> interface, so that user code can derive
+   * from it. We don't care about its contents; the interface will be
+   * re-generated by the IDL 3 to IDL 2 stage.
+   */
+
+  if (in->def_kind() == CORBA::dk_Interface) {
+    make_ccm_interface (in);
+  }
+#endif
 }
 
 void
@@ -1630,9 +1936,6 @@ IDLParser::collectAttribute( CORBA::Container_ptr con, ParseNode *node,
   ParseNode * p = node->getBranch (2);
   ParseNode * exceps = p->getBranch (3);
 
-  if (exceps && node->getType() == t_attribute) {
-  }
-
   do {
     assert( p->getType() == simple_declarators );
     char *name = p->getBranch( 1 )->getIdent();
@@ -1654,19 +1957,41 @@ IDLParser::collectAttribute( CORBA::Container_ptr con, ParseNode *node,
 				    mode );
     }
 
+    if (exceps) {
+      CORBA::ExceptionDefSeq get_exceps, set_exceps;
+      collectRaisesExpr (con, exceps->getBranch (1), get_exceps);
+      collectRaisesExpr (con, exceps->getBranch (2), set_exceps);
+
+      CORBA::ExtAttributeDef_var ead = CORBA::ExtAttributeDef::_narrow (attr);
+      assert (!CORBA::is_nil (ead));
+
+      ead->get_exceptions (get_exceps);
+      ead->set_exceptions (set_exceps);
+    }
+
     p = p->getBranch( 2 );
   } while( p != NULL );
 }
 
 void
-IDLParser::collectValueForwardDcl( CORBA::Container_ptr con, ParseNode *node,
-				   CORBA::Boolean is_abstract )
+IDLParser::collectValueForwardDcl (CORBA::Container_ptr con, ParseNode *node)
 {
-  assert( node->getType() == value_forward_dcl ||
-	  node->getType() == abstract_value_forward_dcl );
+  assert (node->getType() == value_forward_dcl ||
+	  node->getType() == abstract_value_forward_dcl ||
+	  node->getType() == event_forward_dcl ||
+	  node->getType() == abstract_event_forward_dcl);
   
   CORBA::String_var scoped_name = gen_scoped_name( node );
   CORBA::String_var repo_id = gen_repo_id( node );
+  CORBA::Boolean is_abstract;
+
+  if (node->getType() == value_forward_dcl ||
+      node->getType() == event_forward_dcl) {
+    is_abstract = 0;
+  }
+  else {
+    is_abstract = 1;
+  }
 
   if( is_iface_defined( scoped_name ) )
     return;
@@ -1678,17 +2003,39 @@ IDLParser::collectValueForwardDcl( CORBA::Container_ptr con, ParseNode *node,
       CORBA::ValueDefSeq abstract_base_values; // Leave empty for now
       CORBA::InterfaceDefSeq supported_interfaces; // dito
       CORBA::InitializerSeq initializers;
-  
-      in = con->create_value( repo_id,
-			      node->getIdent(),
-			      _version,
-			      FALSE,
-			      is_abstract,
-			      CORBA::ValueDef::_nil(),
-			      FALSE,
-			      abstract_base_values,
-			      supported_interfaces,
-			      initializers );
+
+      if (node->getType() == value_forward_dcl ||
+	  node->getType() == abstract_value_forward_dcl) {
+	in = con->create_value( repo_id,
+				node->getIdent(),
+				_version,
+				FALSE,
+				is_abstract,
+				CORBA::ValueDef::_nil(),
+				FALSE,
+				abstract_base_values,
+				supported_interfaces,
+				initializers );
+      }
+      else {
+#ifdef USE_CCM
+	CORBA::ComponentIR::Container_var ccm_con =
+	  CORBA::ComponentIR::Container::_narrow (con);
+	CORBA::ExtInitializerSeq ext_initializers;
+	assert (!CORBA::is_nil (ccm_con));
+
+	in = ccm_con->create_event( repo_id,
+				    node->getIdent(),
+				    _version,
+				    FALSE,
+				    is_abstract,
+				    CORBA::ValueDef::_nil(),
+				    FALSE,
+				    abstract_base_values,
+				    supported_interfaces,
+				    ext_initializers );
+#endif
+      }
     } else {
       /*
        * Since we don't feed included defs, we only have to make
@@ -1733,11 +2080,12 @@ IDLParser::collectValueBoxDcl( CORBA::Container_ptr con, ParseNode *node )
 }
 
 void
-IDLParser::collectValueDcl( CORBA::Container_ptr con, ParseNode *node,
-			    CORBA::Boolean is_abstract )
+IDLParser::collectValueDcl( CORBA::Container_ptr con, ParseNode *node )
 {
   assert( node->getType() == value_dcl ||
-	  node->getType() == abstract_value_dcl );
+	  node->getType() == abstract_value_dcl ||
+	  node->getType() == event_dcl ||
+	  node->getType() == abstract_event_dcl );
 
   ParseNode * header = node->getBranch (1);
 
@@ -1758,6 +2106,7 @@ IDLParser::collectValueDcl( CORBA::Container_ptr con, ParseNode *node,
   CORBA::InitializerSeq initializers;
   CORBA::Boolean is_custom;
   CORBA::Boolean is_truncatable;
+  CORBA::Boolean is_abstract;
 
   if (header->getType() == custom_value_header) {
     is_custom = TRUE;
@@ -1766,6 +2115,13 @@ IDLParser::collectValueDcl( CORBA::Container_ptr con, ParseNode *node,
     is_custom = FALSE;
   }
 
+  if (node->getType() == value_dcl || node->getType() == event_dcl) {
+    is_abstract = 0;
+  }
+  else {
+    is_abstract = 1;
+  }
+
   if( node->is_toplevel() || _feed_included_defs ) {
     CORBA::ContainedSeq_var cs = con->lookup_name (header->getIdent(), 1,
 						   CORBA::dk_all, 1);
@@ -1797,16 +2153,38 @@ IDLParser::collectValueDcl( CORBA::Container_ptr con, ParseNode *node,
 	    exit( 1 );
 	}
     } else {
-	val = con->create_value( repo_id,
-				 header->getIdent(),
-				 _version,
-				 FALSE,
-				 is_abstract,
-				 CORBA::ValueDef::_nil(),
-				 FALSE,
-				 abstract_base_values,
-				 supported_interfaces,
-				 initializers );
+        if (node->getType() == value_dcl ||
+	    node->getType() == abstract_value_dcl) {
+	  val = con->create_value( repo_id,
+				   header->getIdent(),
+				   _version,
+				   FALSE,
+				   is_abstract,
+				   CORBA::ValueDef::_nil(),
+				   FALSE,
+				   abstract_base_values,
+				   supported_interfaces,
+				   initializers );
+	}
+	else {
+#ifdef USE_CCM
+	  CORBA::ComponentIR::Container_var ccm_con =
+	    CORBA::ComponentIR::Container::_narrow (con);
+	  CORBA::ExtInitializerSeq ext_initializers;
+	  assert (!CORBA::is_nil (ccm_con));
+
+	  val = ccm_con->create_event( repo_id,
+				       header->getIdent(),
+				       _version,
+				       FALSE,
+				       is_abstract,
+				       CORBA::ValueDef::_nil(),
+				       FALSE,
+				       abstract_base_values,
+				       supported_interfaces,
+				       ext_initializers );
+#endif
+	}
     }
   } else {
     /*
@@ -2306,7 +2684,7 @@ Const IDLParser::collectConst( CORBA::Container_ptr con, ParseNode *p )
   case t_fixed_pt_literal:
     return Const( CORBA::_tc_longdouble, p->getFixedPtLiteral(), p );
   case t_integer_literal:
-    return Const( CORBA::_tc_ulonglong, p->getIntegerLiteral(), p );
+    return Const( CORBA::_tc_longlong, p->getIntegerLiteral(), p );
   case t_string_literal:
     return Const( CORBA::_tc_string, p->getStringLiteral(), p );
   case t_wstring_literal:
@@ -2418,12 +2796,11 @@ IDLParser::collectComponent (CORBA::Container_ptr con, ParseNode * node)
       CORBA::ComponentIR::Container_var ccv =
 	CORBA::ComponentIR::Container::_narrow (con);
       assert (!CORBA::is_nil (ccv));
-      CORBA::InterfaceDefSeq supported_interfaces;
       comp = ccv->create_component (repo_id,
 				    header->getIdent(),
 				    _version,
 				    CORBA::ComponentIR::ComponentDef::_nil(),
-				    supported_interfaces);
+				    CORBA::InterfaceDefSeq (0));
     }
   } else {
     /*
@@ -2506,6 +2883,14 @@ IDLParser::collectComponent (CORBA::Container_ptr con, ParseNode * node)
   // regenerate repo id in case it was redefined with a typeprefix
   repo_id = gen_repo_id (header);
   comp->id (repo_id);
+
+  /*
+   * make implicit CCM_<name> interface, so that user code can derive
+   * from it. We don't care about its contents; the interface will be
+   * re-generated by the IDL 3 to IDL 2 stage.
+   */
+
+  make_ccm_interface (comp);
 }
 
 void
@@ -2573,7 +2958,7 @@ IDLParser::collectUses (CORBA::Container_ptr con, ParseNode *node)
 }
 
 void
-IDLParser::collectEvent (CORBA::Container_ptr con, ParseNode *node)
+IDLParser::collectEventPort (CORBA::Container_ptr con, ParseNode *node)
 {
   assert (node->getType() == emits_dcl ||
 	  node->getType() == publishes_dcl ||
@@ -2585,7 +2970,8 @@ IDLParser::collectEvent (CORBA::Container_ptr con, ParseNode *node)
 
   CORBA::Contained_var c =
     con->lookup (collectScopedName(node->getBranch(1)));
-  CORBA::ValueDef_var event = CORBA::ValueDef::_narrow (c);
+  CORBA::ComponentIR::EventDef_var event =
+    CORBA::ComponentIR::EventDef::_narrow (c);
 
   if (CORBA::is_nil (event)) {
     cerr << IDLError (2, node->getBranch(1), node->getBranch(1)->getIdent())
@@ -2596,7 +2982,7 @@ IDLParser::collectEvent (CORBA::Container_ptr con, ParseNode *node)
   char * name = node->getIdent ();
   CORBA::String_var repo_id = gen_repo_id (node);
 
-  CORBA::ComponentIR::EventDef_var e;
+  CORBA::ComponentIR::EventPortDef_var e;
 
   switch (node->getType()) {
   case emits_dcl:
@@ -2656,12 +3042,14 @@ IDLParser::collectHome (CORBA::Container_ptr con, ParseNode * node)
     } else {
       CORBA::ComponentIR::Container_var ccv =
 	CORBA::ComponentIR::Container::_narrow (con);
+      CORBA::InterfaceDefSeq supported_interfaces;
       assert (!CORBA::is_nil (ccv));
       home = ccv->create_home (repo_id,
 			       header->getIdent(),
 			       _version,
 			       CORBA::ComponentIR::HomeDef::_nil(),
 			       CORBA::ComponentIR::ComponentDef::_nil(),
+			       CORBA::InterfaceDefSeq (0),
 			       CORBA::ValueDef::_nil());
     }
   } else {
@@ -2746,12 +3134,46 @@ IDLParser::collectHome (CORBA::Container_ptr con, ParseNode * node)
     home->primary_key (primary_key);
   }
 
+  if (header->getBranch (4)) {
+    CORBA::InterfaceDefSeq supported_interfaces;
+    ParseNode * supports = header->getBranch (4);
+    CORBA::ULong length = 0;
+
+    while (supports != 0) {
+      ParseNode* node_base = supports->getBranch( 1 );
+      CORBA::IDLType_var t = collectType( con, node_base );
+      CORBA::InterfaceDef_var supported = CORBA::InterfaceDef::_narrow (t);
+      if (CORBA::is_nil (supported)) {
+	cerr << IDLError( 8, node_base, node_base->getIdent() ) << endl;
+	exit (1);
+      }
+      scoped_name = supported->absolute_name();
+      if (_db->is_in_forward_dcl (scoped_name)) {
+	cerr << IDLError (9, node_base, node_base->getIdent()) << endl;
+	exit (1);
+      }
+      supported_interfaces.length (length+1);
+      supported_interfaces[length++] = supported;
+      supports = supports->getBranch (2);
+    }
+    
+    home->supported_interfaces (supported_interfaces);
+  }
+
   scan (home, node->getBranch (2));
   _db->leave_scope();
 
   // regenerate repo id in case it was redefined with a typeprefix
   repo_id = gen_repo_id (header);
   home->id (repo_id);
+
+  /*
+   * make implicit CCM_<name> interface, so that user code can derive
+   * from it. We don't care about its contents; the interface will be
+   * re-generated by the IDL 3 to IDL 2 stage.
+   */
+
+  make_ccm_interface (home);
 }
 
 void
@@ -2802,4 +3224,54 @@ IDLParser::collectFactory (CORBA::Container_ptr con, ParseNode *node)
   }
 }
 
+/*
+ * Make a placeholder for the CCM_<name> equivalent interface that is
+ * generated for components and homes, and mark this interface as im-
+ * plicit, so that no code gets generated for it.
+ *
+ * This is done so that the user can write IDL interfaces that inherit
+ * from the equivalent interface.
+ *
+ * The CCM Transformator will then generate the "real" equivalent
+ * interfaces and ignore the one that is generated here.
+ *
+ * We make the implicit interface inherit from the component or home
+ * itself. This is ugly, but causes the dependency generator to work
+ * correctly.
+ */
+
+void
+IDLParser::make_ccm_interface (CORBA::InterfaceDef_ptr iface)
+{
+  CORBA::Container_var co = iface->defined_in ();
+
+  CORBA::String_var id = iface->id ();
+  CORBA::String_var name = iface->name ();
+  CORBA::String_var version = iface->version ();
+
+  CORBA::InterfaceDef_var dummyres;
+  CORBA::InterfaceDefSeq dummybases;
+  dummybases.length (1);
+  dummybases[0] = CORBA::InterfaceDef::_duplicate (iface);
+
+  string newid (id.in());
+  newid.insert (0, "CCM_");
+
+  string newname (name.in());
+  newname.insert (0, "CCM_");
+
+  dummyres = co->create_local_interface (newid.c_str(), newname.c_str(),
+					 version.in(), dummybases);
+
+  string oldid (id.in());
+  if (_db->is_repoid_defined (oldid)) {
+    string fname = _db->get_fname_for_repoid (oldid);
+    _db->register_repoid (newid, fname);
+  }
+
+  CORBA::String_var an = dummyres->absolute_name ();
+  _db->add_implicit (an.in());
+}
+
 #endif
+
