@@ -6,6 +6,15 @@
  *  The MICO CORBA Component Project was sponsored by Alcatel.
  */
 
+#ifdef FAST_PCH
+#include "idl_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #include <CORBA.h>
 #ifdef USE_CCM
 #include <mico/ir3.h>
@@ -20,6 +29,11 @@
 #include "params.h"
 #include "idlparser.h"
 
+#endif // FAST_PCH
+
+
+using namespace std;
+
 /*
  * IDL 3 to IDL 2 Transformer
  */
@@ -51,6 +65,7 @@ private:
   CORBA::ValueMemberDef_ptr copy_ValueMember (CORBA::ValueMemberDef_ptr src);
   CORBA::ValueDef_ptr       copy_Value       (CORBA::ValueDef_ptr src);
   CORBA::ValueBoxDef_ptr    copy_ValueBox    (CORBA::ValueBoxDef_ptr src);
+  CORBA::NativeDef_ptr      copy_Native      (CORBA::NativeDef_ptr src);
 
 #ifdef USE_CCM
   CORBA::ComponentIR::ProvidesDef_ptr
@@ -108,7 +123,7 @@ IRCopy::is_included_def (CORBA::Contained_ptr src)
   CORBA::String_var id = src->id ();
   string strid = id.in();
 
-  if (!_db.repoid_defined (strid)) {
+  if (!_db.is_repoid_defined (strid)) {
     return true;
   }
 
@@ -148,6 +163,12 @@ IRCopy::copy (CORBA::Container_ptr src,
   CORBA::Container_var dummy2;
 
   for (CORBA::ULong i=0; i<contents->length(); i++) {
+    CORBA::String_var absname = contents[i]->absolute_name ();
+
+    if (_db.is_implicit (absname)) {
+      continue;
+    }
+
     if (!is_included_def (contents[i]) || _feed_included_defs ||
 	contents[i]->def_kind() == CORBA::dk_Module) {
       dummy1 = copy_Contained (contents[i]);
@@ -269,7 +290,8 @@ IRCopy::copy_Contained (CORBA::Contained_ptr src)
     break;
   }
 
-  case CORBA::dk_Value: {
+  case CORBA::dk_Value: 
+  case CORBA::dk_Event: {
     CORBA::ValueDef_var el = CORBA::ValueDef::_narrow (src);
     res = copy_Value (el);
     break;
@@ -281,6 +303,12 @@ IRCopy::copy_Contained (CORBA::Contained_ptr src)
     break;
   }
 
+  case CORBA::dk_Native: {
+    CORBA::NativeDef_var el = CORBA::NativeDef::_narrow (src);
+    res = copy_Native (el);
+    break;
+  }
+
 #ifdef USE_CCM
   case CORBA::dk_Provides: {
     CORBA::ComponentIR::ProvidesDef_var el =
@@ -347,6 +375,7 @@ IRCopy::copy_Contained (CORBA::Contained_ptr src)
 #endif
 
   default:
+    cout << src->def_kind() << endl;
     assert (0);
     break;
   }
@@ -662,6 +691,30 @@ IRCopy::copy_Alias (CORBA::AliasDef_ptr src)
   return res;
 }
 
+CORBA::NativeDef_ptr
+IRCopy::copy_Native (CORBA::NativeDef_ptr src)
+{
+  CORBA::String_var id = src->id ();
+  CORBA::String_var name = src->name ();
+  CORBA::String_var version = src->version ();
+
+  CORBA::Contained_var existing = _repo->lookup_id (id);
+  CORBA::NativeDef_ptr res = CORBA::NativeDef::_narrow (existing);
+
+  CORBA::Container_var defin = src->defined_in ();
+  CORBA::Container_var dest = copy_Container (defin);
+
+  if (CORBA::is_nil (dest)) {
+    return CORBA::NativeDef::_nil ();
+  }
+
+  if (CORBA::is_nil (res)) {
+    res = dest->create_native (id, name, version);
+  }
+
+  return res;
+}
+
 CORBA::ExceptionDef_ptr
 IRCopy::copy_Exception (CORBA::ExceptionDef_ptr src)
 {
@@ -744,6 +797,28 @@ IRCopy::copy_Attribute (CORBA::AttributeDef_ptr src)
     res->mode (src->mode());
   }
 
+  CORBA::ExtAttributeDef_var ead = CORBA::ExtAttributeDef::_narrow (src);
+  CORBA::ExtAttributeDef_var tead = CORBA::ExtAttributeDef::_narrow (res);
+
+  if (!CORBA::is_nil (ead) && !CORBA::is_nil (tead)) {
+    CORBA::ExceptionDefSeq_var sge = ead->get_exceptions ();
+    CORBA::ExceptionDefSeq_var sse = ead->set_exceptions ();
+    CORBA::ExceptionDefSeq tge, tse;
+    tge.length (sge->length());
+    tse.length (sse->length());
+
+    for (CORBA::ULong i1=0; i1<sge->length(); i1++) {
+      tge[i1] = copy_Exception (sge[i1]);
+    }
+
+    for (CORBA::ULong i2=0; i2<sse->length(); i2++) {
+      tse[i2] = copy_Exception (sse[i2]);
+    }
+
+    tead->get_exceptions (tge);
+    tead->set_exceptions (tse);
+  }
+
   return res;
 }
 
@@ -818,7 +893,7 @@ IRCopy::copy_Operation (CORBA::OperationDef_ptr src)
     res->result_def (tt);
     res->params (tp);
     res->mode (sm);
-    res->exceptions (se);
+    res->exceptions (te);
     res->contexts (sc);
   }
 
@@ -988,14 +1063,39 @@ IRCopy::copy_Value (CORBA::ValueDef_ptr src)
   CORBA::InitializerSeq ti;
 
   if (CORBA::is_nil (res)) {
-    res = dest->create_value (id, name, version,
-			      src->is_custom (),
-			      src->is_abstract (),
-			      tb,
-			      src->is_truncatable (),
-			      tab,
-			      ts,
-			      ti);
+#ifdef USE_CCM
+    if (src->def_kind() == CORBA::dk_Event) {
+      CORBA::ComponentIR::Container_var cdest =
+	CORBA::ComponentIR::Container::_narrow (dest);
+     
+      if (CORBA::is_nil (cdest)) {
+	return CORBA::ValueDef::_nil ();
+      }
+
+      CORBA::ExtInitializerSeq eti;
+
+      res = cdest->create_event (id, name, version,
+				 src->is_custom (),
+				 src->is_abstract (),
+				 tb,
+				 src->is_truncatable (),
+				 tab,
+				 ts,
+				 eti);
+    }
+    else {
+#endif
+      res = dest->create_value (id, name, version,
+				src->is_custom (),
+				src->is_abstract (),
+				tb,
+				src->is_truncatable (),
+				tab,
+				ts,
+				ti);
+#ifdef USE_CCM
+    }
+#endif
   }
   else {
     res->is_custom (src->is_custom());
@@ -1162,7 +1262,8 @@ IRCopy::copy_Emits (CORBA::ComponentIR::EmitsDef_ptr src)
 
   CORBA::IDLType_var st = src->event ();
   CORBA::IDLType_var ttmp = copy_IDLType (st);
-  CORBA::ValueDef_var tt = CORBA::ValueDef::_narrow (ttmp);
+  CORBA::ComponentIR::EventDef_var tt =
+    CORBA::ComponentIR::EventDef::_narrow (ttmp);
 
   if (CORBA::is_nil (tt)) {
     return CORBA::ComponentIR::EmitsDef::_nil ();
@@ -1200,7 +1301,8 @@ IRCopy::copy_Publishes (CORBA::ComponentIR::PublishesDef_ptr src)
 
   CORBA::IDLType_var st = src->event ();
   CORBA::IDLType_var ttmp = copy_IDLType (st);
-  CORBA::ValueDef_var tt = CORBA::ValueDef::_narrow (ttmp);
+  CORBA::ComponentIR::EventDef_var tt =
+    CORBA::ComponentIR::EventDef::_narrow (ttmp);
 
   if (CORBA::is_nil (tt)) {
     return CORBA::ComponentIR::PublishesDef::_nil ();
@@ -1238,7 +1340,8 @@ IRCopy::copy_Consumes (CORBA::ComponentIR::ConsumesDef_ptr src)
 
   CORBA::IDLType_var st = src->event ();
   CORBA::IDLType_var ttmp = copy_IDLType (st);
-  CORBA::ValueDef_var tt = CORBA::ValueDef::_narrow (ttmp);
+  CORBA::ComponentIR::EventDef_var tt =
+    CORBA::ComponentIR::EventDef::_narrow (ttmp);
 
   if (CORBA::is_nil (tt)) {
     return CORBA::ComponentIR::ConsumesDef::_nil ();
@@ -1458,6 +1561,19 @@ IRCopy::copy_Home (CORBA::ComponentIR::HomeDef_ptr src)
     }
   }
 
+  CORBA::InterfaceDefSeq_var ss = src->supported_interfaces ();
+  CORBA::InterfaceDefSeq ts;
+  ts.length (ss->length());
+
+  for (CORBA::ULong i=0; i<ss->length(); i++) {
+    CORBA::IDLType_var ttmp = copy_IDLType (ss[i]);
+    ts[i] = CORBA::InterfaceDef::_narrow (ttmp);
+
+    if (CORBA::is_nil (ts[i])) {
+      return CORBA::ComponentIR::HomeDef::_nil ();
+    }
+  }
+
   CORBA::IDLType_var sm = src->managed_component ();
   CORBA::ComponentIR::ComponentDef_var tm =
     CORBA::ComponentIR::ComponentDef::_nil ();
@@ -1484,7 +1600,7 @@ IRCopy::copy_Home (CORBA::ComponentIR::HomeDef_ptr src)
   }
 
   if (CORBA::is_nil (res)) {
-    res = cdest->create_home (id, name, version, tb, tm, tk);
+    res = cdest->create_home (id, name, version, tb, tm, ts, tk);
   }
   else {
     res->base_home (tb);
@@ -1532,6 +1648,7 @@ IRCopier (DB & db, IDLParam & params,
     res = CORBA::Container::_duplicate (repo);
   }
 
-  return res;
+  return res._retn();
 }
 
+
