@@ -21,9 +21,22 @@
  */
 
 
+#ifdef FAST_PCH
+#include "idl_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #include <CORBA.h>
 #include <stdio.h>
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <iostream>
+#else
 #include <iostream.h>
+#endif
 #include <unistd.h>
 #include <mico/ir_creator.h>
 #include <mico/util.h>
@@ -33,13 +46,14 @@
 #include "codegen-c++.h"
 #include "codegen-idl.h"
 #include "codegen-midl.h"
+#include "codegen-wsdl.h"
 #include "params.h"
 #include "db.h"
 
+#endif // FAST_PCH
 
-#ifndef CPP
-#define CPP "mico-cpp"
-#endif
+
+using namespace std;
 
 #ifdef USE_CCM
 CORBA::Container_ptr
@@ -59,11 +73,11 @@ int main( int argc, char *argv[] )
   CORBA::Container_var container;
   CORBA::Repository_var repository, c2repo;
   DB db;
-  
+
   // ORB initialization
   CORBA::ORB_var orb = CORBA::ORB_init (argc, argv);
 
-#ifdef _WINDOWS
+#ifdef _WIN32
   // Some parts of the idl-compiler can't handle backslashes in paths
   // replace all the backslashes by slashes
   // runtime library can handle slashes fine
@@ -106,7 +120,7 @@ int main( int argc, char *argv[] )
     micopath += "/include";
     params.inc_paths.push_back (micopath);
 
-    string cmd = CPP;
+    string cmd = params.cpp;
     cmd += " -D __MICO_IDL__ ";
     cmd += params.cpp_options;
     cmd += " -I";
@@ -123,7 +137,7 @@ int main( int argc, char *argv[] )
     
     int dummy_char = fgetc (inp_file);
     if (dummy_char == EOF) {
-      cerr << "error: cannot execute " << CPP << endl;
+      cerr << "error: cannot execute " << params.cpp << endl;
       exit( 1 );
     }
     ungetc (dummy_char, inp_file);
@@ -151,7 +165,7 @@ int main( int argc, char *argv[] )
      */
 
     db.set_toplevel_fname (params.file.c_str());
-    IDLParser idlParser (db);
+    IDLParser idlParser (db, params);
     idlParser.collect (repository, parser.getRootNode());
     db.set_repoids (repository);
   }
@@ -218,7 +232,7 @@ int main( int argc, char *argv[] )
   if( (int)pos >= 0 && pos < name.length() )
     name = name.substr( 0, pos );
   pos = name.rfind( "/" );
-#ifdef _WINDOWS
+#ifdef _WIN32
    if (pos==string::npos)
      pos = name.rfind( "\\" );
 #endif
@@ -255,6 +269,11 @@ int main( int argc, char *argv[] )
 	params.idl3toidl2 = true;
 	break;
       }
+      tmp = cs[pos]->contents (CORBA::dk_Event, 1);
+      if (tmp->length() > 0) {
+	params.idl3toidl2 = true;
+	break;
+      }
       tmp = cs[pos]->contents (CORBA::dk_Module, 1);
       for (CORBA::ULong ti=0; ti<tmp->length(); ti++) {
 	cs.push_back (CORBA::Container::_narrow (tmp[ti]));
@@ -272,6 +291,7 @@ int main( int argc, char *argv[] )
     c2repo = MICO::create_interface_repository (orb);
     container = CCMTransform (db, params, c2repo, container);
     assert (!CORBA::is_nil (container));
+    db.clear_implicits ();
   }
 #endif
 
@@ -296,6 +316,12 @@ int main( int argc, char *argv[] )
     gen->emit( name );
     delete gen;
   }
+  
+  if( params.codegen_wsdl ) {
+    CodeGen *gen = new CodeGenWSDL( db, params, container );
+    gen->emit( name );
+    delete gen;
+  }
 
   if (params.feed_ir) {
     CORBA::Object_var obj =
