@@ -1,6 +1,6 @@
 /*
  *  MICO --- an Open Source CORBA implementation
- *  Copyright (c) 1997-2001 by The Mico Team
+ *  Copyright (c) 1997-2003 by The Mico Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,16 +21,32 @@
  */
 
 
+#ifdef FAST_PCH
+#include "idl_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #include <CORBA.h>
 #include <stdio.h>
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <iostream>
+#else
 #include <iostream.h>
+#endif
 #include <unistd.h>
 #include <mico/ir_creator.h>
 #include <mico/util.h>
 #include "params.h"
 
+#endif // FAST_PCH
 
 
+using namespace std;
+
 IDLParam::IDLParam( int argc, char *argv[] )
 {
   _argc = argc;
@@ -52,16 +68,28 @@ IDLParam::IDLParam()
 void
 IDLParam::set_defaults()
 {
+#ifndef _WIN32
+  cpp = "mico-cpp";
+#else // _WIN32
+  cpp = "mico-cpp.exe";
+  char sz_path[256] = "";
+  GetModuleFileName(0, sz_path, 255);
+  string p(sz_path);
+  p.erase(p.find_last_of("\\") + 1, p.length());
+  cpp = "\"" + p + cpp + "\"";
+#endif // _WIN32
+
   cpp_suffix = "cc";
   hh_suffix = "h";
   relative_paths = false;
   feed_ir = false;
-  feed_included_defs = false;
+  feed_included_defs = true;
   codegen_cpp = false;
   cpp_impl = false;
   cpp_skel = false;
   codegen_idl = false;
   codegen_midl = false;
+  codegen_wsdl = false;
   emit_repoids = false;
   query_server_for_narrow = true;
 #ifdef HAVE_EXCEPTIONS
@@ -69,6 +97,9 @@ IDLParam::set_defaults()
 #else
   no_exceptions = true;
 #endif
+  missing_nscorba = true;
+  support_id = false;
+  wsi_wsdl = false;
   pseudo = false;
   boa = false;
   poa = true;
@@ -77,11 +108,21 @@ IDLParam::set_defaults()
   poa_ties = false;
   poa_stubs = true;
   windows_dll=false;
+  windows_dll_with_export = false;
   mico_core = false;
   gen_included_defs = false;
   gen_full_dispatcher = false;
   throw_decl = false;
   idl3toidl2 = false;
+  include_prefix = false;
+  i_prefix = "";
+  output_dir = "";
+  skel_suffix = "_skel";
+  dll_def_prefix = "";
+  tc_export = "extern";
+  static_tc_export = "static";
+  class_export = "";
+  operator_export = "";
 }
 
 void
@@ -98,6 +139,8 @@ IDLParam::parse_params()
   opts["--no-codegen-idl"]        = "";
   opts["--codegen-midl"]          = "";
   opts["--no-codegen-midl"]       = "";
+  opts["--codegen-wsdl"]          = "";
+  opts["--no-codegen-wsdl"]       = "";
   opts["--c++-suffix"]            = "arg-expected";
   opts["--c++-impl"]              = "";
   opts["--c++-skel"]              = "";
@@ -107,6 +150,8 @@ IDLParam::parse_params()
   opts["--do-not-query-server-for-narrow"] = "";
   opts["--repo-id"]               = "arg-expected";
   opts["--name"]                  = "arg-expected";
+  opts["--wsdl-map"]              = "arg-expected";
+  opts["--ifdef-prefix"]          = "arg-expected";
   opts["--version"]               = "";
   opts["--config"]                = "";
   opts["--help"]                  = "";
@@ -122,13 +167,24 @@ IDLParam::parse_params()
   opts["--poa-ties"]              = "";
   opts["--no-poa-ties"]           = "";
   opts["--no-poa-stubs"]          = "";
-  opts["--windows-dll"]           = "";
+  opts["--windows-dll"]           = "arg-expected";
+  opts["--windows-dll-with-export"] = "arg-expected";
   opts["--mico-core"]             = "";
   opts["--gen-included-defs"]     = "";
   opts["--gen-full-dispatcher"]   = "";
   opts["--throw-decl"]            = "";
   opts["--no-throw-decl"]         = "";
   opts["--idl3toidl2"]            = "";
+  opts["--missing-nscorba"]       = "";
+  opts["--not-missing-nscorba"]   = "";
+  opts["--support-id"]            = "";
+  opts["--dont-support-id"]       = "";
+  opts["--wsi-wsdl"]              = "";
+  opts["--no-wsi-wsdl"]           = "";
+  opts["--cpp"]                   = "arg-expected";
+  opts["--include-prefix"]        = "arg-expected";
+  opts["--output-dir"]       	  = "arg-expected";
+  opts["--skel-suffix"]       	  = "arg-expected";
   
   // Preprocessor options
   opts["-I"]                      = "arg-expected";
@@ -209,14 +265,24 @@ IDLParam::parse_params()
       codegen_idl = false;
     } else if (arg == "--codegen-midl") {
       codegen_midl = true;
+    } else if (arg == "--codegen-wsdl") {
+      codegen_wsdl = true;
     } else if (arg == "--no-codegen-midl") {
       codegen_midl = false;
+    } else if (arg == "--no-codegen-wsdl") {
+      codegen_wsdl = false;
     } else if( arg == "--c++-suffix" ) {
       cpp_suffix = val;
     } else if( arg == "--c++-impl" ) {
       cpp_impl = true;
     } else if( arg == "--c++-skel" ) {
       cpp_skel = true;
+    } else if( arg == "--output-dir" ) {
+      output_dir = val;
+      if ( !output_dir.empty() && output_dir[output_dir.length() - 1] != '/' )
+        output_dir += "/";
+    } else if( arg == "--skel-suffix" ) {
+      skel_suffix = val;
     } else if( arg == "--hh-suffix" ) {
       hh_suffix = val;
     } else if( arg == "--relative-paths" ) {
@@ -229,8 +295,22 @@ IDLParam::parse_params()
       repo_id = val;
     } else if (arg == "--name") {
       name = val;
+    } else if (arg == "--wsdl-map") {
+      wsdl_map = val;
+    } else if (arg == "--ifdef-prefix") {
+      ifdef_prefix = val;
     } else if (arg == "--boa") {
       boa = true;
+      cerr << "WARNING: BOA is long time deprecated feature which will probably be removed" << endl
+	   << "WARNING: in some of the future MICO releases! Please consider to rewrite your application" << endl
+	   << "WARNING: to use POA instead of BOA and then please remove ``--boa'' parameter" << endl
+	   << "WARNING: from the IDL compiler command-line." << endl;
+#ifdef HAVE_THREADS
+      cerr << endl
+	   << "WARNING2: Please note that MICO BOA implementation is not multi-threaded safe" << endl
+	   << "WARNING2: and so there is a big chance that your BOA-based application" << endl
+	   << "WARNING2: will not be stable." << endl << endl;
+#endif // HAVE_THREADS
     } else if (arg == "--no-boa") {
       boa = false;
     } else if (arg == "--poa") {
@@ -245,6 +325,16 @@ IDLParam::parse_params()
       any = false;
     } else if (arg == "--windows-dll") {
       windows_dll = true;
+      dll_def_prefix = "";
+      for (unsigned long i = 0; i < val.size(); i++) {
+	  dll_def_prefix += toupper(val[i]);
+      }
+    } else if (arg == "--windows-dll-with-export") {
+      windows_dll_with_export = true;
+      dll_def_prefix = "";
+      for (unsigned long i = 0; i < val.size(); i++) {
+	  dll_def_prefix += toupper(val[i]);
+      }
     } else if (arg == "--typecode") {
       typecode = true;
     } else if (arg == "--no-typecode") {
@@ -269,6 +359,23 @@ IDLParam::parse_params()
       throw_decl = false;
     } else if (arg == "--idl3toidl2") {
       idl3toidl2 = true;
+    } else if (arg == "--missing-nscorba") {
+      missing_nscorba = true;
+    } else if (arg == "--not-missing-nscorba") {
+      missing_nscorba = false;
+    } else if (arg == "--support-id") {
+      support_id = true;
+    } else if (arg == "--dont-support-id") {
+      support_id = false;
+    } else if (arg == "--wsi-wsdl") {
+      wsi_wsdl = true;
+    } else if (arg == "--no-wsi-wsdl") {
+      wsi_wsdl = false;
+    } else if( arg == "--cpp" ) {
+      cpp = val;
+    } else if (arg == "--include-prefix") {
+      include_prefix = true;
+      i_prefix = val;
     } else if (arg == "--version") {
       cout << "MICO Version " << MICO_VERSION << endl;
       exit( 0 );
@@ -299,7 +406,7 @@ IDLParam::cpp_only()
 void
 IDLParam::check_params()
 {
-  if (!codegen_idl && !codegen_midl && !codegen_cpp && !feed_ir) {
+  if (!codegen_idl && !codegen_midl && !codegen_wsdl && !codegen_cpp && !feed_ir) {
 
     if (file.length() == 0) {
       usage ();
@@ -346,6 +453,9 @@ IDLParam::check_params()
     cerr << "with --codegen-c++" << endl;
     exit( 1 );
   }
+  if( codegen_wsdl && wsdl_map == "" ) {
+    cerr << "warning: --codegen-wsdl without --wsdl-map: wsdl services not emitted" << endl;
+  }
 
   if( any )
     typecode = true;
@@ -369,41 +479,65 @@ IDLParam::usage()
   cerr << "    --no-codegen-idl" << endl;
   cerr << "    --codegen-midl" << endl;
   cerr << "    --no-codegen-midl" << endl;
+  cerr << "    --cpp <cpp>" << endl;
   cerr << "    --c++-suffix <filename-suffix>" << endl;
   cerr << "    --c++-impl" << endl;
   cerr << "    --c++-skel" << endl;
+  cerr << "    --codegen-wsdl" << endl;
+  cerr << "    --no-codegen-wsdl" << endl;
   cerr << "    --hh-suffix <filename-suffix>" << endl;
   cerr << "    --relative-paths" << endl;
   cerr << "    --emit-repoids" << endl;
   cerr << "    --do-not-query-server-for-narrow" << endl;
   cerr << "    --repo-id <repository-id>" << endl;
   cerr << "    --name <filename-prefix>" << endl;
+  cerr << "    --wsdl-map <filename>" << endl;
+  cerr << "    --ifdef-prefix <prefix>" << endl;
   cerr << "    --poa" << endl;
   cerr << "    --no-poa" << endl;
-  cerr << "    --boa" << endl;
+  cerr << "    --boa    --    (deprecated)" << endl;
   cerr << "    --no-boa" << endl;
   cerr << "    --no-poa-ties" << endl;
   cerr << "    --no-poa-stubs" << endl;
   cerr << "    --pseudo" << endl;
   cerr << "    --any" << endl;
   cerr << "    --typecode" << endl;
-  cerr << "    --windows-dll" << endl;
+  cerr << "    --windows-dll <dll-prefix>" << endl;
+  cerr << "    --windows-dll-with-export <dll-prefix>" << endl;
   cerr << "    --mico-core" << endl;
   cerr << "    --gen-included-defs" << endl;
   cerr << "    --gen-full-dispatcher" << endl;
+  cerr << "    --missing-nscorba" << endl;
+  cerr << "    --not-missing-nscorba" << endl;
+  cerr << "    --support-id" << endl;
+  cerr << "    --dont-support-id" << endl;
+  cerr << "    --wsi-wsdl" << endl;
+  cerr << "    --no-wsi-wsdl" << endl;
+  cerr << "    --include-prefix <include-prefix>" << endl;
   exit( 1 );
 }
 
 void
 IDLParam::config()
 {
-  cout << "MICO version...........: " << MICO_VERSION << endl;
+  cout << "MICO version...........: " << MICO_VERSION;
+#ifdef HAVE_THREADS
+  cout << "(MT)";
+#endif // HAVE_THREADS
+  cout << endl;
 
   cout << "supported CORBA version: ";
   string version = MICO_VERSION;
   int i = version.find_last_of( "." );
   cout << version.substr( 0, i ) << endl;
 
+  cout << "CCM support............: ";
+#ifdef USE_CCM
+  cout << "yes" << endl;
+#else // USE_CCM
+  cout << "no" << endl;
+#endif // USE_CCM
+
   cout << "exceptions.............: ";
 #ifdef HAVE_EXCEPTIONS
 #ifdef HAVE_STD_EH
@@ -415,12 +549,7 @@ IDLParam::config()
   cout << "no" << endl;
 #endif
 
-  cout << "modules are mapped to..: ";
-#ifdef HAVE_NAMESPACE
-  cout << "namespaces" << endl;
-#else
-  cout << "structs" << endl;
-#endif
+  cout << "modules are mapped to..: namespaces" << endl;
 
   cout << "STL is.................: ";
 #ifdef HAVE_MINI_STL
@@ -429,12 +558,19 @@ IDLParam::config()
   cout << "system supplied" << endl;
 #endif
 
-  cout << "SSL support............: ";
+  cout << "Security support.......: ";
 #ifdef HAVE_SSL
-  cout << "yes" << endl;
-#else
-  cout << "no" << endl;
+  cout << "TLS/SSL";
+#ifdef USE_CSIV2
+  cout << ", CSIv2";
+#endif // USE_CSIV2
+#ifdef USE_CSL2
+  cout << ", CSL2";
+#endif // USE_CSL2
+#else // HAVE_SSL
+  cout << "no";
 #endif
+  cout << endl;
 
   cout << "loadable modules.......: ";
 #ifdef HAVE_DYNAMIC
@@ -443,5 +579,36 @@ IDLParam::config()
   cout << "no" << endl;
 #endif
 
+  cout << "multi-threading........: ";
+#ifdef HAVE_THREADS
+  cout << "yes (";
+#else
+  cout << "no" << endl;
+#endif
+
+#ifdef HAVE_PTHREADS
+  cout << "Posix Draft 10 PThreads";
+#endif // HAVE_PTHREADS
+
+#ifdef HAVE_DCETHREADS
+  cout << "OSF/Draft 4 (DCE) Threads";
+#endif // HAVE_DCETHREADS
+
+#ifdef HAVE_SOLARIS_THREADS
+  cout << "Solaris Threads";
+#endif // HAVE_SOLARIS_THREADS
+
+#ifdef HAVE_PTH_THREADS
+  cout << "GNU Pth Threads";
+#endif // HAVE_PTH_THREADS
+
+#ifdef HAVE_WIN_THREADS
+  cout << "\"Windows\" Threads";
+#endif // HAVE_WIN_THREADS
+
+#ifdef HAVE_THREADS
+  cout << ")" << endl;
+#endif // HAVE_THREADS
+
   exit (0);
 }
