@@ -1,6 +1,6 @@
 /*
  *  MICO --- an Open Source CORBA implementation
- *  Copyright (c) 1997-2001 by The Mico Team
+ *  Copyright (c) 1997-2003 by The Mico Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,14 +27,17 @@
 class IDLParam 
 {
 public:
-  string         cpp_options;
-  string         name;
-  string         file;
-  string         cpp_suffix;
-  string         hh_suffix;
+  std::string         cpp;
+  std::string         cpp_options;
+  std::string         name;
+  std::string         ifdef_prefix;
+  std::string         file;
+  std::string         cpp_suffix;
+  std::string         hh_suffix;
+  std::string         wsdl_map;
   bool           relative_paths;
-  string         repo_id;
-  vector<string> inc_paths;
+  std::string         repo_id;
+  std::vector<std::string> inc_paths;
   bool           feed_ir;
   bool           feed_included_defs;
   bool           codegen_cpp;
@@ -42,9 +45,13 @@ public:
   bool           cpp_skel;
   bool           codegen_idl;
   bool           codegen_midl;
+  bool           codegen_wsdl;
   bool           emit_repoids;
   bool           query_server_for_narrow;
   bool           no_exceptions;
+  bool           missing_nscorba;
+  bool           support_id;
+  bool           wsi_wsdl;
   bool           pseudo;
   bool           boa;
   bool           poa;
@@ -53,11 +60,21 @@ public:
   bool           poa_ties;
   bool           poa_stubs;
   bool           windows_dll;
+  bool           windows_dll_with_export;
   bool           mico_core;
   bool           gen_included_defs;
   bool           gen_full_dispatcher;
   bool           throw_decl;
   bool           idl3toidl2;
+  bool           include_prefix;
+  std::string    i_prefix;
+  std::string	output_dir;
+  std::string	skel_suffix;
+  std::string   dll_def_prefix;
+  std::string   tc_export;
+  std::string   static_tc_export;
+  std::string   class_export;
+  std::string   operator_export;
   
 public:
   IDLParam( int argc, char *argv[] );
