@@ -1,6 +1,6 @@
 /*
  *  MICO --- an Open Source CORBA implementation
- *  Copyright (c) 1997-2001 by The Mico Team
+ *  Copyright (c) 1997-2003 by The Mico Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,14 +27,20 @@
 #include <assert.h>
 #endif
 
-#include <fstream.h>
-#ifdef _WINDOWS
+#if defined(_WIN32) && !defined(__MINGW32__)
 #include <mico/config-win32.h>
 #else
 #include <mico/config.h>
 #endif
 
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <fstream>
+#else
+#include <fstream.h>
+#endif
+
 #include <string>
+#include <keymap.h>
 
 #define MAX_NODE_SIZE 255
 
@@ -164,7 +170,11 @@ typedef enum
   home_dcl,
   home_header,
   factory_dcl,
-  finder_dcl
+  finder_dcl,
+  event_forward_dcl,
+  abstract_event_forward_dcl,
+  abstract_event_dcl,
+  event_dcl
 } ParseNodeType;
 
 
@@ -195,6 +205,8 @@ private:
   ParseNode *b4;
   ParseNode *b5;
 
+  static KeywordMap* _keymap;
+
   void printBranch( int indent, char *tag, bool detailed );
 
 public:
