@@ -8,7 +8,7 @@ class strbuf {
 public:
 	int gcount ()
 	{
-		NOT_IMPLEMENTED;
+		MICO_NOT_IMPLEMENTED;
 		return 0;
 	}
 };
@@ -17,27 +17,27 @@ class ifstream : public virtual istream {
 public:
 	ifstream (const char*)
 	{
-		NOT_IMPLEMENTED;
+		MICO_NOT_IMPLEMENTED;
 	}
 	int good ()
 	{
-		NOT_IMPLEMENTED;
+		MICO_NOT_IMPLEMENTED;
 		return 0;
 	}
 	int eof ()
 	{
-		NOT_IMPLEMENTED;
+		MICO_NOT_IMPLEMENTED;
 		return 0;
 	}
 	strbuf read (char*, int)
 	{
-		NOT_IMPLEMENTED;
+		MICO_NOT_IMPLEMENTED;
 		strbuf ret;
 		return ret;
 	}
 	ifstream& getline (char*, int)
 	{
-		NOT_IMPLEMENTED;
+		MICO_NOT_IMPLEMENTED;
 		return *this;
 	}
 };
@@ -53,7 +53,7 @@ class ofstream : public virtual ostream {
 public:
 	ofstream (const char*, ios::mode)
 	{
-		NOT_IMPLEMENTED;
+		MICO_NOT_IMPLEMENTED;
 	}
 };
 
