@@ -20,18 +20,30 @@
  *  http://www.mico.org/
  */
 
+#ifdef FAST_PCH
+#include "orb_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #define MICO_CONF_IMR
 
 #include <CORBA-SMALL.h>
-#ifndef _WINDOWS
+#ifndef _WIN32
 #include <string.h>
 #endif
-#include <iostream.h>
 #include <mico/os-net.h>
 #include <mico/impl.h>
 #include <mico/template_impl.h>
 #include <mico/util.h>
 
+#endif // FAST_PCH
+
+
+using namespace std;
 
 /************************* misc dtors *******************************/
 
@@ -48,13 +60,9 @@ CORBA::Address::~Address ()
 /*************************** Address ********************************/
 
 
-#ifdef HAVE_NAMESPACE
 namespace CORBA {
   vector<CORBA::AddressParser *> *Address::parsers = 0;
-};
-#else
-vector<CORBA::AddressParser *> *CORBA::Address::parsers = 0;
-#endif
+}
 
 void
 CORBA::Address::copy (Address *a)
@@ -271,6 +279,7 @@ MICO::InetAddress::resolve_ip () const
     }
 
     if (MICO::Logger::IsLogged (MICO::Logger::Warning)) {
+      MICOMT::AutoDebugLock __lock;
       MICO::Logger::Stream (MICO::Logger::Warning)
 	<< "Warning: cannot resolve hostname '" << _host
 	<< "' into an IP address." << endl;
@@ -357,15 +366,20 @@ MICO::InetAddress::proto () const
 CORBA::Transport *
 MICO::InetAddress::make_transport () const
 {
+    CORBA::Transport *ret; 
     switch (_family) {
     case STREAM:
-	return new TCPTransport;
+	ret = new TCPTransport;
+	break;
     case DGRAM:
-	return new UDPTransport;
+	ret = new UDPTransport;
+	break;
     default:
 	assert (0);
 	return 0;
     }
+    ret->open();
+    return ret;
 }
 
 CORBA::TransportServer *
@@ -726,7 +740,9 @@ MICO::UnixAddress::proto () const
 CORBA::Transport *
 MICO::UnixAddress::make_transport () const
 {
-    return new UnixTransport;
+    CORBA::Transport *ret = new UnixTransport;
+    ret->open();
+    return ret;
 }
 
 CORBA::TransportServer *
