@@ -20,19 +20,32 @@
  *  http://www.mico.org/
  */
 
+#ifdef FAST_PCH
+#include "orb_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #define MICO_CONF_IMR
 
 #include <CORBA-SMALL.h>
-#ifdef _WINDOWS
+#ifdef _WIN32
 #include <string.h>
 #endif
-#include <iostream.h>
 #include <mico/impl.h>
 #include <mico/util.h>
 #include <mico/throw.h>
 #include <mico/os-math.h>
 #include <mico/template_impl.h>
 
+#endif // FAST_PCH
+
+
+using namespace std;
+
 CORBA::Any::Any ()
 {
     thetc = _tc_void;
@@ -300,7 +313,6 @@ CORBA::Any::copy_any (Any &a, MapLL &vidmap, Boolean recurse)
     case tk_enum: {
 	ULong en;
 	return a.enum_get (en) && enum_put (en);
-	break;
     }
     case tk_string: {
 	const char *thestring;
@@ -600,7 +612,6 @@ CORBA::Any::compare_any (Any &a, MapLL &vidmap)
     case tk_enum: {
 	ULong en1, en2;
 	return a.enum_get (en1) && enum_get (en2) && en1 == en2;
-	break;
     }
     case tk_string: {
 	const char *str1, *str2;
@@ -2721,7 +2732,7 @@ public:
 	case SignedValue:
 	    return _sv;
 	case UnsignedValue:
-#ifdef _WINDOWS
+#ifdef _WIN32
 	    return (CORBA::LongLong)_uv;
 #else
 	    return _uv;
@@ -2734,11 +2745,12 @@ public:
     CORBA::Boolean signed_range (CORBA::LongLong lower,
 				 CORBA::LongLong upper) const
     {
+	assert(upper >= 0);
 	switch (_kind) {
 	case SignedValue:
 	    return lower <= _sv && _sv <= upper;
 	case UnsignedValue:
-	    return _uv <= upper;
+	    return _uv <= (CORBA::ULongLong)upper;
 	case FloatValue:
 	    return lower <= _fv && _fv <= upper;
 	}
@@ -2748,11 +2760,11 @@ public:
     {
 	switch (_kind) {
 	case SignedValue:
-	    return 0 <= _sv && _sv <= upper;
+	    return 0 <= _sv && (CORBA::ULongLong)_sv <= upper;
 	case UnsignedValue:
 	    return _uv <= upper;
 	case FloatValue:
-#ifdef _WINDOWS
+#ifdef _WIN32
 	    return 0 <= _fv && _fv <= (CORBA::LongLong)upper;
 #else
 	    return 0 <= _fv && _fv <= upper;
@@ -2767,7 +2779,7 @@ public:
 	case SignedValue:
 	    return lower <= _sv && _sv <= upper;
 	case UnsignedValue:
-#ifdef _WINDOWS
+#ifdef _WIN32
 	    return lower <= (CORBA::LongLong)_uv && (CORBA::LongLong)_uv <= upper;
 #else
 	    return lower <= _uv && _uv <= upper;
