@@ -20,20 +20,32 @@
  *  http://www.mico.org/
  */
 
+#ifdef FAST_PCH
+#include "orb_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #define MICO_CONF_IMR
 #define MICO_CONF_IR
 
 #include <CORBA-SMALL.h>
-#ifndef _WINDOWS
+#ifndef _WIN32
 #include <string.h>
 #endif
-#include <iostream.h>
 #include <mico/impl.h>
 #include <mico/intercept.h>
 #include <mico/template_impl.h>
 #include <mico/util.h>
 #include <mico/os-misc.h>
 
+#endif // FAST_PCH
+
+
+using namespace std;
 
 /******************************* BOA ******************************/
 
@@ -281,7 +293,7 @@ MICO::BOAServImpl::impl_inactive ()
 
 /**************************** BOAImpl ******************************/
 
-#ifdef _WINDOWS
+#ifdef _WIN32
 // error/OK?
 #pragma warning( disable : 4355)
 #endif
@@ -551,7 +563,7 @@ MICO::BOAImpl::unqueue ()
 }
 
 CORBA::Boolean
-MICO::BOAImpl::must_queue (MsgId id)
+MICO::BOAImpl::must_queue (CORBA::ORBMsgId id)
 {
     if (_queue_count > 0)
         return TRUE;
@@ -931,8 +943,8 @@ MICO::BOAImpl::has_object (CORBA::Object_ptr o)
 
     CORBA::Long objkeylen;
     const CORBA::Octet *objkey = prof->objectkey (objkeylen);
-    if (objkeylen >= sizeof (_id_template) &&
-	!memcmp (objkey, _id_template, sizeof (_id_template)))
+    if (objkeylen >= 0 && (CORBA::ULong)objkeylen >= sizeof (_id_template)
+	&& !memcmp (objkey, _id_template, sizeof (_id_template)))
 	return TRUE;
 
     if (!CORBA::is_nil (_oamed) && objkeylen >= 3 && !memcmp (objkey, "OAD", 3))
@@ -1042,15 +1054,23 @@ MICO::BOAImpl::builtin_invoke (CORBA::Object_ptr o,
     }
 }
 
+#ifdef USE_CSL2
+CORBA::Principal_ptr 
+MICO::BOAImpl::get_principal(CORBA::Object_ptr obj){
+  assert(0);
+  return (CORBA::Principal_ptr)0;
+}
+#endif /* USE_CSL2  */
+
 CORBA::Boolean
-MICO::BOAImpl::invoke (MsgId msgid,
+MICO::BOAImpl::invoke (CORBA::ORBMsgId orbid,
 		       CORBA::Object_ptr o,
 		       CORBA::ORBRequest *req,
 		       CORBA::Principal_ptr pr,
 		       CORBA::Boolean response_exp)
 {
-    if (must_queue (msgid)) {
-	_queue.add (new MICO::ReqQueueRec (msgid, req, o, pr, response_exp));
+    if (must_queue (orbid)) {
+	_queue.add (new MICO::ReqQueueRec (orbid, req, o, pr, response_exp));
 	return TRUE;
     }
     /*
@@ -1077,7 +1097,7 @@ MICO::BOAImpl::invoke (MsgId msgid,
 	unqueue();
 	assert (!CORBA::is_nil (fwd_obj));
 
-	_orb->answer_invoke (msgid, CORBA::InvokeForward, fwd_obj, req, 0);
+	_orb->answer_invoke (orbid, CORBA::InvokeForward, fwd_obj, req, 0);
 	return TRUE;
     }
     if (!rec) {
@@ -1087,14 +1107,14 @@ MICO::BOAImpl::invoke (MsgId msgid,
 	 */
         CORBA::OBJECT_NOT_EXIST ex;
 	req->set_out_args (&ex);
-	_orb->answer_invoke (msgid, CORBA::InvokeSysEx,
+	_orb->answer_invoke (orbid, CORBA::InvokeSysEx,
 			     CORBA::Object::_nil(), req, 0);
 	return TRUE;
     }
 
     if (is_builtin_invoke (req->op_name())) {
 	CORBA::ServerRequest_var svreq
-	    = new CORBA::ServerRequest (req, o, msgid, this, pr);
+	    = new CORBA::ServerRequest (req, o, orbid, this, pr);
 
 	builtin_invoke (o, svreq, pr);
     } else {
@@ -1104,7 +1124,7 @@ MICO::BOAImpl::invoke (MsgId msgid,
 	assert (skel);
 
 	CORBA::ServerRequestBase_var svreq
-	    = skel->make_request (req, o, msgid, this, pr);
+	    = skel->make_request (req, o, orbid, this, pr);
         /*
 	 * XXX setting _curr_env is a hack to enable method
 	 * implementations to call get_principal (..., Environment::_nil()),
@@ -1120,7 +1140,7 @@ MICO::BOAImpl::invoke (MsgId msgid,
 }
 
 void
-MICO::BOAImpl::answer_invoke (MsgId msgid,
+MICO::BOAImpl::answer_invoke (CORBA::ORBMsgId msgid,
                               CORBA::Object_ptr obj,
                               CORBA::ORBRequest *req,
 			      CORBA::InvokeStatus stat)
@@ -1201,7 +1221,7 @@ MICO::BOAImpl::activate (const char *repoid)
 }
 
 CORBA::Boolean
-MICO::BOAImpl::dobind (MsgId msgid, const char *repoid,
+MICO::BOAImpl::dobind (CORBA::ORBMsgId msgid, const char *repoid,
 		       const CORBA::ORB::ObjectTag &oid,
 		       CORBA::Address *addr)
 {
@@ -1220,7 +1240,7 @@ MICO::BOAImpl::dobind (MsgId msgid, const char *repoid,
 }
 
 CORBA::Boolean
-MICO::BOAImpl::bind (MsgId msgid, const char *repoid,
+MICO::BOAImpl::bind (CORBA::ORBMsgId msgid, const char *repoid,
 		     const CORBA::ORB::ObjectTag &oid,
 		     CORBA::Address *addr)
 {
@@ -1257,7 +1277,7 @@ MICO::BOAImpl::bind (MsgId msgid, const char *repoid,
 }
 
 CORBA::Boolean
-MICO::BOAImpl::locate (MsgId msgid, CORBA::Object_ptr o)
+MICO::BOAImpl::locate (CORBA::ORBMsgId msgid, CORBA::Object_ptr o)
 {
     _orb->answer_locate (msgid, CORBA::LocateHere, CORBA::Object::_nil(), 0);
     return TRUE;
@@ -1275,7 +1295,7 @@ MICO::BOAImpl::skeleton (CORBA::Object_ptr o)
 }
 
 void
-MICO::BOAImpl::cancel (MsgId msgid)
+MICO::BOAImpl::cancel (CORBA::ORBMsgId msgid)
 {
 }
 
