@@ -20,10 +20,24 @@
  *  http://www.mico.org/
  */
 
+#ifdef FAST_PCH
+#include "orb_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #define MICO_CONF_IMR
 
 #include <CORBA-SMALL.h>
 #include <string.h>
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <iostream>
+#else
+#include <iostream.h>
+#endif
 #include <mico/util.h>
 #include <mico/impl.h>
 #include <mico/intercept.h>
@@ -31,16 +45,15 @@
 
 #include <mico/code_set_registry.h>
 
+#endif // FAST_PCH
+
+
+using namespace std;
 
-#ifdef HAVE_NAMESPACE
 namespace CORBA {
   CORBA::Codeset *Codeset::_special[CORBA::Codeset::_SpecialMax] = { 0 };
   CORBA::Boolean Codeset::_disabled = FALSE;
-};
-#else
-CORBA::Codeset *CORBA::Codeset::_special[CORBA::Codeset::_SpecialMax] = { 0 };
-CORBA::Boolean CORBA::Codeset::_disabled = FALSE;
-#endif
+}
 
 CORBA::Codeset::Codeset()
 {
@@ -144,6 +157,35 @@ CORBA::Codeset::is_compatible (CodesetId id) const
     return is_compatible (&cs);
 }
 
+void
+CORBA::Codeset::free()
+{
+	CORBA::Codeset* cs = NULL;
+	cs = CORBA::Codeset::special_cs(CORBA::Codeset::NativeCS);
+	if (cs)
+	    delete cs;
+	cs = CORBA::Codeset::special_cs(CORBA::Codeset::NativeWCS);
+	if (cs)
+	    delete cs;
+	cs = CORBA::Codeset::special_cs(CORBA::Codeset::FallbackCS);
+	if (cs)
+	    delete cs;
+	cs = CORBA::Codeset::special_cs(CORBA::Codeset::FallbackWCS);
+	if (cs)
+	    delete cs;
+	cs = CORBA::Codeset::special_cs(CORBA::Codeset::DefaultCS);
+	if (cs)
+	    delete cs;
+	cs = CORBA::Codeset::special_cs(CORBA::Codeset::DefaultWCS);
+	if (cs)
+	    delete cs;
+	CORBA::Codeset::set_special_cs(CORBA::Codeset::NativeCS, NULL);
+	CORBA::Codeset::set_special_cs(CORBA::Codeset::NativeWCS, NULL);
+	CORBA::Codeset::set_special_cs(CORBA::Codeset::FallbackCS, NULL);
+	CORBA::Codeset::set_special_cs(CORBA::Codeset::FallbackWCS, NULL);
+	CORBA::Codeset::set_special_cs(CORBA::Codeset::DefaultCS, NULL);
+	CORBA::Codeset::set_special_cs(CORBA::Codeset::DefaultWCS, NULL);
+}
 
 //-----------------
 
@@ -465,6 +507,7 @@ MICO::UniCodesetConv::convert (const char *f, CORBA::ULong len, char *t)
 
 	if (ret != C_OK) {
 	  if (MICO::Logger::IsLogged (MICO::Logger::Error)) {
+	    MICOMT::AutoDebugLock __lock;
 	    MICO::Logger::Stream (MICO::Logger::Error)
 	      << "Error: codeset conversion error in stage 1" << endl;
 	  }
@@ -481,6 +524,7 @@ MICO::UniCodesetConv::convert (const char *f, CORBA::ULong len, char *t)
 
 	if (ret != C_OK) {
 	  if (MICO::Logger::IsLogged (MICO::Logger::Error)) {
+	    MICOMT::AutoDebugLock __lock;
 	    MICO::Logger::Stream (MICO::Logger::Error)
 	      << "Error: codeset conversion error in stage 2" << endl;
 	  }
@@ -502,6 +546,7 @@ MICO::UniCodesetConv::convert (const char *f, CORBA::ULong len, char *t)
 
     if (ret != C_OK) {
       if (MICO::Logger::IsLogged (MICO::Logger::Error)) {
+	MICOMT::AutoDebugLock __lock;
 	MICO::Logger::Stream (MICO::Logger::Error)
 	  << "Error: codeset conversion error in stage 3" << endl;
       }
@@ -513,6 +558,7 @@ MICO::UniCodesetConv::convert (const char *f, CORBA::ULong len, char *t)
 
     if (ret != C_OK) {
       if (MICO::Logger::IsLogged (MICO::Logger::Error)) {
+	MICOMT::AutoDebugLock __lock;
 	MICO::Logger::Stream (MICO::Logger::Error)
 	  << "Error: codeset conversion error in stage 4" << endl;
       }
@@ -655,7 +701,7 @@ CORBA::Long
 MICO::UniCodesetConv::decode (CORBA::Buffer &f, CORBA::ULong len,
 			      CORBA::Char *t, CORBA::Boolean terminate)
 {
-    char *cp, *from, *to;
+    char *cp, *to;
 
     CORBA::ULong flen = _from->codepoint_size();
     if (flen == 3)
@@ -892,8 +938,9 @@ MICO::GIOP_1_0_CodeSetCoder::get_chars (CORBA::DataDecoder & decoder,
   if (!_conv) {
     return decoder.buffer()->get (data, count);
   }
-
-  if (_conv->decode (*decoder.buffer(), count, data, FALSE) != count) {
+  CORBA::Long res;
+  if ((res = _conv->decode (*decoder.buffer(), count, data, FALSE)) < 0
+      || (CORBA::ULong)res != count) {
     return FALSE;
   }
 
@@ -932,7 +979,9 @@ MICO::GIOP_1_0_CodeSetCoder::get_string (CORBA::DataDecoder & decoder,
     }
   }
   else {
-    if (_conv->decode (*decoder.buffer(), length-1, buf, TRUE) != length) {
+    CORBA::Long res;
+    if ((res = _conv->decode (*decoder.buffer(), length-1, buf, TRUE)) < 0
+	|| (CORBA::ULong) res != length) {
       CORBA::string_free (buf);
       return FALSE;
     }
@@ -1001,8 +1050,9 @@ MICO::GIOP_1_0_CodeSetCoder::put_chars (CORBA::DataEncoder & encoder,
     encoder.buffer()->put (data, count);
     return TRUE;
   }
-
-  if (_conv->encode (data, count, *encoder.buffer(), FALSE) != count) {
+  CORBA::Long res;
+  if ((res = _conv->encode (data, count, *encoder.buffer(), FALSE)) < 0
+      || (CORBA::ULong)res != count) {
     return FALSE;
   }
 
@@ -1028,7 +1078,9 @@ MICO::GIOP_1_0_CodeSetCoder::put_string (CORBA::DataEncoder & encoder,
     encoder.buffer()->put (data, length+1);
   }
   else {
-    if (_conv->encode (data, length, *encoder.buffer(), FALSE) != length) {
+    CORBA::Long res;
+    if ((res = _conv->encode (data, length, *encoder.buffer(), FALSE)) < 0
+	|| (CORBA::ULong)res != length) {
       return FALSE;
     }
     encoder.put_octet (0);
@@ -1108,6 +1160,7 @@ MICO::GIOP_1_1_CodeSetCoder::GIOP_1_1_CodeSetCoder (CORBA::Codeset::CodesetId tc
 
     _codepoint_size = cstcs_1->codepoint_size();
     _max_codepoints = cstcs_1->max_codepoints();
+    _native_maxcp   = csnative_1->max_codepoints();
 
     _conv_n2t = ::MICO::CodesetConv::create (csnative_1, cstcs_1);
     _conv_t2n = ::MICO::CodesetConv::create (cstcs_2, csnative_2);
@@ -1182,7 +1235,9 @@ MICO::GIOP_1_1_CodeSetCoder::get_chars (CORBA::DataDecoder & decoder,
   }
 
   if (_max_codepoints == 1 && _codepoint_size == 1) {
-    if (_conv_t2n->decode (*decoder.buffer(), count, data, FALSE) != count) {
+    CORBA::Long res;
+    if ((res = _conv_t2n->decode (*decoder.buffer(), count, data, FALSE)) < 0
+	|| (CORBA::ULong)res != count) {
       return FALSE;
     }
   }
@@ -1239,13 +1294,16 @@ MICO::GIOP_1_1_CodeSetCoder::get_string (CORBA::DataDecoder & decoder,
       return FALSE;
     }
   }
-  else if (_max_codepoints == 1 && _codepoint_size == 1) {
+  else if (_max_codepoints == 1 && _codepoint_size == 1 &&
+	   _native_maxcp == 1) {
     if (length > decoder.buffer()->length()) {
       return FALSE;
     }
 
     buf = CORBA::string_alloc (length-1);
-    if (_conv_t2n->decode (*decoder.buffer(), length-1, buf, TRUE) != length) {
+    CORBA::Long res;
+    if ((res = _conv_t2n->decode (*decoder.buffer(), length-1, buf, TRUE)) < 0
+	|| (CORBA::ULong)res != length) {
       CORBA::string_free (buf);
       return FALSE;
     }
@@ -1356,7 +1414,7 @@ MICO::GIOP_1_1_CodeSetCoder::get_string (CORBA::DataDecoder & decoder,
       return FALSE;
     }
 
-    buf = CORBA::string_alloc (codepoints);
+    buf = CORBA::string_alloc (codepoints * _native_maxcp);
 
     if (_conv_t2n->decode (*decoder.buffer(), codepoints, buf, TRUE) < 0) {
       CORBA::string_free (buf);
@@ -1444,7 +1502,9 @@ MICO::GIOP_1_1_CodeSetCoder::put_chars (CORBA::DataEncoder & encoder,
   }
 
   if (_max_codepoints == 1 && _codepoint_size == 1) {
-    if (_conv_n2t->encode (data, count, *encoder.buffer(), FALSE) != count) {
+    CORBA::Long res;
+    if ((res = _conv_n2t->encode (data, count, *encoder.buffer(), FALSE)) < 0
+	|| (CORBA::ULong)res != count) {
       return FALSE;
     }
   }
@@ -1481,9 +1541,12 @@ MICO::GIOP_1_1_CodeSetCoder::put_string (CORBA::DataEncoder & encoder,
     encoder.put_ulong (length+1);
     encoder.buffer()->put (data, length+1);
   }
-  else if (_max_codepoints == 1 && _codepoint_size == 1) {
+  else if (_max_codepoints == 1 && _codepoint_size == 1 &&
+	   _native_maxcp == 1) {
     encoder.put_ulong (length+1);
-    if (_conv_n2t->encode (data, length, *encoder.buffer(), FALSE) != length) {
+    CORBA::Long res;
+    if ((res = _conv_n2t->encode (data, length, *encoder.buffer(), FALSE)) < 0
+	|| (CORBA::ULong)res != length) {
       return FALSE;
     }
     encoder.put_octet (0);
@@ -2075,6 +2138,11 @@ MICO::GIOP_1_2_CodeSetCoder::put_wstring (CORBA::DataEncoder & encoder,
     return FALSE;
   }
 
+  if (length == 0) {
+    encoder.put_ulong (0);
+    return TRUE;
+  }
+
   if (!_w_conv_n2t) {
 #ifndef HAVE_BYTEORDER_BE
     if (_tcsw == 0x00010109) {
@@ -2472,7 +2540,7 @@ static MICO::CodesetComponentDecoder codeset_component_decoder;
 static class CodesetInit : public Interceptor::InitInterceptor {
 public:
     CodesetInit ()
-        : MICO_SCOPE(Interceptor,InitInterceptor) (0)
+        : Interceptor::InitInterceptor(0)
     {
     }
     Interceptor::Status initialize (CORBA::ORB_ptr orb,
@@ -2517,6 +2585,7 @@ public:
 	CORBA::Codeset *cs = CORBA::Codeset::create (cs_name);
 	if (!cs) {
 	  if (MICO::Logger::IsLogged (MICO::Logger::Error)) {
+	    MICOMT::AutoDebugLock __lock;
 	    MICO::Logger::Stream (MICO::Logger::Error)
 	      << "Error: unknown native char code set: " << cs_name << endl;
 	  }
@@ -2531,6 +2600,7 @@ public:
 	cs = CORBA::Codeset::create (wcs_name);
 	if (!cs) {
 	  if (MICO::Logger::IsLogged (MICO::Logger::Error)) {
+	    MICOMT::AutoDebugLock __lock;
 	    MICO::Logger::Stream (MICO::Logger::Error)
 	      << "Error: unknown native wide char code set: "
 	      << wcs_name << endl;
@@ -2544,6 +2614,7 @@ public:
 	cs = CORBA::Codeset::create (cs_name);
 	if (!cs) {
 	  if (MICO::Logger::IsLogged (MICO::Logger::Error)) {
+	    MICOMT::AutoDebugLock __lock;
 	    MICO::Logger::Stream (MICO::Logger::Error)
 	      << "Error: unknown default char code set: " << cs_name << endl;
 	  }
@@ -2556,6 +2627,7 @@ public:
 	cs = CORBA::Codeset::create (cs_name);
 	if (!cs) {
 	  if (MICO::Logger::IsLogged (MICO::Logger::Error)) {
+	    MICOMT::AutoDebugLock __lock;
 	    MICO::Logger::Stream (MICO::Logger::Error)
 	      << "Error: unknown default wide char code set: "
 	      << cs_name << endl;
@@ -2569,6 +2641,7 @@ public:
 	cs = CORBA::Codeset::create (cs_name);
 	if (!cs) {
 	  if (MICO::Logger::IsLogged (MICO::Logger::Error)) {
+	    MICOMT::AutoDebugLock __lock;
 	    MICO::Logger::Stream (MICO::Logger::Error)
 	      << "Error: unknown fallback char code set: "
 	      << cs_name << endl;
@@ -2582,6 +2655,7 @@ public:
 	cs = CORBA::Codeset::create (cs_name);
 	if (!cs) {
 	  if (MICO::Logger::IsLogged (MICO::Logger::Error)) {
+	    MICOMT::AutoDebugLock __lock;
 	    MICO::Logger::Stream (MICO::Logger::Error)
 	      << "Error: unknown fallback wide char code set: "
 	      << cs_name << endl;
@@ -2611,5 +2685,22 @@ public:
 void
 CORBA::Codeset::_init ()
 {
-  __cs_init;
+  // Is it really needed? E.g. for some obsolete compilers?
+  // In that case, please uncomment following line
+  //__cs_init;
 }
+
+struct __init_CODESET
+{
+    __init_CODESET()
+    {
+    }
+
+    ~__init_CODESET()
+    {
+	CORBA::Codeset::free();
+    }
+};
+
+static __init_CODESET __module_init_CODESET;
+
