@@ -1,6 +1,6 @@
 /*
  *  MICO --- an Open Source CORBA implementation
- *  Copyright (c) 1997-2001 by The Mico Team
+ *  Copyright (c) 1997-2003 by The Mico Team
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -20,14 +20,25 @@
  *  http://www.mico.org/
  */
 
+#ifdef FAST_PCH
+#include "orb_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #define MICO_CONF_IMR
+#define MICO_CONF_IR
+#define MICO_CONF_INTERCEPT
+#define MICO_CONF_POA
 
 #include <CORBA-SMALL.h>
-#ifndef _WINDOWS
+#ifndef _WIN32
 #include <string.h>
 #endif
 #include <ctype.h>
-#include <iostream.h>
 #include <mico/throw.h>
 #include <mico/impl.h>
 #include <mico/intercept.h>
@@ -35,24 +46,22 @@
 #include <mico/template_impl.h>
 #include <mico/os-misc.h>
 #include <mico/basic_seq.h>
+#include <mico/pi_impl.h>
 
-#if defined(_WINDOWS) && !defined(__MINGW32__) && !defined(__BORLANDC__)
-#include <atlbase.h>
-#if ( !defined(_MSC_VER) || !defined(_ATL_VER) || (_MSC_VER < 1100) || (_ATL_VER < 0x0202))
-#error "You need Visual C++ 5.0 with Service Pack 3"
-// Visual C++ 5.0 with Service Pack 3 is needed for all files,
-// this check is used (only) here because:
-// 1: it is the first in the Makefile.win32
-// 2: the large atlbase.h abused to determine the SPn
-// 3: necessary because of the FAQ "why doesn't it compile"
-#endif
-#endif
+#ifndef USE_CSL2
+#  include <mico/ssl.h>
+#endif /* NOT USE_CSL2  */
 
+#endif // FAST_PCH
+
+
+using namespace std;
 
 /*****************************  NamedValue  ***************************/
 
 
-CORBA::NamedValue::NamedValue (const char *nameval, const Any &val, Flags flagsval)
+CORBA::NamedValue::NamedValue(const char *nameval, const Any &val,
+			      Flags flagsval)
     : _n (nameval),  _v (val), _f (flagsval)
 {
 }
@@ -236,13 +245,9 @@ CORBA::Environment::clear ()
 /*****************************  Principal  ***************************/
 
 
-#ifdef HAVE_NAMESPACE
 namespace CORBA {
   vector<CORBA::Octet> Principal::_peer_info;
-};
-#else
-vector<CORBA::Octet> CORBA::Principal::_peer_info;
-#endif
+}
 
 CORBA::Principal::Principal (CORBA::Transport *t)
 {
@@ -259,6 +264,15 @@ CORBA::Principal::Principal (DataDecoder &dc, CORBA::Transport *t)
     assert (r);
 }
 
+#ifdef USE_CSL2
+CORBA::Transport*
+CORBA::Principal::transport()
+{
+	return _transp;
+}
+#endif /* USE_CSL2  */
+
+
 CORBA::Principal::~Principal ()
 {
 }
@@ -318,6 +332,18 @@ CORBA::Principal::list_properties ()
     return pl;
 }
 
+#ifdef USE_CSL2
+CORBA::Any* CORBA::Principal::get_attribute(unsigned long fam, unsigned long attr) {
+  assert (0);
+  return (CORBA::Any*)0;
+}
+
+CORBA::Boolean CORBA::Principal::check(string& s)
+{
+	return 0;
+}
+#endif /* USE_CSL2  */
+
 CORBA::Any*
 CORBA::Principal::get_property (const char *prop_name)
 {
@@ -511,9 +537,12 @@ CORBA::Request::Request (Object_ptr o, Context_ptr c,
     orb->create_context_list (_clist);
     orb->create_exception_list (_elist);
     _orbreq = new MICO::LocalRequest (this);
-    _iceptreq = Interceptor::ClientInterceptor::_create_request (
-	_object, _opname, *_orbreq->context(), this);
+    _iceptreq = Interceptor::ClientInterceptor::_create_request
+	(_object, _opname, *_orbreq->context(), this);
     _cb = 0;
+    _cri = PInterceptor::PI::_create_cri
+	(_object, _opname, 0, _args, _elist, _clist, _context, _res, TRUE);
+    _invoke_flag = FALSE;
 }
 
 CORBA::Request::Request (Object_ptr o,
@@ -558,9 +587,12 @@ CORBA::Request::Request (Object_ptr o,
     if (CORBA::is_nil (_elist))
 	orb->create_exception_list (_elist);
     _orbreq = new MICO::LocalRequest (this);
-    _iceptreq = Interceptor::ClientInterceptor::_create_request (
-	_object, _opname, *_orbreq->context(), this);
+    _iceptreq = Interceptor::ClientInterceptor::_create_request
+	(_object, _opname, *_orbreq->context(), this);
     _cb = 0;
+    _cri = PInterceptor::PI::_create_cri
+	(_object, _opname, 0, _args, _elist, _clist, _context, _res, TRUE);
+    _invoke_flag = FALSE;
 }
 
 CORBA::Request::Request (Object_ptr o, const char *op)
@@ -588,17 +620,21 @@ CORBA::Request::Request (Object_ptr o, const char *op)
     // XXX use IR to fill elist/clist
     orb->create_context_list (_clist);
     orb->create_exception_list (_elist);
+    _orbid = 0;
     _flags = 0;
     _orbreq = new MICO::LocalRequest (this);
-    _iceptreq = Interceptor::ClientInterceptor::_create_request (
-	_object, _opname, *_orbreq->context(), this);
+    _iceptreq = Interceptor::ClientInterceptor::_create_request
+	(_object, _opname, *_orbreq->context(), this);
     _cb = 0;
+    _cri = PInterceptor::PI::_create_cri
+	(_object, _opname, 0, _args, _elist, _clist, _context, _res, TRUE);
+    _invoke_flag = FALSE;
 }
 
 CORBA::Request::~Request ()
 {
     if (_invoke_pending)
-        _object->_orbnc()->cancel (_msgid);
+        _object->_orbnc()->cancel (_orbid);
     CORBA::release (_context);
     CORBA::release (_environm);
     CORBA::release (_args);
@@ -607,11 +643,14 @@ CORBA::Request::~Request ()
     CORBA::release (_elist);
     CORBA::release (_orbreq);
     CORBA::release (_iceptreq);
+    CORBA::release (_cri);
 }
 
 void
 CORBA::Request::decode_user_exception ()
 {
+  bool known_exception = false;
+
     Exception *ex = _environm->exception();
     if (ex && _elist->count() > 0) {
 	UnknownUserException *uuex =
@@ -624,17 +663,57 @@ CORBA::Request::decode_user_exception ()
 		tc = _elist->item (i);
 		if (!strcmp (repoid, tc->id())) {
 		    uuex->exception (tc);
+		    known_exception = true;
 		    break;
 		}
 	    }
 	}
+	else {
+	    UserException *uex = 
+		UserException::_downcast (ex);
+	    if (uex) {
+		// check UserException using ExceptionList ...
+		const char *repoid = uex->_repoid ();
+		TypeCode_ptr tc;
+		for (CORBA::ULong i = 0; i < _elist->count(); ++i) {
+		    tc = _elist->item (i);
+		    if (!strcmp (repoid, tc->id())) {
+			known_exception = true;
+			break;
+		    }
+		}
+	    }
+	}
+    }
+
+    if (ex && !known_exception) {
+      // oops, unknown exception
+      CORBA::Exception * nex =
+	new CORBA::UNKNOWN (CORBA::OMGVMCID | 1,
+			    CORBA::COMPLETED_MAYBE);
+      _environm->exception (nex);
     }
 }
 
+CORBA::Boolean
+CORBA::Request::waitfor (CORBA::ORB_ptr orb, CORBA::ORBMsgId msgid,
+			 CORBA::ORBCallback::Event ev, CORBA::Long tmout)
+{
+#ifdef HAVE_THREADS
+    return async_callback.waitfor(orb, msgid, ev, tmout);
+#else // HAVE_THREADS
+    assert(0);
+    return TRUE;
+#endif // HAVE_THREADS
+}
+
 void
-CORBA::Request::callback (ORB_ptr, ULong, ORBCallback::Event ev)
+CORBA::Request::notify (ORB_ptr orb, CORBA::ORBMsgId id, ORBCallback::Event ev)
 {
     assert (ev == ORBCallback::Invoke);
+#ifdef HAVE_THREADS
+    async_callback.notify(orb, id, ev);
+#endif // HAVE_THREADS
     if (_cb) {
         RequestCallback *tmpcb = _cb;
         _cb = 0;
@@ -702,8 +781,10 @@ CORBA::Request::return_value ()
 void
 CORBA::Request::invoke ()
 {
+    _invoke_flag = TRUE;
     send_deferred ();
     get_response ();
+    _invoke_flag = FALSE;
 }
 
 void
@@ -716,10 +797,27 @@ CORBA::Request::send_oneway ()
 				  _environm)) {
 	return;
     }
-
     ORB_ptr orb = _object->_orbnc();
-    orb->invoke_async (_object, _orbreq, Principal::_nil(), FALSE);
+    PInterceptor::PI::_send_request_ip
+	(_cri, CORBA::ORB::get_msgid(_orbid), _args, this->contexts(), this->ctx(),
+	 _orbreq->context(), FALSE);
 
+    CORBA::ORBMsgId __id = orb->invoke_async
+	(_object, _orbreq, Principal::_nil(), FALSE);
+
+//      Corba::ULong mid = orb->invoke_async
+//  	(_object, _orbreq, Principal::_nil(), FALSE);
+//      if (mid > 0) { // object not exists
+    if (__id != NULL) {
+	cerr << "dii.cc hack pi...." << endl;
+	CORBA::OBJECT_NOT_EXIST ex;
+	PInterceptor::PI::_receive_exception_ip
+	    (_cri, PortableInterceptor::SYSTEM_EXCEPTION, ex._clone(),
+	     _orbreq->context());
+    }
+    else {
+	PInterceptor::PI::_receive_other_ip(_cri);
+    }
     Interceptor::ClientInterceptor::
 	_exec_after_marshal ((Interceptor::LWRequest_ptr)_iceptreq, _environm);
 }
@@ -743,13 +841,22 @@ CORBA::Request::send_deferred (RequestCallback *cb)
     _invoke_pending = TRUE;
 
     ORB_ptr orb = _object->_orbnc();
-    _msgid = orb->new_msgid();
-    orb->invoke_async (_object, _orbreq, Principal::_nil(),
-                       TRUE, _cb ? this : 0, _msgid);
+    _orbid = orb->new_orbid();
+//      orb->invoke_async (_object, _orbreq, Principal::_nil(),
+//                         TRUE, _cb ? this : 0, _orbid);
 
+//      _msgid = orb->new_msgid();
+    PInterceptor::PI::_send_request_ip
+	(_cri, CORBA::ORB::get_msgid(_orbid), _args, this->contexts(),
+	 this->ctx(), _orbreq->context());
+    
+    orb->invoke_async(_object, _orbreq, Principal::_nil(),
+		      TRUE, _cb ? this : 0, _orbid);
+	
     if (_invoke_pending && !Interceptor::ClientInterceptor::
-	_exec_after_marshal ((Interceptor::LWRequest_ptr)_iceptreq, _environm)) {
-	orb->cancel (_msgid);
+	_exec_after_marshal ((Interceptor::LWRequest_ptr)_iceptreq,
+			     _environm)) {
+	orb->cancel (_orbid);
 	_invoke_pending = FALSE;
 	if (_cb)
 	    _cb->callback (this, RequestCallback::RequestDone);
@@ -774,10 +881,10 @@ CORBA::Request::get_response (Boolean block)
 	ORBRequest *dummy;
 
 	if (block) {
-	    CORBA::Boolean r = orb->wait (_msgid);
+	    CORBA::Boolean r = orb->wait (_orbid);
 	    assert (r);
 	} else {
-	    if (!orb->wait (_msgid, 0))
+	    if (!orb->wait (_orbid, 0))
 		return;
 	}
 
@@ -785,28 +892,48 @@ CORBA::Request::get_response (Boolean block)
 	if (!Interceptor::ClientInterceptor::
 	    _exec_before_unmarshal ((Interceptor::LWRequest_ptr)_iceptreq,
 				    _environm)) {
-	    orb->cancel (_msgid);
+	    orb->cancel (_orbid);
 	    _invoke_pending = FALSE;
 	    _cb = 0;
 	    return;
 	}
 
 	GIOP::AddressingDisposition ad;
-	rs = orb->get_invoke_reply (_msgid, nobj, dummy, ad);
-
+	rs = orb->get_invoke_reply (_orbid, nobj, dummy, ad);
+//  	rs = orb->get_invoke_reply (_msgid, nobj, dummy, ad);
+	
 	switch (rs) {
 	case InvokeForward:
 	    // XXX what if 'object' is not a stub ???
 	    assert (_object.in() && nobj.in());
 	    _object->_forward (nobj);
-	    _msgid = orb->invoke_async (_object, _orbreq, Principal::_nil());
+	    PInterceptor::PI::_receive_other_ip
+		(_cri, PortableInterceptor::LOCATION_FORWARD, _object,
+		 dummy->context());
+	    CORBA::release(_cri);
+	    //_msgid = orb->new_msgid();
+	    _orbid = orb->new_orbid();
+	    _cri = PInterceptor::PI::_create_cri
+		(_object, _opname, _elist, _res);
+	    PInterceptor::PI::_send_request_ip
+		(_cri, CORBA::ORB::get_msgid(_orbid), _args, _clist, _context,
+		 _orbreq->context());
+	    //_msgid = orb->invoke_async (_object, _orbreq, Principal::_nil(),
+	    //TRUE, 0, _msgid);
+	    _orbid = orb->invoke_async (_object, _orbreq, Principal::_nil(),
+					TRUE, 0, _orbid);
 	    break;
 
 	case InvokeOk:
+	    PInterceptor::PI::_receive_reply_ip
+		(_cri, *_res->value(), _args, dummy->context(), TRUE);
 	    _invoke_pending = FALSE;
 	    break;
 
 	case CORBA::InvokeSysEx:
+	    PInterceptor::PI::_receive_exception_ip
+		(_cri, PortableInterceptor::SYSTEM_EXCEPTION,
+		 _environm->exception()->_clone(), dummy->context());
 	    if (_object->_is_forwarded()) {
 		/*
 		 * [15-44] says:
@@ -829,8 +956,20 @@ CORBA::Request::get_response (Boolean block)
                      CORBA::OBJECT_NOT_EXIST::_downcast (sysex))) {
 		    _object->_unforward();
 		    env()->clear();
-		    _msgid = orb->invoke_async (_object, _orbreq,
-                                                Principal::_nil());
+		    CORBA::release (_cri);
+		    //_msgid = orb->new_msgid();
+		    if (_orbid != NULL)
+			delete _orbid;
+		    _orbid = orb->new_orbid();
+		    _cri = PInterceptor::PI::_create_cri
+			(_object, _opname, _elist, _res);
+		    PInterceptor::PI::_send_request_ip
+			(_cri, CORBA::ORB::get_msgid(_orbid), _args, _clist,
+			 _context, _orbreq->context());
+		    //_msgid = orb->invoke_async
+		    //(_object, _orbreq, Principal::_nil(), TRUE, 0, _msgid);
+		    _orbid = orb->invoke_async
+			(_object, _orbreq, Principal::_nil(), TRUE, 0, _orbid);
 		    break;
 		}
 	    }
@@ -839,12 +978,30 @@ CORBA::Request::get_response (Boolean block)
 
 	case InvokeUsrEx:
 	    decode_user_exception ();
+	    PInterceptor::PI::_receive_exception_ip
+		(_cri, PortableInterceptor::USER_EXCEPTION,
+		 _environm->exception()->_clone(), dummy->context());
 	    _invoke_pending = FALSE;
 	    break;
 
 	case InvokeAddrDisp:
 	    _object->_ior_fwd()->addressing_disposition (ad);
-	    _msgid = orb->invoke_async (_object, _orbreq, Principal::_nil());
+	    PInterceptor::PI::_receive_other_ip
+		(_cri, PortableInterceptor::TRANSPORT_RETRY, dummy->context());
+	    CORBA::release (_cri);
+	    //_msgid = orb->new_msgid();
+	    if (_orbid != NULL)
+		delete _orbid;
+	    _orbid = orb->new_orbid();
+	    _cri = PInterceptor::PI::_create_cri
+		(_object, _opname, _elist, _res);
+	    PInterceptor::PI::_send_request_ip
+		(_cri, CORBA::ORB::get_msgid(_orbid), _args, _clist,
+		 _context, _orbreq->context());
+	    //_msgid = orb->invoke_async (_object, _orbreq, Principal::_nil(),
+	    //TRUE, 0, _msgid);
+	    _orbid = orb->invoke_async(_object, _orbreq, Principal::_nil(),
+				       TRUE, 0, _orbid);
 	    break;
 
 	default:
@@ -853,7 +1010,8 @@ CORBA::Request::get_response (Boolean block)
     }
 
     if (!Interceptor::ClientInterceptor::
-	_exec_finish_request ((Interceptor::LWRequest_ptr)_iceptreq, _environm)) {
+	_exec_finish_request ((Interceptor::LWRequest_ptr)_iceptreq,
+			      _environm)) {
 	_cb = 0;
 	return;
     }
@@ -868,7 +1026,7 @@ CORBA::Boolean
 CORBA::Request::poll_response ()
 {
     // XXX call get_response() to check for redirections...
-    get_response (FALSE);
+    this->get_response (FALSE);
     return !_invoke_pending;
 }
 
@@ -1165,3 +1323,5 @@ MICO::LocalRequest::request ()
 {
     return _req;
 }
+
+
