@@ -20,20 +20,32 @@
  *  http://www.mico.org/
  */
 
+#ifdef FAST_PCH
+#include "orb_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #define MICO_CONF_IMR
 
 #include <CORBA-SMALL.h>
-#ifndef _WINDOWS
+#ifndef _WIN32
 #include <string.h>
 #endif
 #include <errno.h>
-#include <iostream.h>
 #include <mico/impl.h>
 #include <mico/template_impl.h>
 #include <mico/util.h>
 #include <mico/os-misc.h>
 #include <mico/os-net.h>
 
+#endif // FAST_PCH
+
+
+using namespace std;
 
 /**************************** misc dtors *******************************/
 
@@ -77,6 +89,8 @@ CORBA::Timeout::callback (Dispatcher *d, Event ev)
     case CORBA::Dispatcher::Moved:
 	_disp = d;
 	break;
+    default:
+	break;
     }
 }
 
@@ -112,6 +126,8 @@ public:
     }
 };
 
+CORBA::Boolean MICO::SelectDispatcher::_isblocking = FALSE;
+
 MICO::SelectDispatcher::SelectDispatcher ()
     : last_update (0), init (TRUE), locked (0), modified (FALSE)
 {
@@ -132,6 +148,18 @@ MICO::SelectDispatcher::~SelectDispatcher ()
 	(*j).cb->callback (this, Remove);
 }
 
+void 
+MICO::SelectDispatcher::block (CORBA::Boolean b)
+{
+  _isblocking = b;
+}
+
+CORBA::Boolean 
+MICO::SelectDispatcher::isblocking ()
+{
+  return _isblocking;
+}
+
 CORBA::Long
 MICO::SelectDispatcher::gettime () const
 {
@@ -251,6 +279,7 @@ MICO::SelectDispatcher::handle_fevents (FDSet &rset, FDSet &wset, FDSet &xset)
 	    }
 	}
     }
+
     unlock ();
 }
 
@@ -270,13 +299,20 @@ MICO::SelectDispatcher::unlock ()
     assert (locked == 0);
 
     if (modified) {
-	list<FileEvent>::iterator i, next;
-	for (i = fevents.begin(); i != fevents.end(); i = next) {
-	    next = i;
-	    ++next;
-	    if ((*i).deleted)
-		fevents.erase (i);
+	list<FileEvent>::iterator i;
+	bool again;
+
+	do {
+	  again = false;
+	  for (i = fevents.begin(); i != fevents.end(); i++) {
+	    if ((*i).deleted) {
+	      fevents.erase (i);
+	      again = true;
+	      break;
+	    }
+	  }
 	}
+	while (again);
     }
 }
 
@@ -345,30 +381,45 @@ MICO::SelectDispatcher::remove (CORBA::DispatcherCallback *cb, Event e)
 
     if (e == All || e == Timer) {
 	list<TimerEvent>::iterator i, next;
-	for (i = tevents.begin(); i != tevents.end(); i = next) {
+	bool again;
+
+	do {
+	  again = false;
+	  for (i = tevents.begin(); i != tevents.end(); i++) {
 	    next = i;
 	    ++next;
 	    if ((*i).cb == cb) {
-		CORBA::Long delta = (*i).delta;
-		if (next != tevents.end())
-		    (*next).delta += delta;
-		tevents.erase (i);
+	      CORBA::Long delta = (*i).delta;
+	      if (next != tevents.end())
+		(*next).delta += delta;
+	      tevents.erase (i);
+	      again = true;
+	      break;
 	    }
+	  }
 	}
+	while (again);
     }
     if (e == All || e == Read || e == Write || e == Except) {
-	list<FileEvent>::iterator i, next;
-	for (i = fevents.begin(); i != fevents.end(); i = next) {
-	    next = i;
-	    ++next;
+	list<FileEvent>::iterator i;
+	bool again;
+
+	do {
+	  again = false;
+	  for (i = fevents.begin(); i != fevents.end(); i++) {
 	    if ((*i).cb == cb && (e == All || (*i).event == e)) {
 		if (islocked()) {
 		    (*i).deleted = TRUE;
 		} else {
 		    fevents.erase (i);
+		    again = true;
+		    break;
 		}
 	    }
+	  }
 	}
+	while (again);
+
 	update_fevents ();
     }
 }
