@@ -1,6 +1,6 @@
 /*
  *  MICO --- an Open Source CORBA implementation
- *  Copyright (c) 1997-2001 by The Mico Team
+ *  Copyright (c) 1997-2003 by The Mico Team
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -20,18 +20,33 @@
  *  http://www.mico.org/
  */
 
+#ifdef FAST_PCH
+#include "orb_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #define MICO_CONF_IMR
 #define MICO_CONF_IR
+#define MICO_CONF_INTERCEPT
+#define MICO_CONF_POA
 
 #include <CORBA-SMALL.h>
-#ifndef _WINDOWS
+#ifndef _WIN32
 #include <string.h>
 #endif
-#include <iostream.h>
 #include <mico/intercept.h>
 #include <mico/template_impl.h>
 #include <mico/util.h>
+#include <mico/pi_impl.h>
 
+#endif // FAST_PCH
+
+
+using namespace std;
 
 CORBA::ImplementationBase::ImplementationBase ()
 {
@@ -103,10 +118,11 @@ CORBA::ImplementationBase::_find_impl (const char *repoid, const char *name)
 
     if (impls->length() > 0) {
       if (MICO::Logger::IsLogged (MICO::Logger::Info)) {
+	MICOMT::AutoDebugLock __lock;
 	MICO::Logger::Stream (MICO::Logger::Info)
 	  << "Info: found impl repository entry: " << name << endl;
       }
-      
+
       impl = CORBA::ImplementationDef::_duplicate (impls[(CORBA::ULong)0]);
       CORBA::ImplementationDef::ObjectInfoList_var objs = impl->objs();
       CORBA::ULong i;
@@ -116,21 +132,23 @@ CORBA::ImplementationBase::_find_impl (const char *repoid, const char *name)
       }
       if (i == objs->length()) {
 	if (MICO::Logger::IsLogged (MICO::Logger::Info)) {
+	  MICOMT::AutoDebugLock __lock;
 	  MICO::Logger::Stream (MICO::Logger::Info)
 	    << "Info: adding repo id: " << repoid
 	    << " to impl repository" << endl;
 	}
-	
+
 	objs->length (objs->length()+1);
 	objs[objs->length()-1].repoid = CORBA::string_dup (repoid);
 	impl->objs (objs.in());
       }
     } else {
       if (MICO::Logger::IsLogged (MICO::Logger::Info)) {
+	MICOMT::AutoDebugLock __lock;
 	MICO::Logger::Stream (MICO::Logger::Info)
 	  << "Info: creating impl repository entry: " << name << endl;
       }
-      
+ 
       CORBA::ImplementationDef::ObjectInfoList objs;
       objs.length (1);
       objs[0].repoid = CORBA::string_dup (repoid);
@@ -220,11 +238,11 @@ CORBA::DynamicImplementation::~DynamicImplementation ()
 CORBA::ServerRequestBase_ptr
 CORBA::DynamicImplementation::make_request (CORBA::ORBRequest *req,
 					    CORBA::Object_ptr obj,
-					    CORBA::ULong msgid,
+					    CORBA::ORBMsgId id,
 					    CORBA::ObjectAdapter *oa,
 					    CORBA::Principal_ptr pr)
 {
-    return new ServerRequest (req, obj, msgid, oa, pr);
+    return new ServerRequest (req, obj, id, oa, pr);
 }
 
 void
@@ -239,9 +257,9 @@ CORBA::DynamicImplementation::doinvoke (ServerRequestBase_ptr req,
 
 
 CORBA::ServerRequest::ServerRequest (ORBRequest *r, Object_ptr o,
-                                     MsgId id, CORBA::ObjectAdapter *oa,
+                                     CORBA::ORBMsgId id, CORBA::ObjectAdapter *oa,
 				     CORBA::Principal_ptr pr)
-    : _oa (oa), _msgid (id),
+    : _oa (oa), _id (id),
       _obj(Object::_duplicate (o)),
       _req(ORBRequest::_duplicate (r)),
       _env (pr)
@@ -277,7 +295,14 @@ CORBA::ServerRequest::~ServerRequest ()
                 stat = CORBA::InvokeUsrEx;
             }
         }
-        _oa->answer_invoke (_msgid, _obj, _req, stat);
+	CORBA::ORBInvokeRec* rec = _id;
+	if (rec) {
+	    PInterceptor::PI::_set_sri_exception
+		(rec->requestinfo(), this->exception());
+	}
+        
+        //_oa->answer_invoke (_msgid, _obj, _req, stat);
+        _oa->answer_invoke (_id, _obj, _req, stat);
 
         if (!_aborted && !Interceptor::ServerInterceptor::
             _exec_finish_request ((Interceptor::LWServerRequest_ptr)_iceptreq,
@@ -364,6 +389,7 @@ CORBA::ServerRequest::params (NVList_ptr p)
 
     if (!_req->get_in_args (_dir_params, _context)) {
       if (MICO::Logger::IsLogged (MICO::Logger::Warning)) {
+	MICOMT::AutoDebugLock __lock;
 	MICO::Logger::Stream (MICO::Logger::Warning)
 	  << "Warning: cannot decode args in ServerRequest::params" << endl;
       }
@@ -378,6 +404,34 @@ CORBA::ServerRequest::params (NVList_ptr p)
 	_aborted = TRUE;
 	return FALSE;
     }
+
+    CORBA::ORB_ptr orb = _obj->_orbnc();
+    CORBA::ORBInvokeRec* rec = _id;
+    if (!rec)
+    	return TRUE;
+    PInterceptor::ServerRequestInfo_impl* sri = rec->requestinfo();
+    if (!sri)
+	return TRUE;
+    // point of receive_request call
+    try {
+	PInterceptor::PI::_receive_request_ip(sri, _dir_params, _context, _oa, _obj);
+    } catch (CORBA::SystemException_catch& ex) {
+	exception(ex->_clone());
+	_aborted = TRUE;
+	return FALSE;
+    } catch (PortableInterceptor::ForwardRequest_catch& exc) {
+	_obj = Object::_duplicate(exc->forward);
+	orb->answer_invoke(rec, CORBA::InvokeForward, exc->forward, _req, 0);
+    } catch (CORBA::UserException& ex) {
+	exception(ex._clone());
+	_aborted = TRUE;
+	return FALSE;
+    } catch (...) {
+	CORBA::UNKNOWN ex;
+	exception(ex._clone());
+	_aborted = TRUE;
+	return FALSE;
+    }
     return TRUE;
 }
 
@@ -436,14 +490,28 @@ CORBA::ServerRequest::set_out_args ()
 
     if (_env.exception()) {
         _req->set_out_args (_env.exception());
-    } else if (!CORBA::is_nil (_dir_params)) {
+    } 
+    else if (!CORBA::is_nil (_dir_params)) {
+	CORBA::ORBInvokeRec* rec = _id;
+	if (rec) {
+	    PInterceptor::ServerRequestInfo_impl* sri = rec->requestinfo();
+	    CORBA::TypeCode_ptr tc;
+	    if (_res && (tc = _res->type())) {
+		if (tc->kind() != CORBA::tk_void 
+		    && tc->kind() != CORBA::tk_null) {
+		    PInterceptor::PI::_set_sri_result(sri, *_res);
+		}
+	    }
+	    PInterceptor::PI::_set_sri_args(sri, _dir_params, TRUE);
+	}
+
 	if (!_req->set_out_args (_res, _dir_params)) {
-	  if (MICO::Logger::IsLogged (MICO::Logger::Warning)) {
-	    MICO::Logger::Stream (MICO::Logger::Warning)
-	      << "Warning: cannot ServerRequest::set_out_args" << endl;
-	  }
-	  CORBA::MARSHAL ex (0, CORBA::COMPLETED_YES);
-	  _req->set_out_args (&ex);
+	    if (MICO::Logger::IsLogged (MICO::Logger::Warning)) {
+		MICO::Logger::Stream (MICO::Logger::Warning)
+		    << "Warning: cannot ServerRequest::set_out_args" << endl;
+	    }
+	    CORBA::MARSHAL ex (0, CORBA::COMPLETED_YES);
+	    _req->set_out_args (&ex);
         }
     }
 }
