@@ -20,17 +20,34 @@
  *  http://www.mico.org/
  */
 
+#ifdef FAST_PCH
+#include "orb_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #define MICO_CONF_IMR
 #define MICO_CONF_DYNANY
 
-#include <strstream.h>
 #include <CORBA-SMALL.h>
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <sstream>
+#else
+#include <strstream.h>
+#endif
 #include <mico/dynany_impl.h>
 #include <mico/impl.h>
 #include <mico/throw.h>
 #include <mico/template_impl.h>
 #include <mico/util.h>
 
+#endif // FAST_PCH
+
+
+using namespace std;
 
 /************************** DynAny *********************************/
 
@@ -92,10 +109,13 @@ DynAny_impl::type ()
 void
 DynAny_impl::destroy ()
 {
-    for (CORBA::ULong i = 0; i < _elements.size(); ++i)
-	_elements[i]->destroy();
-    _elements.erase (_elements.begin(), _elements.end());
-    CORBA::release (this);
+  for (CORBA::ULong i = 0; i < _elements.size(); ++i) {
+    if (!CORBA::is_nil(_elements[i])) {
+      _elements[i]->destroy();
+    }
+  }
+  _elements.erase (_elements.begin(), _elements.end());
+  CORBA::release (this);
 }
 
 void
@@ -751,6 +771,7 @@ DynAny_impl::next ()
     if ((CORBA::ULong)_index+1 == _elements.size())
 	return FALSE;
     ++_index;
+    update_element (_index);
     return TRUE;
 }
 
@@ -992,10 +1013,16 @@ DynFixed_impl::get_value ()
     CORBA::Boolean r = (_value >>= CORBA::Any::to_fixed (f, digits, scale));
     assert (r);
 
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+    ostringstream ostr;
+    ostr << f;
+    char *out = CORBA::string_dup (ostr.str().c_str());
+#else
     ostrstream ostr;
     ostr << f << ends;
     char *out = CORBA::string_dup (ostr.str());
     ostr.rdbuf()->freeze (0);
+#endif
 
     return out;
 }
@@ -1008,7 +1035,12 @@ DynFixed_impl::set_value (const char *in)
 
     Fixed f (digits, scale);
 
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+    string instr (in);
+    istringstream istr (instr);
+#else
     istrstream istr (in);
+#endif
     // XXX check for bad input string...
     istr >> f;
 
@@ -2207,8 +2239,7 @@ DynValue_impl::DynValue_impl (CORBA::TypeCode_ptr tc)
     if (tc->kind() != CORBA::tk_value)
 	mico_throw (DynamicAny::DynAnyFactory::InconsistentTypeCode ());
     for (CORBA::ULong i = 0; i < tc->member_count_inherited(); ++i) {
-	CORBA::TypeCode_var memtc = tc->member_type_inherited(i);
-	_elements.push_back(_factory()->create_dyn_any_from_type_code (memtc));
+	_elements.push_back (DynamicAny::DynAny::_nil());
     }
     _index = -1;
 }
@@ -2229,8 +2260,7 @@ DynValue_impl::DynValue_impl (const CORBA::Any &a)
     if (is_ref) {
 
       for (CORBA::ULong i = 0; i < tc->member_count_inherited(); ++i) {
-	CORBA::TypeCode_var memtc = tc->member_type_inherited(i);
-	_elements.push_back(_factory()->create_dyn_any_from_type_code (memtc));
+	_elements.push_back (DynamicAny::DynValue::_nil ());
       }
 
       if (!value_id) {
@@ -2271,6 +2301,17 @@ DynValue_impl::~DynValue_impl ()
 {
 }
 
+void
+DynValue_impl::update_element (CORBA::Long idx)
+{
+  assert(idx >= 0);
+  if ((CORBA::ULong)idx < _type->member_count_inherited() &&
+      CORBA::is_nil (_elements[idx])) {
+    CORBA::TypeCode_var member_tc = _type->member_type_inherited (idx);
+    _elements[idx] = _factory()->create_dyn_any_from_type_code (member_tc);
+  }
+}
+
 void
 DynValue_impl::from_any (const CORBA::Any& a)
 {
@@ -2328,6 +2369,7 @@ DynValue_impl::to_any ()
     assert (r);
 
     for (CORBA::ULong i = 0; i < _elements.size(); ++i) {
+        update_element (i);
 	CORBA::Any_var el = _elements[i]->to_any ();
 	r = a->any_put (el.inout());
 	assert (r);
@@ -2368,6 +2410,7 @@ DynValue_impl::get_members ()
     seq->length (_elements.size());
     CORBA::TypeCode_ptr tc = _type->unalias ();
     for (CORBA::ULong i = 0; i < _elements.size(); ++i) {
+        update_element (i);
 	(*seq)[i].id = tc->member_name_inherited (i);
 	CORBA::Any_var a = _elements[i]->to_any ();
 	(*seq)[i].value = a.in();
@@ -2403,6 +2446,7 @@ DynValue_impl::get_members_as_dyn_any ()
     seq->length (_elements.size());
     CORBA::TypeCode_ptr tc = _type->unalias ();
     for (CORBA::ULong i = 0; i < _elements.size(); ++i) {
+        update_element (i);
 	(*seq)[i].id = tc->member_name_inherited (i);
 	(*seq)[i].value = _elements[i]->copy();
     }
