@@ -20,18 +20,39 @@
  *  http://www.mico.org/
  */
 
+#ifdef FAST_PCH
+#include "orb_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #define MICO_CONF_IMR
 
 #include <CORBA-SMALL.h>
-#ifndef _WINDOWS
+#ifndef _WIN32
 #include <string.h>
 #endif
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <iostream>
+#include <sstream>
+#else
 #include <iostream.h>
 #include <strstream.h>
+#endif
 #include <mico/throw.h>
 #include <mico/impl.h>
 #include <mico/template_impl.h>
+#if defined(HAVE_BACKTRACE) && defined(HAVE_BACKTRACE_SYMBOLS)
+#include <execinfo.h>
+#endif // HAVE_BACKTRACE && HAVE_BACKTRACE_SYMBOLS
 
+#endif // FAST_PCH
+
+
+using namespace std;
 
 /*************************** Exception ***************************/
 
@@ -39,18 +60,35 @@ void (*CORBA::Exception::_old_terminate_handler) () = NULL;
 
 #ifndef HAVE_STD_EH
 
-#ifdef HAVE_NAMESPACE
 namespace CORBA {
   char *Exception::_last_except_desc = 0;
 };
-#else
-char *CORBA::Exception::_last_except_desc = 0;
-#endif
 
 #endif
 
 CORBA::Exception::Exception ()
 {
+    if (MICO::Logger::IsLogged(MICO::Logger::Exception)) {
+	MICOMT::AutoDebugLock dlock;
+	MICO::Logger::Stream(MICO::Logger::Exception)
+	    << "CORBA::Exception::ctor called" << endl;
+#if defined(HAVE_BACKTRACE) && defined(HAVE_BACKTRACE_SYMBOLS)
+	const int max_length = 500;
+	void* array[max_length];
+	size_t size = backtrace(array, max_length);
+	char** strings = backtrace_symbols(array, size);
+	MICO::Logger::Stream(MICO::Logger::Exception)
+	    << "backtrace length: " << size << endl;
+	for (size_t i=0; i<size; i++) {
+	    MICO::Logger::Stream(MICO::Logger::Exception)
+		<< strings[i] << endl;
+	}
+	free (strings);
+#else // HAVE_BACKTRACE && HAVE_BACKTRACE_SYMBOLS
+	MICO::Logger::Stream(MICO::Logger::Exception)
+	    << "ERROR: backtrace support functions are not available" << endl;
+#endif // HAVE_BACKTRACE && HAVE_BACKTRACE_SYMBOLS
+    }
 }
 
 CORBA::Exception::Exception (const Exception &e)
@@ -116,14 +154,23 @@ CORBA::Exception::_clone () const
 void
 CORBA::Exception::_prepare_throw () const
 {
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+    ostringstream ostr;
+#else
     ostrstream ostr;
+#endif
     _print (ostr);
-    ostr << "" << ends;
 
     if (_last_except_desc)
 	CORBA::string_free (_last_except_desc);
+
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+    _last_except_desc = CORBA::string_dup (ostr.str().c_str());
+#else
+    ostr << "" << ends;
     _last_except_desc = CORBA::string_dup (ostr.str());
     ostr.rdbuf()->freeze (0);
+#endif
 }
 
 #endif
@@ -154,7 +201,7 @@ CORBA::Exception::_throw_failed (const Exception *ex)
     ex->_print (cerr);
     cerr << endl;
 #ifdef _POCKET_PC
-	NOT_IMPLEMENTED;
+	MICO_NOT_IMPLEMENTED;
 #else
     abort ();
 #endif
@@ -578,10 +625,9 @@ CORBA::UnknownUserException::_except_repoid ()
 {
     if (_excpt) {
 	String_var repoid;
-	CORBA::Boolean r =
-	    _excpt->except_get_begin (repoid.out()) &&
-	    _excpt->except_get_end ();
+	CORBA::Boolean r = _excpt->except_get_begin (repoid.out());
 	assert (r);
+	_excpt->rewind ();
 	_ex_repoid = repoid;
     } else {
 	assert (_dc);
