@@ -20,9 +20,19 @@
  *  http://www.mico.org/
  */
 
+#ifdef FAST_PCH
+#include "orb_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #include <CORBA-SMALL.h>
 #include <mico/template_impl.h>
 
+#endif // FAST_PCH
 
 // workaround for "define char *string" in some g++ versions ...
 #define string __ex_string
@@ -41,6 +51,9 @@
 #include <terminate.h>
 #endif
 
+
+using namespace std;
+
 /*************************** Exception ***************************/
 
 void
@@ -195,7 +208,7 @@ CORBA::name::name (ULong minorval, CompletionStatus status)		  \
 CORBA::name &								  \
 CORBA::name::operator= (const name &ex)					  \
 {									  \
-    MICO_SCOPE(CORBA,SystemException::operator=) (ex);			  \
+    CORBA::SystemException::operator=(ex);			  \
     return *this;							  \
 }									  \
 									  \
