@@ -20,11 +20,38 @@
  *  http://www.mico.org/
  */
 
+#ifdef FAST_PCH
+#include "orb_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #include <CORBA-SMALL.h>
-#include <mico/template_impl.h>
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <iostream>
+#else
 #include <iostream.h>
+#endif
 #include <ctype.h>
 #include <mico/os-math.h>
+#include <mico/template_impl.h>
+
+#endif // FAST_PCH
+
+
+using namespace std;
+
+/*
+ * On AIX, the div() function is implemented as a macro, which screws
+ * our definition of div below.
+ */
+
+#ifdef div
+#undef div
+#endif
 
 /*
  * one would usually use pow(10,x) but for some strange
@@ -167,7 +194,7 @@ FixedBase::FixedBase ()
 FixedBase::FixedBase (MICO_Long v)
 {
     char buf[100];
-    sprintf (buf, "%i", v);
+    sprintf (buf, "%i", (int)v);
     compute_params (buf, _digits, _scale);
     _val = v * power10 (_scale);
     adjust ();
@@ -176,7 +203,7 @@ FixedBase::FixedBase (MICO_Long v)
 FixedBase::FixedBase (MICO_ULong v)
 {
     char buf[100];
-    sprintf (buf, "%u", v);
+    sprintf (buf, "%u", (unsigned int)v);
     compute_params (buf, _digits, _scale);
     _val = v * power10 (_scale);
     adjust ();
@@ -186,7 +213,7 @@ FixedBase::FixedBase (MICO_LongLong v)
 {
     char buf[100];
     // XXX
-    sprintf (buf, "%i", (MICO_Long)v);
+    sprintf (buf, "%i", (int)v);
     compute_params (buf, _digits, _scale);
     _val = v * power10 (_scale);
     adjust ();
@@ -196,14 +223,9 @@ FixedBase::FixedBase (MICO_ULongLong v)
 {
     char buf[100];
     // XXX
-    sprintf (buf, "%u", (MICO_ULong)v);
+    sprintf (buf, "%u", (unsigned int)v);
     compute_params (buf, _digits, _scale);
-#ifdef _WINDOWS
-    // VC++ 5.0 does not support unsigned __int64
-    _val = ((MICO_LongLong)v) * power10 (_scale);
-#else
     _val = v * power10 (_scale);
-#endif
     adjust ();
 }
 
@@ -212,7 +234,7 @@ FixedBase::FixedBase (MICO_Double v)
     MICO_Double v2 = (v < 0 ? -v : v) + epsilon (v);
 
     char buf[100];
-    sprintf (buf, "%.31le", v2);
+    sprintf (buf, "%.31e", v2);
     compute_params (buf, _digits, _scale, digits());
     _val = v * power10 (_scale);
     adjust ();
