@@ -20,13 +20,28 @@
  *  http://www.mico.org/
  */
 
+#ifdef FAST_PCH
+#include "orb_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #define MICO_CONF_INTERCEPT
+#define MICO_CONF_POA
+#define MICO_CONF_IR
 
 #include <CORBA-SMALL.h>
 #include <string.h>
 #include <mico/throw.h>
 #include <mico/template_impl.h>
 
+#endif // FAST_PCH
+
+
+using namespace std;
 
 template<class T>
 static inline void
@@ -347,14 +362,9 @@ Interceptor::LWServerRequest::request ()
 /************************** ClientInterceptor *****************************/
 
 
-#ifdef HAVE_NAMESPACE
 namespace Interceptor {
   list<Interceptor::ClientInterceptor_ptr> *ClientInterceptor::_interceptors;
-};
-#else
-list<Interceptor::ClientInterceptor_ptr> *
-    Interceptor::ClientInterceptor::_interceptors;
-#endif
+}
 
 list<Interceptor::ClientInterceptor_ptr> &
 Interceptor::ClientInterceptor::_ics ()
@@ -369,7 +379,7 @@ Interceptor::ClientInterceptor::ClientInterceptor ()
 }
 
 Interceptor::ClientInterceptor::ClientInterceptor (Priority p)
-    : MICO_SCOPE(Interceptor,Root) (p)
+    : Interceptor::Root(p)
 {
     insert_interceptor (_ics(), this);
 }
@@ -566,14 +576,9 @@ Interceptor::ClientInterceptor::_create_request (
 /************************** ServerInterceptor *****************************/
 
 
-#ifdef HAVE_NAMESPACE
 namespace Interceptor {
   list<Interceptor::ServerInterceptor_ptr> *ServerInterceptor::_interceptors;
-};
-#else
-list<Interceptor::ServerInterceptor_ptr> *
-    Interceptor::ServerInterceptor::_interceptors;
-#endif
+}
 
 list<Interceptor::ServerInterceptor_ptr> &
 Interceptor::ServerInterceptor::_ics ()
@@ -588,7 +593,7 @@ Interceptor::ServerInterceptor::ServerInterceptor ()
 }
 
 Interceptor::ServerInterceptor::ServerInterceptor (Priority p)
-    : MICO_SCOPE(Interceptor,Root) (p)
+    : Interceptor::Root(p)
 {
     insert_interceptor (_ics(), this);
 }
@@ -785,14 +790,9 @@ Interceptor::ServerInterceptor::_create_request (
 /************************** InitInterceptor *****************************/
 
 
-#ifdef HAVE_NAMESPACE
 namespace Interceptor {
   list<Interceptor::InitInterceptor_ptr> *InitInterceptor::_interceptors;
-};
-#else
-list<Interceptor::InitInterceptor_ptr> *
-    Interceptor::InitInterceptor::_interceptors;
-#endif
+}
 
 list<Interceptor::InitInterceptor_ptr> &
 Interceptor::InitInterceptor::_ics ()
@@ -807,7 +807,7 @@ Interceptor::InitInterceptor::InitInterceptor ()
 }
 
 Interceptor::InitInterceptor::InitInterceptor (Priority p)
-    : MICO_SCOPE(Interceptor,Root) (p)
+    : Interceptor::Root(p)
 {
     insert_interceptor (_ics(), this);
 }
@@ -883,14 +883,9 @@ Interceptor::InitInterceptor::_exec_initialize (CORBA::ORB_ptr orb,
 /************************** BOAInterceptor *****************************/
 
 
-#ifdef HAVE_NAMESPACE
 namespace Interceptor {
   list<Interceptor::BOAInterceptor_ptr> *BOAInterceptor::_interceptors;
-};
-#else
-list<Interceptor::BOAInterceptor_ptr> *
-    Interceptor::BOAInterceptor::_interceptors;
-#endif
+}
 
 list<Interceptor::BOAInterceptor_ptr> &
 Interceptor::BOAInterceptor::_ics ()
@@ -905,7 +900,7 @@ Interceptor::BOAInterceptor::BOAInterceptor ()
 }
 
 Interceptor::BOAInterceptor::BOAInterceptor (Priority p)
-    : MICO_SCOPE(Interceptor,Root) (p)
+    : Interceptor::Root(p)
 {
     insert_interceptor (_ics(), this);
 }
@@ -1033,14 +1028,9 @@ Interceptor::BOAInterceptor::_exec_create (CORBA::Object_ptr obj)
 /************************** ConnInterceptor *****************************/
 
 
-#ifdef HAVE_NAMESPACE
 namespace Interceptor {
   list<Interceptor::ConnInterceptor_ptr> *ConnInterceptor::_interceptors;
-};
-#else
-list<Interceptor::ConnInterceptor_ptr> *
-    Interceptor::ConnInterceptor::_interceptors;
-#endif
+}
 
 list<Interceptor::ConnInterceptor_ptr> &
 Interceptor::ConnInterceptor::_ics ()
@@ -1055,7 +1045,7 @@ Interceptor::ConnInterceptor::ConnInterceptor ()
 }
 
 Interceptor::ConnInterceptor::ConnInterceptor (Priority p)
-    : MICO_SCOPE(Interceptor,Root) (p)
+    : Interceptor::Root(p)
 {
     insert_interceptor (_ics(), this);
 }
