@@ -20,20 +20,46 @@
  *  http://www.mico.org/
  */
 
+#ifdef FAST_PCH
+#include "orb_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #define MICO_CONF_IMR
 
+// Needed by CSIv2 support
+#define MICO_CONF_INTERCEPT
+#define MICO_CONF_IR
+#define MICO_CONF_POA // needed for csi_base.h
+
 #include <CORBA-SMALL.h>
-#ifndef _WINDOWS
+//#include <CORBA.h>
+#ifndef _WIN32
 #include <string.h>
 #endif
 #include <ctype.h>
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <iostream>
+#else
 #include <iostream.h>
+#endif
 #include <mico/impl.h>
 #include <mico/template_impl.h>
 #include <mico/util.h>
 #include <mico/ssl.h>
 #include <mico/os-misc.h>
+#ifdef USE_CSIV2
+#include <mico/security/csiv2_impl.h>
+#endif // USE_CSIV2
 
+#endif // FAST_PCH
+
+
+using namespace std;
 
 /************************* misc dtors *******************************/
 
@@ -88,9 +114,6 @@ CORBA::IOR::IOR (const char *_ior)
 
 CORBA::IOR::IOR (const IOR &ior)
 {
-  _active_profile = 0;
-  _active_profile_index = 0;
-  _addressing_disposition = 0;
     copy (ior);
 }
 
@@ -110,8 +133,6 @@ CORBA::IOR &
 CORBA::IOR::operator= (const IOR &ior)
 {
     if (this != &ior) {
-      _active_profile_index = 0;
-      _addressing_disposition = 0;
 	free ();
 	copy (ior);
     }
@@ -139,12 +160,7 @@ CORBA::IOR::from_string (const char *_ior)
 	    return FALSE;
 	b.put ((mico_from_xdigit (ior[i]) << 4) | mico_from_xdigit (ior[i+1]));
     }
-#ifdef _WINDOWS
-    MICO::CDRDecoder dc (&b, FALSE, CORBA::DefaultEndian,
-			 0, TRUE, 0, TRUE);
-#else
     MICO::CDRDecoder dc (&b, FALSE);
-#endif
 
     /*
      * XXX [10-7] says we should encaps it, but *not* as sequence<octet>,
@@ -220,10 +236,6 @@ CORBA::IOR::operator< (const IOR &ior) const
 void
 CORBA::IOR::free ()
 {
-  _active_profile = 0;
-  _active_profile_index = 0;
-  _addressing_disposition = 0;
-
     for (ULong i = 0; i < tags.size(); ++i)
 	delete tags[i];
     tags.erase (tags.begin(), tags.end());
@@ -236,6 +248,9 @@ CORBA::IOR::copy (const IOR &ior)
     for (ULong i = 0; i < ior.tags.size(); ++i)
 	add_profile (ior.tags[i]->clone());
     oid = ior.oid;
+    _active_profile_index = ior._active_profile_index;
+    _active_profile = get_profile (_active_profile_index);
+    _addressing_disposition = ior._addressing_disposition;
 }
 
 #define check(exp) if (!(exp)) goto bad
@@ -509,13 +524,9 @@ CORBA::IOR::print (ostream &o) const
 /****************************** IORProfile *****************************/
 
 
-#ifdef HAVE_NAMESPACE
 namespace CORBA {
   vector<CORBA::IORProfileDecoder *> *IORProfile::decoders = 0;
-};
-#else
-vector<CORBA::IORProfileDecoder *> *CORBA::IORProfile::decoders = 0;
-#endif
+}
 
 void
 CORBA::IORProfile::copy (IORProfile *p)
@@ -692,7 +703,7 @@ MICO::LocalProfile::objectkey (CORBA::Long &l) const
 CORBA::Boolean
 MICO::LocalProfile::reachable ()
 {
-#ifdef _WINDOWS
+#ifdef _WIN32
     assert( 0 );
     return true;
 #else
@@ -802,11 +813,7 @@ MICO::LocalProfileDecoder::has_id (CORBA::IORProfile::ProfileId id) const
     return tagid == id;
 }
 
-#ifdef _WINDOWS
-static MICO::LocalProfileDecoder local_ior_decoder (CORBA::IORProfile::TAG_LOCAL);
-#else
 static MICO::LocalProfileDecoder local_ior_decoder;
-#endif
 
 
 /****************************** IIOPProfile *****************************/
@@ -1059,6 +1066,25 @@ MICO::IIOPProfileDecoder::decode (CORBA::DataDecoder &dc, ProfileId,
 	    ip = new MICOSSL::SSLProfile (ip, sa);
 	}
 #endif
+#ifdef HAVE_SSL
+#ifdef USE_CSIV2
+	CORBA::Component* __t_comp = NULL;
+	__t_comp = comps.component(CSIIOP::TAG_CSI_SEC_MECH_LIST);
+	CSIv2::Component* __c = NULL;
+	__c = dynamic_cast<CSIv2::Component*>(__t_comp);
+	if (__c != NULL) {
+  	    CSIIOP::CompoundSecMechList* __list = __c->mech_list();
+	    if (__list->mechanism_list[0].transport_mech.tag == CSIIOP::TAG_TLS_SEC_TRANS) {
+		// CSIv2 uses TLS support
+		if (MICO::Logger::IsLogged(MICO::Logger::Security))
+		    MICO::Logger::Stream(MICO::Logger::Security)
+			<< "CSIv2(ior.cc): decoding IIOP profile with CSIv2 TLS component." << endl;
+		MICOSSL::SSLAddress sa (ip->addr()->clone());
+		ip = new MICOSSL::SSLProfile (ip, sa);
+	    }
+	}
+#endif // USE_CSIV2
+#endif // HAVE_SSL
     }
     check (dc.struct_end ());
     return ip;
@@ -1699,13 +1725,9 @@ CORBA::ComponentDecoder::~ComponentDecoder ()
 /****************************** Component *****************************/
 
 
-#ifdef HAVE_NAMESPACE
 namespace CORBA {
   vector<CORBA::ComponentDecoder *> *Component::decoders = 0;
-};
-#else
-vector<CORBA::ComponentDecoder *> *CORBA::Component::decoders = 0;
-#endif
+}
 
 CORBA::Component::~Component ()
 {
@@ -1846,14 +1868,14 @@ void
 MICO::UnknownComponent::print (ostream &o) const
 {
   char buf[10];
-  int j, k;
+  CORBA::ULong j, k;
 
   o << "Unknown Component" << endl;
   o << "              Tag Id:  " << tagid << endl;
   o << "                 Tag:  ";
 
   for (j=0; j<tagdata.size(); j+=8) {
-    for (k=0; k<j+8 && k<tagdata.size(); k++) {
+    for (k=j; k<j+8 && k<tagdata.size(); k++) {
       sprintf (buf, "%02x ", (int) tagdata[k]);
       o << buf;
     }
@@ -2074,9 +2096,4 @@ MICO::MultiCompProfileDecoder::has_id (ProfileId id) const
     return id == _tagid;
 }
 
-#ifdef _WINDOWS
-static MICO::MultiCompProfileDecoder
-    multicomp_profile_decoder (CORBA::IORProfile::TAG_MULTIPLE_COMPONENTS);
-#else
 static MICO::MultiCompProfileDecoder multicomp_profile_decoder;
-#endif
