@@ -20,20 +20,37 @@
  *  http://www.mico.org/
  */
 
+#ifdef FAST_PCH
+#include "orb_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #define MICO_CONF_IMR
 
 #include <CORBA-SMALL.h>
-#ifndef _WINDOWS
+#ifndef _WIN32
 #include <string.h>
 #endif
-#include <iostream.h>
 #include <mico/template_impl.h>
 #include <mico/util.h>
 #include <mico/throw.h>
 #ifdef USE_MEMTRACE
 #include <mico/memtrace.h>
 #endif
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <sstream>
+#else // HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <strstream.h>
+#endif // HAVE_ANSI_CPLUSPLUS_HEADERS
 
+#endif // FAST_PCH
+
+
+using namespace std;
 
 /*************************** MagicChecker ****************************/
 
@@ -87,6 +104,7 @@ CORBA::ServerlessObject::~ServerlessObject ()
 void
 CORBA::ServerlessObject::_ref ()
 {
+    MICOMT::AutoLock lock(refslock);
     _check ();
     ++refs;
 }
@@ -94,6 +112,7 @@ CORBA::ServerlessObject::_ref ()
 CORBA::Boolean
 CORBA::ServerlessObject::_deref ()
 {
+    MICOMT::AutoLock lock(refslock);
     return _check_nothrow() && --refs <= 0;
 }
 
@@ -113,8 +132,7 @@ CORBA::Object::Object (IOR *i)
 {
     ior = i;
     fwd_ior = 0;
-    orb = CORBA::ORB_instance ("mico-local-orb");
-    assert (orb);
+    orb = CORBA::ORB::_nil();
 }
 
 CORBA::Object::Object (const Object &o)
@@ -178,7 +196,7 @@ CORBA::Object::_setup_domains (CORBA::Object_ptr parent)
 {
     if (CORBA::is_nil (parent)) {
 	CORBA::DomainManager_var def_manager;
-	orb->get_default_domain_manager (def_manager);
+	this->_orbnc()->get_default_domain_manager (def_manager);
 
 	_managers.length (1);
 
@@ -416,26 +434,88 @@ CORBA::Object::_hash (ULong max)
     return mico_string_hash (s.c_str(), max);
 }
 
-void *
-CORBA::LocalObject::_narrow_helper (const char * _repoid)
+CORBA::ORB_ptr
+CORBA::Object::_orbnc()
 {
-  if (strcmp (_repoid, "IDL:omg.org/CORBA/LocalObject:1.0") == 0) {
-    return (void *) this;
-  }
-
-  return Object::_narrow_helper (_repoid);
+    if (CORBA::is_nil(orb))
+	orb = CORBA::ORB_instance ("mico-local-orb");
+    return orb;
 }
 
-CORBA::LocalObject_ptr
-CORBA::LocalObject::_narrow (Object_ptr o)
+// ref-counting added in ptc/03-03-09
+
+void
+CORBA::LocalObject::_add_ref()
 {
-  if (!CORBA::is_nil (o)) {
-    void *_p;
-
-    if ((_p = o->_narrow_helper ("IDL:omg.org/CORBA/LocalObject:1.0"))) {
-      return _duplicate ((LocalObject_ptr) _p);
-    }
-  }
-
-  return _nil ();
+    this->_ref();
 }
+
+void
+CORBA::LocalObject::_remove_ref()
+{
+    if (this->_deref())
+	delete this;
+}
+
+CORBA::ULong
+CORBA::LocalObject::_refcount_value() const
+{
+    return this->_refcnt();
+}
+
+// non-implemented methods of CORBA::Object
+
+CORBA::InterfaceDef*
+CORBA::LocalObject::_get_interface()
+{
+    mico_throw(CORBA::NO_IMPLEMENT());
+    // never reached, just to avoid warning
+    return NULL;
+}
+
+CORBA::DomainManagerList*
+CORBA::LocalObject::_get_domain_managers()
+{
+    mico_throw(CORBA::NO_IMPLEMENT());
+    // never reached, just to avoid warning
+    return NULL;
+}
+
+CORBA::Policy_ptr
+CORBA::LocalObject::_get_policy(PolicyType __policy_type)
+{
+    mico_throw(CORBA::NO_IMPLEMENT());
+    // never reached, just to avoid warning
+    return CORBA::Policy::_nil();
+}
+
+CORBA::Object_ptr
+CORBA::LocalObject::_set_policy_overrides
+(const PolicyList& __policies, SetOverrideType __set_add)
+{
+    mico_throw(CORBA::NO_IMPLEMENT());
+    // never reached, just to avoid warning
+    return CORBA::Object::_nil();
+}
+
+CORBA::ULong
+CORBA::LocalObject::_hash(CORBA::ULong __max)
+{
+    // we simply hash string which represent
+    // address of this object e.g '0xbffff5e8'
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+    stringstream __ss;
+#else // HAVE_ANSI_CPLUSPLUS_HEADERS
+    ostrstream __ss;
+#endif // HAVE_ANSI_CPLUSPLUS_HEADERS
+    __ss << hex << (void*)this;
+    string __str = __ss.str();
+    return mico_string_hash(__str.c_str(), __max);
+}
+
+CORBA::Boolean
+CORBA::LocalObject::_is_equivalent(CORBA::Object_ptr __obj)
+{
+    return this == __obj;
+}
+
