@@ -25,17 +25,10 @@
 #include <mico/template_impl.h>
 
 // begin-idl-hack
-#ifdef HAVE_NAMESPACE
 namespace PortableServer {
     POA_ptr _the_root_poa = POA::_nil();
     Current_ptr _the_poa_current = Current::_nil();
-};
-#else
-PortableServer::POA_ptr
-PortableServer::_the_root_poa = PortableServer::POA::_nil();
-PortableServer::Current_ptr
-PortableServer::_the_poa_current = PortableServer::Current::_nil();
-#endif
+}
 
 /*
  * ServantBase implementation
@@ -110,8 +103,8 @@ PortableServer::ServantBase::_this ()
 {
   if (!CORBA::is_nil (_the_poa_current) &&
       _the_poa_current->iscurrent() &&
-      _the_poa_current->get_serv() == (Servant) this) {
-    return _the_poa_current->make_ref ();
+      _the_poa_current->get_servant() == (Servant) this) {
+    return _the_poa_current->get_reference ();
   }
 
   PortableServer::POA_var poa;
@@ -197,6 +190,10 @@ PortableServer::RefCountServantBase::operator= (const RefCountServantBase &)
 void
 PortableServer::RefCountServantBase::_add_ref ()
 {
+  MICOMT::AutoLock l(_ref_lock);
+
+  assert( _ref_count != 0 );
+
   _check ();
   _ref_count++;
 }
@@ -204,10 +201,14 @@ PortableServer::RefCountServantBase::_add_ref ()
 void
 PortableServer::RefCountServantBase::_remove_ref ()
 {
+  _ref_lock.lock();
   _check ();
   if (--_ref_count == 0) {
+    _ref_lock.unlock ();
     delete this;
   }
+  else
+    _ref_lock.unlock ();
 }
 
 void
@@ -235,19 +236,19 @@ PortableServer::ValueRefCountBase::~ValueRefCountBase ()
 void
 PortableServer::ValueRefCountBase::_add_ref ()
 {
-  MICO_SCOPE(CORBA,DefaultValueRefCountBase)::_add_ref ();
+  CORBA::DefaultValueRefCountBase::_add_ref ();
 }
 
 void
 PortableServer::ValueRefCountBase::_remove_ref ()
 {
-  MICO_SCOPE(CORBA,DefaultValueRefCountBase)::_remove_ref ();
+  CORBA::DefaultValueRefCountBase::_remove_ref ();
 }
 
 CORBA::ULong
 PortableServer::ValueRefCountBase::_refcount_value ()
 {
-  return MICO_SCOPE(CORBA,DefaultValueRefCountBase)::_refcount_value ();
+  return CORBA::DefaultValueRefCountBase::_refcount_value ();
 }
 
 /*
@@ -261,11 +262,11 @@ PortableServer::DynamicImplementation::~DynamicImplementation ()
 CORBA::ServerRequestBase_ptr
 PortableServer::DynamicImplementation::make_request (CORBA::ORBRequest *req,
 						     CORBA::Object_ptr obj,
-						     CORBA::ULong msgid,
+						     CORBA::ORBMsgId id,
 						     CORBA::ObjectAdapter *oa,
 						     CORBA::Principal_ptr pr)
 {
-  return new CORBA::ServerRequest (req, obj, msgid, oa, pr);
+  return new CORBA::ServerRequest (req, obj, id, oa, pr);
 }
 
 void
@@ -277,7 +278,7 @@ PortableServer::DynamicImplementation::doinvoke (CORBA::ServerRequestBase_ptr re
 CORBA::Object_ptr
 PortableServer::DynamicImplementation::_this ()
 {
-  return MICO_SCOPE(PortableServer,ServantBase::_this) ();
+  return PortableServer::ServantBase::_this();
 }
 
 CORBA::Boolean
@@ -294,7 +295,7 @@ PortableServer::DynamicImplementation::_is_a (const char * repoid)
 
   if (!CORBA::is_nil (_the_poa_current) &&
       _the_poa_current->iscurrent() &&
-      _the_poa_current->get_serv() == (Servant) this) {
+      _the_poa_current->get_servant() == (Servant) this) {
     poa = _the_poa_current->get_POA ();
     oid = _the_poa_current->get_object_id ();
   }
@@ -351,7 +352,7 @@ PortableServer::DynamicImplementation::_get_interface ()
 
   if (!CORBA::is_nil (_the_poa_current) &&
       _the_poa_current->iscurrent() &&
-      _the_poa_current->get_serv() == (Servant) this) {
+      _the_poa_current->get_servant() == (Servant) this) {
     poa = _the_poa_current->get_POA ();
     oid = _the_poa_current->get_object_id ();
   }
@@ -401,11 +402,11 @@ PortableServer::StaticImplementation::~StaticImplementation ()
 CORBA::ServerRequestBase_ptr
 PortableServer::StaticImplementation::make_request (CORBA::ORBRequest *req,
 						    CORBA::Object_ptr obj,
-						    CORBA::ULong msgid,
+						    CORBA::ORBMsgId id,
 						    CORBA::ObjectAdapter *oa,
 						    CORBA::Principal_ptr pr)
 {
-  return new CORBA::StaticServerRequest (req, obj, msgid, oa, pr);
+  return new CORBA::StaticServerRequest (req, obj, id, oa, pr);
 }
 
 void
@@ -421,7 +422,7 @@ PortableServer::StaticImplementation::doinvoke (CORBA::ServerRequestBase_ptr req
 
 PortableServer::StubBase::StubBase (PortableServer::POA_ptr poa)
 {
-  _colocated_poa = MICO_SCOPE(PortableServer,POA::_duplicate) (poa);
+  _colocated_poa = PortableServer::POA::_duplicate(poa);
 }
 
 PortableServer::StubBase::StubBase ()
@@ -445,7 +446,7 @@ PortableServer::StubBase::_preinvoke ()
 
   if (serv == NULL) {
     CORBA::release (_colocated_poa);
-    _colocated_poa = MICO_SCOPE(PortableServer,POA::_nil) ();
+    _colocated_poa = PortableServer::POA::_nil();
     return NULL;
   }
 
@@ -552,7 +553,7 @@ PortableServer::wstring_to_ObjectId (const CORBA::WChar * s)
  */
 
 PortableServer::AdapterActivator_stub_clp::AdapterActivator_stub_clp (PortableServer::POA_ptr poa, CORBA::Object_ptr obj)
-  : MICO_SCOPE(PortableServer,StubBase) (poa), MICO_SCOPE(CORBA,Object) (*obj)
+  : CORBA::Object(*obj), PortableServer::StubBase(poa)
 {
 }
 
@@ -594,7 +595,7 @@ PortableServer::AdapterActivator_stub_clp::unknown_adapter( PortableServer::POA_
 }
 
 PortableServer::ServantManager_stub_clp::ServantManager_stub_clp (PortableServer::POA_ptr poa, CORBA::Object_ptr obj)
-  : MICO_SCOPE(PortableServer,StubBase) (poa), MICO_SCOPE(CORBA,Object) (*obj)
+  : CORBA::Object(*obj), PortableServer::StubBase(poa)
 {
 }
 
@@ -603,8 +604,8 @@ PortableServer::ServantManager_stub_clp::~ServantManager_stub_clp ()
 }
 
 PortableServer::ServantActivator_stub_clp::ServantActivator_stub_clp (PortableServer::POA_ptr poa, CORBA::Object_ptr obj)
-  : MICO_SCOPE(PortableServer,StubBase) (poa), MICO_SCOPE(CORBA,Object) (*obj),
-  MICO_SCOPE(PortableServer,ServantManager_stub_clp) (poa, obj)
+  : CORBA::Object(*obj), PortableServer::StubBase(poa), 
+  PortableServer::ServantManager_stub_clp(poa, obj)
 {
 }
 
@@ -676,8 +677,8 @@ PortableServer::ServantActivator_stub_clp::etherealize( const PortableServer::Ob
 }
 
 PortableServer::ServantLocator_stub_clp::ServantLocator_stub_clp (PortableServer::POA_ptr poa, CORBA::Object_ptr obj)
-  : MICO_SCOPE(PortableServer,StubBase) (poa), MICO_SCOPE(CORBA,Object) (*obj),
-  MICO_SCOPE(PortableServer,ServantManager_stub_clp) (poa, obj)
+  : CORBA::Object(*obj), PortableServer::StubBase(poa), 
+  PortableServer::ServantManager_stub_clp(poa, obj)
 {
 }
 
@@ -756,7 +757,7 @@ POA_PortableServer::AdapterActivator::~AdapterActivator()
 PortableServer::AdapterActivator_ptr
 POA_PortableServer::AdapterActivator::_this ()
 {
-  CORBA::Object_var obj = MICO_SCOPE(PortableServer,ServantBase::_this) ();
+  CORBA::Object_var obj = PortableServer::ServantBase::_this();
   return PortableServer::AdapterActivator::_narrow (obj);
 }
 
@@ -844,7 +845,7 @@ POA_PortableServer::ServantManager::~ServantManager()
 PortableServer::ServantManager_ptr
 POA_PortableServer::ServantManager::_this ()
 {
-  CORBA::Object_var obj = MICO_SCOPE(PortableServer,ServantBase::_this) ();
+  CORBA::Object_var obj = PortableServer::ServantBase::_this();
   return PortableServer::ServantManager::_narrow (obj);
 }
 
@@ -932,7 +933,7 @@ POA_PortableServer::ServantActivator::~ServantActivator()
 PortableServer::ServantActivator_ptr
 POA_PortableServer::ServantActivator::_this ()
 {
-  CORBA::Object_var obj = MICO_SCOPE(PortableServer,ServantBase::_this) ();
+  CORBA::Object_var obj = PortableServer::ServantBase::_this();
   return PortableServer::ServantActivator::_narrow (obj);
 }
 
@@ -942,7 +943,7 @@ POA_PortableServer::ServantActivator::_is_a (const char * repoid)
   if (strcmp (repoid, "IDL:omg.org/PortableServer/ServantActivator:1.0") == 0) {
     return TRUE;
   }
-  if (MICO_SCOPE(POA_PortableServer,ServantManager)::_is_a (repoid)) {
+  if (POA_PortableServer::ServantManager::_is_a (repoid)) {
     return TRUE;
   }
   return FALSE;
@@ -982,7 +983,7 @@ POA_PortableServer::ServantActivator::_narrow_helper (const char * repoid)
   if (strcmp (repoid, "IDL:omg.org/PortableServer/ServantActivator:1.0") == 0) {
     return (void *) this;
   }
-  if ((p = MICO_SCOPE(POA_PortableServer,ServantManager)::_narrow_helper (repoid)) != NULL) {
+  if ((p = POA_PortableServer::ServantManager::_narrow_helper (repoid)) != NULL) {
     return p;
   }
   return NULL;
@@ -1026,7 +1027,7 @@ POA_PortableServer::ServantLocator::~ServantLocator()
 PortableServer::ServantLocator_ptr
 POA_PortableServer::ServantLocator::_this ()
 {
-  CORBA::Object_var obj = MICO_SCOPE(PortableServer,ServantBase::_this) ();
+  CORBA::Object_var obj = PortableServer::ServantBase::_this();
   return PortableServer::ServantLocator::_narrow (obj);
 }
 
@@ -1036,7 +1037,7 @@ POA_PortableServer::ServantLocator::_is_a (const char * repoid)
   if (strcmp (repoid, "IDL:omg.org/PortableServer/ServantLocator:1.0") == 0) {
     return TRUE;
   }
-  if (MICO_SCOPE(POA_PortableServer,ServantManager)::_is_a (repoid)) {
+  if (POA_PortableServer::ServantManager::_is_a (repoid)) {
     return TRUE;
   }
   return FALSE;
@@ -1076,7 +1077,7 @@ POA_PortableServer::ServantLocator::_narrow_helper (const char * repoid)
   if (strcmp (repoid, "IDL:omg.org/PortableServer/ServantLocator:1.0") == 0) {
     return (void *) this;
   }
-  if ((p = MICO_SCOPE(POA_PortableServer,ServantManager)::_narrow_helper (repoid)) != NULL) {
+  if ((p = POA_PortableServer::ServantManager::_narrow_helper (repoid)) != NULL) {
     return p;
   }
   return NULL;
