@@ -1,6 +1,6 @@
 /*
  *  MICO --- an Open Source CORBA implementation
- *  Copyright (c) 1997-2001 by The Mico Team
+ *  Copyright (c) 1997-2003 by The Mico Team
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -20,7 +20,19 @@
  *  http://www.mico.org/
  */
 
+#ifdef FAST_PCH
+#include "orb_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #define MICO_CONF_IMR
+#define MICO_CONF_IR
+#define MICO_CONF_INTERCEPT
+#define MICO_CONF_POA
 
 #include <CORBA-SMALL.h>
 #include <string.h>
@@ -29,7 +41,12 @@
 #include <mico/impl.h>
 #include <mico/intercept.h>
 #include <mico/template_impl.h>
+#include <mico/pi_impl.h>
 
+#endif // FAST_PCH
+
+
+using namespace std;
 
 //----------------
 
@@ -1619,7 +1636,6 @@ CORBA::TypeCode_ptr TCSeqAbstractBase::_tc = 0;
 //-------------------
 
 
-#ifdef HAVE_NAMESPACE
 namespace CORBA {
     CORBA::StaticTypeInfo *_stc_void;
     CORBA::StaticTypeInfo *_stc_short;
@@ -1663,51 +1679,7 @@ namespace CORBA {
     CORBA::StaticTypeInfo *_stcseq_Object;
     CORBA::StaticTypeInfo *_stcseq_ValueBase;
     CORBA::StaticTypeInfo *_stcseq_AbstractBase;
-};
-#else
-CORBA::StaticTypeInfo *CORBA::_stc_void;
-CORBA::StaticTypeInfo *CORBA::_stc_short;
-CORBA::StaticTypeInfo *CORBA::_stc_long;
-CORBA::StaticTypeInfo *CORBA::_stc_longlong;
-CORBA::StaticTypeInfo *CORBA::_stc_ushort;
-CORBA::StaticTypeInfo *CORBA::_stc_ulong;
-CORBA::StaticTypeInfo *CORBA::_stc_ulonglong;
-CORBA::StaticTypeInfo *CORBA::_stc_float;
-CORBA::StaticTypeInfo *CORBA::_stc_double;
-CORBA::StaticTypeInfo *CORBA::_stc_longdouble;
-CORBA::StaticTypeInfo *CORBA::_stc_char;
-CORBA::StaticTypeInfo *CORBA::_stc_wchar;
-CORBA::StaticTypeInfo *CORBA::_stc_octet;
-CORBA::StaticTypeInfo *CORBA::_stc_boolean;
-CORBA::StaticTypeInfo *CORBA::_stc_string;
-CORBA::StaticTypeInfo *CORBA::_stc_wstring;
-CORBA::StaticTypeInfo *CORBA::_stc_any;
-CORBA::StaticTypeInfo *CORBA::_stc_TypeCode;
-CORBA::StaticTypeInfo *CORBA::_stc_Principal;
-CORBA::StaticTypeInfo *CORBA::_stc_Context;
-CORBA::StaticTypeInfo *CORBA::_stc_Object;
-CORBA::StaticTypeInfo *CORBA::_stc_ValueBase;
-CORBA::StaticTypeInfo *CORBA::_stc_AbstractBase;
-CORBA::StaticTypeInfo *CORBA::_stcseq_short;
-CORBA::StaticTypeInfo *CORBA::_stcseq_long;
-CORBA::StaticTypeInfo *CORBA::_stcseq_longlong;
-CORBA::StaticTypeInfo *CORBA::_stcseq_ushort;
-CORBA::StaticTypeInfo *CORBA::_stcseq_ulong;
-CORBA::StaticTypeInfo *CORBA::_stcseq_ulonglong;
-CORBA::StaticTypeInfo *CORBA::_stcseq_float;
-CORBA::StaticTypeInfo *CORBA::_stcseq_double;
-CORBA::StaticTypeInfo *CORBA::_stcseq_longdouble;
-CORBA::StaticTypeInfo *CORBA::_stcseq_char;
-CORBA::StaticTypeInfo *CORBA::_stcseq_wchar;
-CORBA::StaticTypeInfo *CORBA::_stcseq_octet;
-CORBA::StaticTypeInfo *CORBA::_stcseq_boolean;
-CORBA::StaticTypeInfo *CORBA::_stcseq_any;
-CORBA::StaticTypeInfo *CORBA::_stcseq_string;
-CORBA::StaticTypeInfo *CORBA::_stcseq_wstring;
-CORBA::StaticTypeInfo *CORBA::_stcseq_Object;
-CORBA::StaticTypeInfo *CORBA::_stcseq_ValueBase;
-CORBA::StaticTypeInfo *CORBA::_stcseq_AbstractBase;
-#endif
+}
 
 static struct InitStaticTypeInfo {
     InitStaticTypeInfo()
@@ -1767,7 +1739,7 @@ CORBA::StaticRequest::StaticRequest (CORBA::Object_ptr obj, const char *opname)
 	// locality constrained object
 	mico_throw (NO_IMPLEMENT());
 
-    _msgid = 0;
+    _id = 0;
     _opname = opname;
     _res = 0;
     _ctx = 0;
@@ -1775,18 +1747,20 @@ CORBA::StaticRequest::StaticRequest (CORBA::Object_ptr obj, const char *opname)
     _ctx_list = 0;
     _obj = CORBA::Object::_duplicate (obj);
 
-    _iceptreq = Interceptor::ClientInterceptor::_create_request (
-	_obj, _opname, *context(), CORBA::Request::_nil());
+    _iceptreq = Interceptor::ClientInterceptor::_create_request
+	(_obj, _opname, *context(), CORBA::Request::_nil());
+    _cri = PInterceptor::PI::_create_cri(_obj, _opname);
 }
 
 CORBA::StaticRequest::~StaticRequest ()
 {
-    if (_msgid)
-        _obj->_orbnc()->cancel (_msgid);
+    if (_id)
+        _obj->_orbnc()->cancel (_id);
     CORBA::release (_ctx);
     CORBA::release (_env);
     CORBA::release (_ctx_list);
     CORBA::release (_iceptreq);
+    CORBA::release (_cri);
 }
 
 const char *
@@ -2143,58 +2117,129 @@ CORBA::StaticRequest::invoke ()
     CORBA::Object_var obj;
     CORBA::ORBRequest *dummy;
     CORBA::Boolean done = FALSE;
-    CORBA::ULong msgid;
+    CORBA::ORBMsgId id;
 
     // no pending invocations please ...
-    assert (!_msgid);
-
+    assert (!_id);
+//      assert (!_msgid);
+//      msgid = orb->new_msgid();
+    id = orb->new_orbid();
+    // here the point for PI "send_request()"
+    // set request_id = _msgid, arguments = _args, response_expected = TRUE,
+    // sync_scope = SYNC_NONE, etc.
+    // we need to set contexts before seting operation_context!
+    // because right value of operation_context is computed with help
+    // of contexts!
+    PInterceptor::PI::_send_request_ip
+	(_cri, CORBA::ORB::get_msgid(id), _args, this->ctx_list(), this->ctx(),
+	 this->context());
     if (_iceptreq && !Interceptor::ClientInterceptor::
 	_exec_initialize_request ((Interceptor::LWRequest_ptr)_iceptreq,
 				  env())) {
 	return;
     }
 
-    msgid = orb->invoke_async (_obj, this, CORBA::Principal::_nil());
+    id = orb->invoke_async (_obj, this, CORBA::Principal::_nil(),
+			    TRUE, 0, id);
+//      msgid = orb->invoke_async (_obj, this, CORBA::Principal::_nil(),
+//  			       TRUE, 0, msgid);// we got _msgid already
 
     if (_iceptreq && !Interceptor::ClientInterceptor::
 	_exec_after_marshal ((Interceptor::LWRequest_ptr)_iceptreq,
 			     env())) {
-	orb->cancel (msgid);
+	orb->cancel (id);
 	return;
     }
     while (!done) {
-	CORBA::Boolean ret = orb->wait (msgid);
+	CORBA::Boolean ret = orb->wait (id);
 	assert (ret);
 
 	// XXX called multiple times in case of Forward
 	if (_iceptreq && !Interceptor::ClientInterceptor::
 	    _exec_before_unmarshal ((Interceptor::LWRequest_ptr)_iceptreq,
 				    env())) {
-	    orb->cancel (msgid);
+	    orb->cancel (id);
 	    return;
 	}
 
 	GIOP::AddressingDisposition ad;
-	CORBA::InvokeStatus rs = orb->get_invoke_reply (msgid, obj, dummy, ad);
+	CORBA::InvokeStatus rs = orb->get_invoke_reply (id, obj, dummy, ad);
+//  	CORBA::InvokeStatus rs = orb->get_invoke_reply (msgid, obj, dummy, ad);
+
+	CORBA::Any r;
+
 	switch (rs) {
 	case CORBA::InvokeForward:
 	    // XXX what if _obj is not a stub ???
 	    assert (_obj.in() && obj.in());
 	    _obj->_forward (obj);
-	    msgid = orb->invoke_async (_obj, this, Principal::_nil());
+	    // receive other
+	    PInterceptor::PI::_receive_other_ip
+		(_cri, PortableInterceptor::LOCATION_FORWARD, _obj,
+		 dummy->context());
+	    CORBA::release(_cri);
+//  	    msgid = orb->new_msgid();
+	    id = orb->new_orbid();
+	    _cri = PInterceptor::PI::_create_cri(_obj, _opname);
+	    PInterceptor::PI::_send_request_ip
+		(_cri, CORBA::ORB::get_msgid(id), _args, this->ctx_list(),
+		 this->ctx(), this->context());
+//  	    msgid = orb->invoke_async (_obj, this, Principal::_nil(),
+//  				       TRUE, 0, msgid);
+	    id = orb->invoke_async (_obj, this, Principal::_nil(),
+				    TRUE, 0, id);
 	    break;
 
 	case CORBA::InvokeAddrDisp:
-	  _obj->_ior_fwd()->addressing_disposition (ad);
-	  msgid = orb->invoke_async (_obj, this, Principal::_nil());
-	  break;
+	    // receive other
+	    _obj->_ior_fwd()->addressing_disposition (ad);
+	    PInterceptor::PI::_receive_other_ip
+		(_cri, PortableInterceptor::TRANSPORT_RETRY, dummy->context());
+	    CORBA::release(_cri);
+//  	    msgid = orb->new_msgid();
+	    id = orb->new_orbid();
+	    _cri = PInterceptor::PI::_create_cri(_obj, _opname);
+	    PInterceptor::PI::_send_request_ip
+		(_cri, CORBA::ORB::get_msgid(id), _args, this->ctx_list(),
+		 this->ctx(), this->context());
+//  	    msgid = orb->invoke_async (_obj, this, Principal::_nil(),
+//  				       TRUE, 0, msgid);
+//  	    break;
+//  	  _obj->_ior_fwd()->addressing_disposition (ad);
+	    id = orb->invoke_async (_obj, this, Principal::_nil(),
+				    TRUE, 0, id);
+	    break;
 
 	case CORBA::InvokeOk:
+	    // receive reply
+	    CORBA::TypeCode_ptr tc;
+	    if (_res && (tc = _res->typecode()) != NULL
+		&& tc->kind() != CORBA::tk_void
+		&& tc->kind() != CORBA::tk_null) {
+		    r.from_static_any (*_res);
+		    PInterceptor::PI::_receive_reply_ip
+			(_cri, r, _args, dummy->context(), TRUE);
+	    }
+	    else {
+		PInterceptor::PI::_receive_reply_ip
+		    (_cri, r, _args, dummy->context(), FALSE);
+	    }
+	    done = TRUE;
+	    break;
+
 	case CORBA::InvokeUsrEx:
+	    // receive_exception (user exception)
+	    PInterceptor::PI::_receive_exception_ip
+		(_cri, PortableInterceptor::USER_EXCEPTION,
+		 this->exception()->_clone(), dummy->context()); // end point
 	    done = TRUE;
 	    break;
 
 	case CORBA::InvokeSysEx:
+	    // receive_exception (system exception)
+	    PInterceptor::PI::_receive_exception_ip
+		(_cri, PortableInterceptor::SYSTEM_EXCEPTION,
+		 this->exception()->_clone(), dummy->context());
 	    if (_obj->_is_forwarded()) {
 		/*
 		 * [15-44] says:
@@ -2217,8 +2262,17 @@ CORBA::StaticRequest::invoke ()
                      CORBA::OBJECT_NOT_EXIST::_downcast (sysex))) {
                     _obj->_unforward();
                     env()->clear();
-                    msgid = orb->invoke_async (_obj, this,
-                                               Principal::_nil());
+		    CORBA::release(_cri);
+//  		    msgid = orb->new_msgid();
+		    id = orb->new_orbid();
+		    _cri = PInterceptor::PI::_create_cri(_obj, _opname);
+		    PInterceptor::PI::_send_request_ip
+			(_cri, CORBA::ORB::get_msgid(id), _args,
+			 this->ctx_list(), this->ctx(), this->context());
+//                      msgid = orb->invoke_async
+//  			(_obj, this, Principal::_nil(), TRUE, 0, msgid);
+                    id = orb->invoke_async (_obj, this, Principal::_nil(),
+					    TRUE, 0, id);
                     break;
 		}
 	    }
@@ -2245,8 +2299,26 @@ CORBA::StaticRequest::oneway ()
 				  env())) {
 	return;
     }
-    _obj->_orbnc()->invoke_async (_obj, this, CORBA::Principal::_nil(), FALSE);
+//      _obj->_orbnc()->invoke_async (_obj, this, CORBA::Principal::_nil(), FALSE);
 
+    PInterceptor::PI::_send_request_ip
+	(_cri, 0, _args, this->ctx_list(), this->ctx(), this->context(), FALSE);
+
+//      CORBA::ULong msgid = _obj->_orbnc()->invoke_async
+//  	(_obj, this, CORBA::Principal::_nil(), FALSE);
+    CORBA::ORBMsgId id = _obj->_orbnc()->invoke_async
+	(_obj, this, CORBA::Principal::_nil(), FALSE);
+//      if (msgid > 0) { // object not exists exc.
+    if (id != NULL) {
+	cerr << "static.cc hack pi...." << endl;
+	CORBA::OBJECT_NOT_EXIST ex;
+	PInterceptor::PI::_receive_exception_ip
+	    (_cri, PortableInterceptor::SYSTEM_EXCEPTION, ex._clone(),
+	     this->ctx_list(), this->ctx(), this->context());
+    }
+    else {
+	PInterceptor::PI::_receive_other_ip(_cri);
+    }
     if (_iceptreq) {
 	Interceptor::ClientInterceptor::
 	    _exec_after_marshal ((Interceptor::LWRequest_ptr)_iceptreq,
@@ -2262,17 +2334,26 @@ CORBA::StaticRequest::send_deferred ()
     if (_iceptreq && !Interceptor::ClientInterceptor::
 	_exec_initialize_request ((Interceptor::LWRequest_ptr)_iceptreq,
 				  env())) {
-        _msgid = 0;
+        _id = 0;
 	return;
     }
-
-    _msgid = orb->invoke_async (_obj, this, CORBA::Principal::_nil());
+    
+//      CORBA::ULong msgid = orb->new_msgid();
+    CORBA::ORBMsgId id = orb->new_orbid();
+    PInterceptor::PI::_send_request_ip
+	(_cri, CORBA::ORB::get_msgid(id), _args, this->ctx_list(), this->ctx(),
+	 this->context());
+//      _msgid = orb->invoke_async (_obj, this, CORBA::Principal::_nil(),
+//  				TRUE, 0, msgid);
+    _id = orb->invoke_async (_obj, this, CORBA::Principal::_nil(),
+			     TRUE, 0, id);
+    PInterceptor::PI::_receive_other_ip(_cri);
 
     if (_iceptreq && !Interceptor::ClientInterceptor::
 	_exec_after_marshal ((Interceptor::LWRequest_ptr)_iceptreq,
 			     env())) {
-	orb->cancel (_msgid);
-	_msgid = 0;
+	orb->cancel (_id);
+	_id = 0;
     }
 }
 
@@ -2285,42 +2366,105 @@ CORBA::StaticRequest::get_response ()
     CORBA::Boolean done = FALSE;
     GIOP::AddressingDisposition ad;
 
-    assert (_msgid);
+    assert (_id);
+//      assert (_msgid);
+
+    PInterceptor::PI::_send_poll_ip(_cri, CORBA::ORB::get_msgid(_id));
 
     while (!done) {
-	CORBA::Boolean ret = orb->wait (_msgid);
+	CORBA::Boolean ret = orb->wait (_id);
 	assert (ret);
 
 	// XXX called multiple times in case of Forward
 	if (_iceptreq && !Interceptor::ClientInterceptor::
 	    _exec_before_unmarshal ((Interceptor::LWRequest_ptr)_iceptreq,
 				    env())) {
-	    orb->cancel (_msgid);
-            _msgid = 0;
+	    orb->cancel (_id);
+            _id = 0;
 	    return;
 	}
 
-	CORBA::InvokeStatus rs = orb->get_invoke_reply (_msgid, obj,
-							dummy, ad);
+	CORBA::InvokeStatus rs = orb->get_invoke_reply (_id, obj, dummy, ad);
+//  	CORBA::InvokeStatus rs = orb->get_invoke_reply (_msgid, obj,
+//  							dummy, ad);
+	CORBA::Any r;
+
 	switch (rs) {
 	case CORBA::InvokeForward:
 	    // XXX what if _obj is not a stub ???
 	    assert (_obj.in() && obj.in());
 	    _obj->_forward (obj);
-	    _msgid = orb->invoke_async (obj, this, Principal::_nil());
+	    PInterceptor::PI::_receive_other_ip
+		(_cri, PortableInterceptor::LOCATION_FORWARD, _obj,
+		 this->ctx_list(), this->ctx(), dummy->context());
+	    CORBA::release(_cri);
+//  	    _msgid = orb->new_msgid();
+	    if (_id != NULL)
+		delete _id;
+	    _id = orb->new_orbid();
+	    _cri = PInterceptor::PI::_create_cri(_obj, _opname);
+	    PInterceptor::PI::_send_request_ip
+		(_cri, CORBA::ORB::get_msgid(_id), _args, this->ctx_list(),
+		 this->ctx(), this->context());
+//  	    _msgid = orb->invoke_async (obj, this, Principal::_nil(),
+//  					TRUE, 0, _msgid);
+	    _id = orb->invoke_async (obj, this, Principal::_nil(),
+				     TRUE, 0, _id);
 	    break;
 
 	case CORBA::InvokeAddrDisp:
-	  _obj->_ior_fwd()->addressing_disposition (ad);
-	  _msgid = orb->invoke_async (_obj, this, Principal::_nil());
-	  break;
+	    _obj->_ior_fwd()->addressing_disposition (ad);
+	    PInterceptor::PI::_receive_other_ip
+		(_cri, PortableInterceptor::TRANSPORT_RETRY, this->ctx_list(),
+		 this->ctx(), dummy->context());
+	    CORBA::release(_cri);
+//  	    _msgid = orb->new_msgid();
+	    if (_id != NULL)
+		delete _id;
+	    _id = orb->new_orbid();
+	    _cri = PInterceptor::PI::_create_cri(_obj, _opname);
+	    PInterceptor::PI::_send_request_ip
+		(_cri, CORBA::ORB::get_msgid(_id), _args, this->ctx_list(),
+		 this->ctx(), this->context());
+//  	    _msgid = orb->invoke_async (_obj, this, Principal::_nil(),
+//  					TRUE, 0, _msgid);
+//  	    break;
+//  	  _obj->_ior_fwd()->addressing_disposition (ad);
+	    _id = orb->invoke_async (_obj, this, Principal::_nil(),
+				     TRUE, 0, _id);
+	    break;
 
 	case CORBA::InvokeOk:
+	    CORBA::TypeCode_ptr tc;
+	    if (_res && (tc = _res->typecode()) != NULL
+		&& tc->kind() != CORBA::tk_void
+		&& tc->kind() != CORBA::tk_null) {
+		r.from_static_any (*_res);
+		PInterceptor::PI::_receive_reply_ip
+		    (_cri, r, _args, this->ctx_list(), this->ctx(),
+		     dummy->context(), TRUE);
+	    }
+	    else {
+		PInterceptor::PI::_receive_reply_ip
+		    (_cri, r, _args, this->ctx_list(), this->ctx(),
+		     dummy->context(), FALSE);
+	    }
+	    done = TRUE;
+	    break;
+
 	case CORBA::InvokeUsrEx:
+	    PInterceptor::PI::_receive_exception_ip
+		(_cri, PortableInterceptor::USER_EXCEPTION,
+		 this->exception()->_clone(), this->ctx_list(),
+		 this->ctx(), dummy->context());
 	    done = TRUE;
 	    break;
 
 	case CORBA::InvokeSysEx:
+	    PInterceptor::PI::_receive_exception_ip
+		(_cri, PortableInterceptor::SYSTEM_EXCEPTION,
+		 this->exception()->_clone(), this->ctx_list(),
+		 this->ctx(), dummy->context());
 	    if (_obj->_is_forwarded()) {
 		/*
 		 * [15-44] says:
@@ -2343,8 +2487,20 @@ CORBA::StaticRequest::get_response ()
                      CORBA::OBJECT_NOT_EXIST::_downcast (sysex))) {
                     _obj->_unforward();
                     env()->clear();
-                    _msgid = orb->invoke_async (_obj, this,
-                                               Principal::_nil());
+		    CORBA::release(_cri);
+//  		    _msgid = orb->new_msgid();
+		    if (_id != NULL)
+			delete _id;
+		    _id = orb->new_orbid();
+		    _cri = PInterceptor::PI::_create_cri(_obj, _opname);
+		    PInterceptor::PI::_send_request_ip
+			(_cri, CORBA::ORB::get_msgid(_id), _args, this->ctx_list(),
+			 this->ctx(), this->context());
+//  		    _msgid = orb->invoke_async (_obj, this, Principal::_nil(),
+//  						TRUE, 0, _msgid);
+//  		    break;
+                    _id = orb->invoke_async (_obj, this, Principal::_nil(),
+					     TRUE, 0, _id);
                     break;
 		}
 	    }
@@ -2361,7 +2517,8 @@ CORBA::StaticRequest::get_response ()
 	    _exec_finish_request ((Interceptor::LWRequest_ptr)_iceptreq,
                                   env());
     }
-    _msgid = 0;
+    // kcg: Is this ok instead of delete _id; _id = 0; ??
+    _id = 0;
 }
 
 
@@ -2379,11 +2536,11 @@ CORBA::StaticImplementation::~StaticImplementation ()
 CORBA::ServerRequestBase_ptr
 CORBA::StaticImplementation::make_request (ORBRequest *req,
 					   CORBA::Object_ptr obj,
-					   CORBA::ULong msgid,
+					   CORBA::ORBMsgId id,
 					   CORBA::ObjectAdapter *oa,
 					   CORBA::Principal_ptr pr)
 {
-    return new StaticServerRequest (req, obj, msgid, oa, pr);
+    return new StaticServerRequest (req, obj, id, oa, pr);
 }
 
 void
@@ -2398,16 +2555,16 @@ CORBA::StaticImplementation::doinvoke (ServerRequestBase_ptr req,
 
 
 CORBA::StaticServerRequest::StaticServerRequest (ORBRequest *req,
-						 Object_ptr obj, MsgId msgid,
+						 Object_ptr obj, CORBA::ORBMsgId id,
 						 CORBA::ObjectAdapter *oa,
 						 CORBA::Principal_ptr pr)
     : _oa (oa), _obj (Object::_duplicate (obj)),
       _ctx (Context::_nil()), _env (pr),
       _req (ORBRequest::_duplicate (req)),
-      _res (0), _msgid (msgid), _aborted (FALSE)
+      _res (0), _id (id), _aborted (FALSE)
 {
-    _iceptreq = Interceptor::ServerInterceptor::_create_request (
-	_obj, req->op_name(), *req->context(), CORBA::ServerRequest::_nil());
+    _iceptreq = Interceptor::ServerInterceptor::_create_request
+	(_obj, req->op_name(), *req->context(), CORBA::ServerRequest::_nil());
 }
 
 CORBA::StaticServerRequest::~StaticServerRequest ()
@@ -2422,7 +2579,14 @@ CORBA::StaticServerRequest::~StaticServerRequest ()
                 stat = CORBA::InvokeUsrEx;
             }
         }
-        _oa->answer_invoke (_msgid, _obj, _req, stat);
+	// change aruments after servant is done        
+	CORBA::ORBInvokeRec* rec = _id;
+	if (rec) {
+	    PInterceptor::PI::_set_sri_exception
+		(rec->requestinfo(), _env.exception());
+	}
+//          _oa->answer_invoke (_msgid, _obj, _req, stat);
+        _oa->answer_invoke (_id, _obj, _req, stat);
 
         if (_iceptreq && !_aborted && !Interceptor::ServerInterceptor::
             _exec_finish_request ((Interceptor::LWServerRequest_ptr)_iceptreq,
@@ -2476,6 +2640,7 @@ CORBA::StaticServerRequest::read_args ()
 
     if (!_req->get_in_args (&_args, _ctx)) {
         if (MICO::Logger::IsLogged (MICO::Logger::Error)) {
+	  MICOMT::AutoDebugLock lock;
 	  MICO::Logger::Stream (MICO::Logger::Error)
             << "Error: cannot decode args in StaticServerRequest" << endl;
 	}
@@ -2492,6 +2657,36 @@ CORBA::StaticServerRequest::read_args ()
 	write_results ();
 	return FALSE;
     }
+    
+    CORBA::ORB_ptr orb = _obj->_orbnc();
+    CORBA::ORBInvokeRec* rec = _id;
+    if (!rec)
+    	return TRUE;
+    PInterceptor::ServerRequestInfo_impl* sri = rec->requestinfo();
+    // point of receive_request call
+    try {
+	PInterceptor::PI::_receive_request_ip(sri, _args, _ctx, _oa, _obj);
+    } catch (CORBA::SystemException_catch& ex) {
+	set_exception(ex->_clone());
+	_aborted = TRUE;
+	write_results();
+	return FALSE;
+    } catch (PortableInterceptor::ForwardRequest_catch& exc) {
+	_obj = Object::_duplicate(exc->forward);
+	orb->answer_invoke(rec, CORBA::InvokeForward, exc->forward,
+			   _req, 0);
+    } catch (CORBA::UserException& ex) {
+	set_exception(ex._clone());
+	_aborted = TRUE;
+	write_results();
+	return FALSE;
+    } catch (...) {
+	CORBA::UNKNOWN ex;
+	set_exception(ex._clone());
+	_aborted = TRUE;
+	write_results();
+	return FALSE;
+    }
     return TRUE;
 }
 
@@ -2514,14 +2709,30 @@ CORBA::StaticServerRequest::write_results ()
 
     if (_env.exception()) {
 	_req->set_out_args (_env.exception());
-    } else {
+    }
+    else {
+	CORBA::ORBInvokeRec* rec = _id;
+	if (rec) {
+	    PInterceptor::ServerRequestInfo_impl* sri = rec->requestinfo();
+	    CORBA::Any r;
+	    CORBA::TypeCode_ptr tc;
+	    if (_res && (tc = _res->typecode())) {
+		if (tc->kind() != CORBA::tk_void
+		    && tc->kind() != CORBA::tk_null) {
+		    r.from_static_any (*_res);
+		    PInterceptor::PI::_set_sri_result(sri, r);
+		}
+	    }
+	    PInterceptor::PI::_set_sri_args(sri, _args, TRUE);
+	}
         if (!_req->set_out_args (_res, &_args)) {
-	  if (MICO::Logger::IsLogged (MICO::Logger::Error)) {
-	    MICO::Logger::Stream (MICO::Logger::Error)
-	      << "Error: cannot set out args in StaticServerRequest" << endl;
-	  }
-	  CORBA::MARSHAL ex (0, CORBA::COMPLETED_YES);
-	  _req->set_out_args (&ex);
+	    if (MICO::Logger::IsLogged (MICO::Logger::Error)) {
+		MICO::Logger::Stream (MICO::Logger::Error)
+		    << "Error: cannot set out args in StaticServerRequest"
+		    << endl;
+	    }
+	    CORBA::MARSHAL ex (0, CORBA::COMPLETED_YES);
+	    _req->set_out_args (&ex);
         }
     }
 }
