@@ -23,11 +23,14 @@
  *	97/06/27 0.1    mueller         CREATED
  */
 
+#ifdef __COMO__
+#pragma no_pch
+#endif // __COMO__
 #include "uni_base64.h"
 #include "uni_errors.h"
 #include "uni_support.h"
 
-#ifdef _WINDOWS
+#ifdef _WIN32
 #pragma warning (disable: 4305 4309 4244 4018)
 // pragma warning (default: 4305 ) // turn on
 #endif
@@ -44,6 +47,7 @@ C_EXPORT uni_slong uni_base64encode(
   int           first;
 
   readCount = writeCount = 0;
+  carry = 0;
   carrybits = 0;
 
   while(readCount < datasize)
@@ -187,7 +191,7 @@ C_EXPORT short uni_base64decode(register void *data, short carry,
   equalSign = false;
   *bytes_written = 0;
 
-  while(character = (*src))
+  while((character = (*src)))
   {
     if( (character > 64) && (character < 91) )
       character -= 65;
