@@ -23,6 +23,9 @@
  *	97/07/01 0.1    mueller         CREATED
  */
 
+#ifdef __COMO__
+#pragma no_pch
+#endif // __COMO__
 #include <ctype.h>
 #include <stdio.h>
 #include <string.h>
@@ -33,7 +36,7 @@
 #include "uni_errors.h"
 #include "uni_conversion.h"
 
-#ifdef _WINDOWS
+#ifdef _WIN32
 #pragma warning (disable: 4305 4309 4244 4018)
 // pragma warning (default: 4305 ) // turn on
 #endif
@@ -875,7 +878,7 @@ uni_slong uni_toUTF8  (char *utf8, const char *src, uni_uword chars,
 
   const ctounitable *table1, *table2, *table3;
   uni_ushort  *utf16;
-  uni_ulong   *ucs4, c;
+  uni_ulong   *ucs4, c = 0;
 
 
   if(string_type == C_UTF8)
