@@ -20,6 +20,15 @@
  *  http://www.mico.org/
  */
 
+#ifdef FAST_PCH
+#include "orb_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #define MICO_CONF_IMR
 
 #include <CORBA-SMALL.h>
@@ -27,9 +36,16 @@
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
+#include <algorithm>
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <iostream>
+#include <fstream>
+#include <sstream>
+#else
 #include <iostream.h>
 #include <fstream.h>
 #include <strstream.h>
+#endif
 #include <mico/impl.h>
 #include <mico/template_impl.h>
 #include <mico/util.h>
@@ -39,6 +55,11 @@
 #include <mico/memtrace.h>
 #endif
 
+#endif // FAST_PCH
+
+
+using namespace std;
+
 #ifdef _POCKET_PC
 extern char* getenv (const char*);
 #endif
@@ -50,6 +71,9 @@ extern char* getenv (const char*);
 void
 mico_assert (const char *file, int line)
 {
+#ifdef HAVE_THREADS
+    cout << "[" << ((MICO_ULong) OSMisc::timestamp()) << " - " << MICOMT::Thread::self() << "] ";
+#endif
     cout << file << ":" << line << ": assertion failed" << endl;
 #ifdef USE_MEMTRACE
     MemTrace_SelfTrace (stderr);
@@ -95,76 +119,97 @@ xstrerror (int err)
 string
 xdec (int i)
 {
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+    ostringstream ostr;
+    ostr << i;
+    return ostr.str();
+#else
     ostrstream ostr;
     ostr << i << ends;
-
     string s = ostr.str();
     ostr.rdbuf()->freeze (0);
     return s;
+#endif
 }
 
 string
 xdec (long i)
 {
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+    ostringstream ostr;
+    ostr << i;
+    return ostr.str();
+#else
     ostrstream ostr;
     ostr << i << ends;
-
     string s = ostr.str();
     ostr.rdbuf()->freeze (0);
     return s;
+#endif
 }
 
 string
 xdec (OSMisc::longlong i)
 {
-    ostrstream ostr;
-#if defined(_WINDOWS) && !defined(__MINGW32__)
-    ostr << long(i) << ends;   //XXX
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+    ostringstream ostr;
+    ostr << i;
+    return ostr.str();
 #else
+    ostrstream ostr;
     ostr << i << ends;
-#endif
-
     string s = ostr.str();
     ostr.rdbuf()->freeze (0);
     return s;
+#endif
 }
 
 string
 xdec (unsigned int i)
 {
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+    ostringstream ostr;
+    ostr << i;
+    return ostr.str();
+#else
     ostrstream ostr;
     ostr << i << ends;
-
     string s = ostr.str();
     ostr.rdbuf()->freeze (0);
     return s;
+#endif
 }
 
 string
 xdec (unsigned long i)
 {
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+    ostringstream ostr;
+    ostr << i;
+    return ostr.str();
+#else
     ostrstream ostr;
     ostr << i << ends;
-
     string s = ostr.str();
     ostr.rdbuf()->freeze (0);
     return s;
+#endif
 }
 
 string
 xdec (OSMisc::ulonglong i)
 {
-    ostrstream ostr;
-
-#if defined(_WINDOWS) && !defined(__MINGW32__) && !defined(__BORLANDC__)
-    ostr << unsigned long(i) << ends; //XXX
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+    ostringstream ostr;
+    ostr << i;
+    return ostr.str();
 #else
+    ostrstream ostr;
     ostr << i << ends;
-#endif
-
     string s = ostr.str();
     ostr.rdbuf()->freeze (0);
     return s;
+#endif
 }
 
 size_t
@@ -268,8 +313,8 @@ mico_url_decode (const char * ptr, CORBA::ULong & len)
 
   while (*ptr) {
     if (*ptr == '%') {
-      if ((ptr[1] < '0' || ptr[1] > '9') && (ptr[1] < 'a' || ptr[1] > 'f') ||
-	  (ptr[2] < '0' || ptr[2] > '9') && (ptr[2] < 'a' || ptr[2] > 'f')) {
+      if (!isxdigit((unsigned char)ptr[1]) ||
+	  !isxdigit((unsigned char)ptr[2])) {
 	CORBA::string_free ((char *) str);
 	return NULL;
       }
@@ -340,7 +385,7 @@ CORBA::MethodDispatcher::invoke (CORBA::ServerRequest_ptr _req,
 	    return;
     }
 #ifdef _POCKET_PC
-    NOT_IMPLEMENTED;
+    MICO_NOT_IMPLEMENTED;
 #else
 	_req->exception (new BAD_OPERATION);
 #endif
@@ -389,14 +434,13 @@ ostream ** MICO::Logger::_out = 0;
 MICO::Logger::Logger ()
 {
   assert (!_instance);
-  _instance = this;
 
   _out = new ostream * [All];
 
   for (int i=0; i<All; i++) {
     _out[i] = 0;
   }
-
+  _instance = this;
   Log (Error);
 }
 
@@ -468,6 +512,27 @@ MICO::Logger::Log (const char * msg, CORBA::Boolean onoff,
   else if (strcmp (msg, "Transport") == 0) {
     Log (Transport, onoff, file);
   }
+  else if (strcmp (msg, "Thread") == 0) {
+    Log (Thread, onoff, file);
+  }
+  else if (strcmp (msg, "POA") == 0) {
+    Log (POA, onoff, file);
+  }
+  else if (strcmp (msg, "ORB") == 0) {
+    Log (ORB, onoff, file);
+  }
+  else if (strcmp (msg, "Support") == 0) {
+    Log (Support, onoff, file);
+  }
+  else if (strcmp (msg, "PI") == 0) {
+    Log (PI, onoff, file);
+  }
+  else if (strcmp (msg, "Security") == 0) {
+    Log (Security, onoff, file);
+  }
+  else if (strcmp (msg, "Exception") == 0) {
+    Log (Exception, onoff, file);
+  }
   else if (strcmp (msg, "All") == 0) {
     Log (All, onoff, file);
   }
@@ -479,24 +544,6 @@ MICO::Logger::Log (const char * msg, CORBA::Boolean onoff,
   }
 }
 
-CORBA::Boolean
-MICO::Logger::IsLogged (MessageType msg)
-{
-  if (!_instance) {
-    return 0;
-  }
-  return (_out[msg] != 0);
-}
-
-ostream &
-MICO::Logger::Stream (MessageType msg)
-{
-  assert (_instance);
-  assert (_out[msg]);
-  return *_out[msg];
-}
-
-
 /***************************** MICOGetOpt ******************************/
 
 
@@ -646,6 +693,127 @@ MICOGetOpt::opts () const
 }
 
 
+#ifdef USE_CSIV2
+
+vector<unsigned int>
+ASN1::Codec::parse_oid(const char* __oid)
+{
+    string __s_tmp = __oid;
+    string __str;
+    if (__s_tmp.find(':') > 0)
+	__str = __s_tmp.substr(__s_tmp.find(':') + 1);
+    else
+	__str = __s_tmp;
+    vector<unsigned int> __buf;
+    string __s1;
+    int __ix = 0;
+    for (;;) {
+	__ix = __str.find('.');
+	if (__ix > 0) {
+	    __s1 = __str.substr(0, __ix);
+	    __str = __str.substr(__ix + 1, __str.length());
+	    __buf.push_back(atoi(__s1.c_str()));
+	}
+	else {
+	    __buf.push_back(atoi(__str.c_str()));
+	    break;
+	}
+    }
+    return __buf;
+}
+
+char*
+ASN1::Codec::decode_oid(CORBA::OctetSeq& __oid)
+{
+    assert(__oid[0] == 6); // OID Tag = 6
+    int __len = __oid[1] + 2;
+    vector<unsigned int> __buf;
+    vector<unsigned int> __big_num;
+    string __res;
+    int __first = __oid[2] / 40; // first number can be 0/1/2
+    int __second = __oid[2] - __first * 40;
+    int __i;
+    for (__i=3; __i<__len; __i++) {
+	if (__oid[__i] < 127) {
+	    if (__big_num.size () == 0)
+		__buf.push_back(__oid[__i]);
+	    else {
+		unsigned int __sum = 0;
+		for (CORBA::ULong __j=0; __j<__big_num.size(); __j++)
+		    __sum += __big_num[__j];
+		__sum += __oid[__i];
+		__big_num.erase(__big_num.begin(), __big_num.end());
+		__buf.push_back(__sum);
+	    }
+	}
+	else {
+	    __big_num.push_back(__oid[__i] % 128);
+	    for (CORBA::ULong __j=0; __j<__big_num.size(); __j++) {
+		unsigned int __tmp = __big_num[__j];
+		__big_num[__j] = __tmp * 128;
+	    }
+	}
+    }
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+    stringstream __sstr;
+#else // HAVE_ANSI_CPLUSPLUS_HEADERS
+    ostrstream __sstr;
+#endif // HAVE_ANSI_CPLUSPLUS_HEADERS
+    __sstr << "oid:" << __first << "." << __second << ".";
+    int __x = __buf.size();
+    for (__i = 0; __i < __x-1; __i++) {
+	__sstr << __buf[__i] << ".";
+    }
+    __sstr << __buf[__x-1];
+    __res = __sstr.str();
+    return CORBA::string_dup(__res.c_str());
+}
+
+CORBA::OctetSeq*
+ASN1::Codec::encode_oid(const char* __o)
+{
+    vector<unsigned int> __oid = ASN1::Codec::parse_oid(__o);
+    assert(__oid.size() < 127);
+    vector<unsigned char> __buf;
+    __buf.push_back(6); // OID Tag
+    __buf.push_back(0); // temporary length
+    int __oid_len = __oid.size();
+    int __len = 1;
+    __buf.push_back(40 * __oid[0] + __oid[1]);
+    for (int __i=2; __i<__oid_len; __i++) {
+	if (__oid[__i] < 127) {
+	    __buf.push_back(__oid[__i]);
+	    __len++;
+	}
+	else {
+	    int __div = __oid[__i];
+	    int __tmp = 0;
+	    vector<char> __tmp_buf;
+	    while(__div) {
+		__tmp = __div;
+		__div /= 128;
+		if (__tmp_buf.size() == 0)
+		    __tmp_buf.push_back(__tmp % 128);
+		else
+		    __tmp_buf.push_back((__tmp % 128) | 0x80);
+	    }
+	    reverse(__tmp_buf.begin(), __tmp_buf.end());
+	    for (vector<char>::iterator __i=__tmp_buf.begin(); __i!=__tmp_buf.end(); __i++) {
+		__buf.push_back(*__i);
+		__len++;
+	    }
+	}
+    }
+    __buf[1] = __len;
+    CORBA::OctetSeq* __ret = new CORBA::OctetSeq;
+    __ret->length(__buf.size());
+    for (CORBA::ULong __j=0; __j<__buf.size(); __j++)
+	(*__ret)[__j] = __buf[__j];
+    return __ret;
+}
+
+#endif // USE_CSIV2
+
 /************************** repo id matcher ***************************/
 
 
