@@ -1,10 +1,19 @@
 #include <CORBA.h>
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <string>
+#include <iostream>
+#include <fstream>
+#else // HAVE_ANSI_CPLUSPLUS_HEADERS
 #include <string.h>
 #include <iostream.h>
 #include <fstream.h>
+#endif // HAVE_ANSI_CPLUSPLUS_HEADERS
 #include <mico/template_impl.h>
 #include <mico/throw.h>
 
+
+using namespace std;
+
 int
 main (int argc, char *argv[])
 {
