@@ -1,9 +1,18 @@
 #include <CORBA.h>
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <string>
+#include <iostream>
+#include <fstream>
+#else // HAVE_ANSI_CPLUSPLUS_HEADERS
 #include <string.h>
 #include <iostream.h>
 #include <fstream.h>
+#endif // HAVE_ANSI_CPLUSPLUS_HEADERS
 #include <mico/template_impl.h>
 
+
+using namespace std;
+
 class CalcImpl : public CORBA::DynamicImplementation {
 public:
     CalcImpl ();
