@@ -1,8 +1,16 @@
 #include <CORBA.h>
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <string>
+#include <iostream>
+#else // HAVE_ANSI_CPLUSPLUS_HEADERS
 #include <string.h>
 #include <iostream.h>
+#endif // HAVE_ANSI_CPLUSPLUS_HEADERS
 #include <mico/template_impl.h>
 
+
+using namespace std;
+
 class CalcImpl : public CORBA::DynamicImplementation {
 public:
     CalcImpl ();
