@@ -1,5 +1,13 @@
+
 #include "ex.h"
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <iostream>
+#else // HAVE_ANSI_CPLUSPLUS_HEADERS
 #include <iostream.h>
+#endif // HAVE_ANSI_CPLUSPLUS_HEADERS
+
+
+using namespace std;
 
 int main (int argc, char *argv[])
 {
