@@ -1,10 +1,19 @@
+
 #include <CORBA.h>
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <string>
+#include <iostream>
+#else // HAVE_ANSI_CPLUSPLUS_HEADERS
 #include <string.h>
 #include <iostream.h>
+#endif // HAVE_ANSI_CPLUSPLUS_HEADERS
 #include <mico/impl.h>
 #include <mico/util.h>
 #include "types.h"
 
+
+using namespace std;
+
 /*
  * general rule for what is passed to StaticAny constructor,
  * StaticTypeInfo::marshal, StaticTypeInfo::demarshal as the
