@@ -1,7 +1,5 @@
 #define FORCE_MARSHALLING
 
-#include <iostream.h>
-
 // These #includes need to be done manually if
 // MICO_NO_TOPLEVEL_MODULES is defined
 #include <CORBA-SMALL.h>
@@ -9,6 +7,15 @@
 
 #include "module.h"
 
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <iostream>
+#else // HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <iostream.h>
+#endif // HAVE_ANSI_CPLUSPLUS_HEADERS
+
+
+using namespace std;
+
 class foo_impl : virtual public M::foo_skel
 {
 public:
