@@ -1,8 +1,12 @@
 #include <CORBA.h>
+#include "ex.h"
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <iostream>
+#else // HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <iostream.h>
+#endif // HAVE_ANSI_CPLUSPLUS_HEADERS
 #include <unistd.h>
 #include <signal.h>
-#include "ex.h"
-#include <iostream.h>
 
 class Calc_impl : virtual public Calc_skel {
 public:
