@@ -1,17 +1,23 @@
 #define FORCE_MARSHALLING
 
-#include <iostream.h>
 #include <CORBA-SMALL.h>
 #include "union.h"
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <iostream>
+#else // HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <iostream.h>
+#endif // HAVE_ANSI_CPLUSPLUS_HEADERS
 
 
+using namespace std;
+
 class foo_impl : virtual public foo_skel
 {
 public:
   foo_impl()
   {
   };
-  void bar( const U& v1, U*& v2, U2& v3 )
+  void bar( const U& v1, U_out v2, U2_out v3 )
   {
     if( v1._d() == 3 )
       cout << v1.z() << endl;
