@@ -1,10 +1,16 @@
 #define FORCE_MARSHALLING
 
-#include <iostream.h>
 #include <CORBA-SMALL.h>
 #include "context.h"
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <iostream>
+#else // HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <iostream.h>
+#endif // HAVE_ANSI_CPLUSPLUS_HEADERS
 
 
+using namespace std;
+
 class A_impl : virtual public A_skel
 {
 public:
@@ -17,7 +23,7 @@ public:
     CORBA::NVList_var vals;
     ctx->get_values( "", 0, "*", vals );
     for( CORBA::ULong i = 0; i < vals->count(); i++ ) {
-      char *s;
+      const char *s;
       CORBA::Boolean r = ( *vals->item( i )->value() >>= s );
       assert (r);
       cout << vals->item( i )->name() << ": " << s << endl;
