@@ -1,10 +1,16 @@
 #define FORCE_MARSHALLING
 
-#include <iostream.h>
 #include <CORBA-SMALL.h>
 #include "calc.h"
+#ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <iostream>
+#else // HAVE_ANSI_CPLUSPLUS_HEADERS
+#include <iostream.h>
+#endif // HAVE_ANSI_CPLUSPLUS_HEADERS
 
 
+using namespace std;
+
 class Calc_impl : virtual public Calc_skel
 {
 public:
