@@ -33,7 +33,7 @@
 
 #include <string.h>
 #include <iostream.h>
-#ifdef _WINDOWS
+#ifdef _WIN32
 #include <malloc.h>
 #endif
 
